## Architext perl library features list.
## Used by architext.pl to figure out which features are
## enabled.  Unless the binaries actually support the features
## enabled in the scripts, none of the advanced features will
## work

#############################################################
## USER CUSTOMIZABLE FEATURES                              ##
#############################################################

# Customized for HyperNews

$new_query_button = 1;
$graphic_relevance_mode = 1;
$query_by_example_mode = 1;
$summary_mode = 0;
$number_of_summary_sentences = 5;
##$maximum_summary_length = 200;
$subject_group_mode = 1;
$number_of_subject_groups = 6;
$show_additional_docs_in_grouping = 1;
$max_docs_to_return = 20;
$show_legend = 'bottom';
##$show_legend = 'top';
$customize_result_list = 1;

##########################################
# HyperNews globals

# Relative URL that is the prefix of all HyperNews pages.
# E.g. $HNgeturl = '/HyperNews/get';
$HNgeturl = '/HyperNews/get';

# How to get from the server document root to the hypernews document root.
# For example, if: doc root is /home/htdocs and hypernews doc root ($hndocs) is
# /home/htdocs/hn then do: $HNhtroot_to_hnroot = '/hn';
$HNhtroot_to_hnroot = '';


#############################################################
##  USER CUSTOMIZABLE FUNCTIONS                            ##
#############################################################
## this function can be used to modify the results lines
## as they appear in the hit-list as presented by the
## search scripts.  This function will not be called unless
## the variable '$customize_result_list', defined above,
## is given a non-zero value.
sub customize_result_list_line {
    local($collection_name, $file, $doc_root, $relevance_qbe, $score,
	  $title, $summary, $original_line) = @_;

    # Compute full path to directory where your HyperNews documents reside.
    $HNroot = "$doc_root$HNhtroot_to_hnroot";  

    local ($urcfile);
    if ($file =~ m/,urc$/) {
      $urcfile = $file;
    }
    elsif ($file =~ m,(-body|-content)?\.html$,) {
      $urcfile = $file;
      $urcfile =~ s,(-body|-content)?\.html$,\.html\,urc,; 
    }

    # $file might be a normal file instead of one of HyperNews'
    if (!$urcfile) {
      return $original_line;
    }

    local ($realfile) = $urcfile;
    $realfile =~ s/\.html\,urc/\-body\.html/; 
    local ($realurl) = ($realfile);
    $realurl =~ s/$HNroot/$HNgeturl/;
    $realurl =~ s/\-body\.html/.html/;

    return '' if (!-e $urcfile);
    local (%urc) = &parseURCFile($urcfile);

    local ($realtitle) = $urc{'Title'} || $title;
    local ($author) = $urc{'From'};  # We could fix this with normalizeUser
    local ($date) = $urc{'Date'};  # We could abbreviate this
    
    $realsummary = ($summary eq '(empty)') ? "($author) - $date" : $summary;

    local ($realline) = qq($relevance_qbe $score 
	   <A HREF="$realurl"><b>$realtitle</b></A> $realsummary<br>);
    
    # for debugging
#    $realline .= "<pre>" . 
#       &escapeHTML("\n\n$relevance_qbe, $score, 
#$file, '$title', '$summary'; $doc_root $HNroot;
#$urcfile, $realfile, $realurl, $realtitle") . 
#       "</pre><p>";

#    $realline =. "<pre>" . &escapeHTML("\n\n$file
#$title
#$summary
#$doc_root
#$HNroot
#$title
#$urcfile
#$realfile
#$realurl
#$realtitle") . "</pre><p>";

    # return '' if $title eq 'responses.html'; not needed with filter
    return '' if ($title eq '' || $title eq 'index.html');
    return '' if ($title eq '(Document not available)');
    # <No title or could not open files>

    return "$realline";
}

# From urc.pl
sub parseURCFile {
    local($urcFile) = @_;
    # Returns values in associative array %in.
    # e.g. %urc = &parseURCFile("filename");
    $urc{'Title'} = "&lt;No title or could not open files&gt;";
    open (URCH, "<$urcFile");
    while (<URCH>) {
        if ($_ =~ "^([^:]*): (.*)") { $urc{$1} = $2; }
    }
    close URCH;
    return %urc;
}          

sub escapeHTML {
    local($string) = @_;

    $string =~ s/\&/\&amp;/g ;
    $string =~ s/</\&lt;/g ;
    $string =~ s/>/\&gt;/g ;
    $string;
}     


##this function serves the same purpose as 'customize_result_list_line',
##but will instead modify the line presented after the user does
##a subject grouping.  There is no reason these functions cannot
##do the same thing, but since the html for the results after
## a query and a subject grouping are slightly different, two
##separate functions are provided.
sub customize_grouping_line {
    local($collection_name, $file, $doc_root, $relevance_qbe,  
	  $title, $summary, $original_line) = @_;
    return &customize_result_list_line
      ($collection_name, $file, $doc_root, $relevance_qbe, $score,
       $title, $summary, $original_line);
}




################################################################
## END OF CUSTOMIZABLE SECTION                                ##
################################################################

## DO NOT change any of the values below, or you could
## cause problems with the functionality of this product

## notifier variables
$notify_icon = "AT-pinkball.gif";
##$notifier_on = 1;
## end of notifier variables

$restrict_beneath_document_root = 1;
$custom_format = 0;
$distributed = 0;
$debugmode = 0;
$advise_on_gather = 1;
$max_docs_per_subgroup = 10;
$gather_options = "m=iavg trunc=10 fid=50 gv=50 i=2";

$productVersion = "1.0P1";
$help_path = "http://www.atext.com/EWS1.0P1/";
$remoteScriptName = 
    "http://www.atext.com/cgi/ews-start.cgi";
$news_url = "http://www.atext.com/EWS1.0P1/News.html";

$warning_icon = "AT-redball.gif";
$error_icon = "AT-stopsign.gif";
$confidence_image = "AT-grouped_confidence.gif";
$subject_image = "AT-grouped_subject.gif";
$admin_banner = "AT-admin_banner.gif";
$query_banner = "AT-search_banner.gif";
$netsearch_button = "AT-netsearch_link.gif";
$result_banner = "AT-search_banner.gif";
$results_by_graphic = "AT-results_label.gif";

$powered_by_excite = 1;

## filenames for different relevance balls and QBE and Summary icons
## old UI graphics
%balls = (
	  '0', 'AT-blueball.gif',
	  '1', 'AT-3stars.gif',
	  '2', 'AT-2stars.gif',
	  '3', 'AT-1stars.gif',
	  '4', 'AT-1stars.gif',
	  );			

## ECO new UI graphics
%icons = (
	  '0', 'AT-blueball.gif',
	  '1', 'AT-red_x.gif',
	  '2', 'AT-black_x.gif',
	  '3', 'AT-black_x.gif',
	  '4', 'AT-black_x.gif',
	  );			

1;
