#!/usr/bin/perl -T
# add-article.pl. - add or update article edited with edit-article.pl.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

# Add current directory and .scripts to load path.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};
$0 =~ m{^(.*)/[^/]+$ }x;
$hnbin = $1 || '.';
unshift(@INC, $hnbin, "$hnbin/.scripts"); 


# Whether command line calls of add-article.pl require authentication
# and access control, typically admin only.
$RESTRICT_COMMAND_LINE = 0;

require "Admin/hnrc";

%main::form = ();


&feval('&addArticlePL()');


sub addArticlePL {
  require "formlib.pl";
  require "urc.pl";
  require "add-lib.pl";
  
  local($debug) = 0;
  &HNMsg("Add Article") if $debug;
  

  if ($ENV{REQUEST_METHOD}) {
      # Called via http
      %form = &GetFormArgs();
  }
  else {
      # Called via shell command line or system.
      $command_line = 1;
      $plaintext = 1;

      # Could do:
      # use lib "$hnbin/.scripts/lib";
      # use autouse Getopt::Declare qw{new};

      push (@INC, "$hnbin/.scripts/lib");
      require Getopt::Declare; import Getopt::Declare;
      #  [required] doesnt work with [ditto]

      # Command line args:
      my $optctl = q{
        --forum <directory>	Relative to hndocs.
	{ $::form{responses} = $directory; }
        --dir <directory>	[ditto]

        --owner <userid>	Owner of the forum.
	{ $::form{UserID} = $userid; }

        --user <remote_user>	The remote user permitted to modify this forum.
	{ $::form{remoteUser} = $remote_user; }

        --password <string>	The password of the remote user.
	{ $::form{adminPassword} = $string; }

        --title <title>		Title string, no HTML.
	{ $::form{title} = $title; }

        --short <short_title>	Short Title.
	{ $::form{shortTitle} = $short_title; }

        --bodytag <body_tag>	Used instead of <BODY> tag.
	{ $::form{bodytag} = $body_tag; }

        --header <text>		Header URL or HTML string.
	{ $::form{headerURL} = $text; }

        --footer <text>		Footer URL or HTML string.
	{ $::form{footerURL} = $text; }

	--check			Check only - do not change anything.
	{ $::check_only = 1; }

 	--debug			Lots of debug output.
        { $::debug = 1; }
};

      $args = new Getopt::Declare($optctl);

      &printAssoc('Form', %form) if $debug;
      print ("Checking only...\n") if $debug && $check_only;
  }

  
  
  $userid = $form{'UserID'};	# Of article owner.
  $password = $form{'Password'};

  $remoteUser = $form{'remoteUser'};
  $adminPassword = $form{'adminPassword'};
  
  $remoteUser ||= $userid;
  
  &checkAuthenticated($userid, $password, 
		      $remoteUser, $adminPassword || $password);

  $title = &escapeHTML($form{'title'});
  $short_title = &escapeHTML($form{'short_title'});

  $responses = &escapeHTML($form{'responses'});
  $responses = &untaintPath($responses);

  $responses =~ s,\.html$,,;    # Remove trailing .html, if any.
  $responses =~ s,/$,$`,;       # Remove trailing /, if any.
  $responses =~ s,^[^/],/$&,;   # Prepend /, if non-/ is there.
  &ErrorIfTainted($responses);
   
  if (!$responses) {
    &HNUserError("You need to specify a directory for the forum.");
  }

  if ($responses =~ m{/[0-9]}) {
      &HNUserError( qq{
Sorry, the forum directory you selected, "$responses", contains a
component that starts with a digit.  That causes problems for
HyperNews, so please change the directory to something else.});
  }
  
  $body = $form{'body'};
  
  $contentType = $form{'contentType'};

  $defaultOutlineDepth = $form{'defaultOutlineDepth'};
  &HNMsg("defaultOutlineDepth: $defaultOutlineDepth\n") if $debug;

  $listAddress = $form{'ListAddress'};

  $NNTPServer = $form{'nntp'};
  $newsgroups = $form{'newsgroups'};
  $frequency = $form{'frequency'};

 #     &HNMsg("nntp2:$NNTPServer\n
 #           frequency:$frequency\n
 #           newsgroups: $newsgroups");

  $headerURL = $form{'headerURL'};
  $footerURL = $form{'footerURL'};
  $bodytag = $form{'bodytag'};  # Should be hidden in forum.

  $moderation = $form{'moderation'};
  $subscribe = ($form{subscribe} eq 'yes') ? 'all' : '';
  $searchURL = $form{'searchURL'};
  
  $upURL = &escapeHTML($form{'upURL'});
  
  $userName = &escapeHTML($form{'userName'});
  $userURL = &escapeHTML($form{'userURL'});
  
  if ($responses && 
      (-e "$hndocs$responses.html,urc") && ($addArticleSF > 1)) {
    %urc = &parseURCFile("$hndocs$responses.html,urc");
    
    $originalOwner = $urc{From};
    $base = $urc{BaseURL};

    &HNMsg("Article URL:$hndocs$responses.html,urc
             Owner: $originalOwner\n") if $debug;
    
    if (!$command_line) {
	&access1($addArticleSF, 2, $originalOwner, $remoteUser,
             "Article already exists, owned by 
<strong> $originalOwner </strong>.
<strong> $remoteUser </strong> can't overwrite!",
             $authenticated, $base);
    }
  }
  else {
    # Check parent article.  May not work yet.
    if (!$command_line) {
	&access1($addArticleSF, 2, $userid, $remoteUser, 
	     "User <strong> $remoteUser </strong> 
is not permitted to add article.<p>", 
	     $authenticated);
    }
  }
  
  if (!$userName) {
    local (%member) = &getMember($userid);
    $userid = $member{'UserID'} || $userid;
    $name = $member{'Name'};
    $userURL = $member{'UserURL'};
  }
  
  # &ErrorIfTainted($responses); # not tainted here

    print ("Checking only ... done!\n") if $debug && $check_only;
    return if $check_only;

  # Package up the values.
  my (%article) =
    ('Title' , $title,
     'ShortTitle' , $short_title,
     'Keywords' , $keywords,
     'Description' , $description,
     'ContentType' , $contentType,
     'Body' , $body,
     'HeaderURL' , $headerURL,
     'FooterURL' , $footerURL,
     'BodyTag' , $bodytag,
     'UpURL' , $upURL,
     'Responses' , $responses,
     'ListAddress' , $listAddress,
     'From' , $userid,
     'Moderation', $moderation,
     'Subscribe', $subscribe,
     'SearchURL', $searchURL,
     'Name' , $name,
     'UserURL' , $userURL,
     'DefaultOutlineDepth', $defaultOutlineDepth,
#     'NNTPServer', $NNTPServer
#     'frequency', $frequency,
#     'newsgroups', $newsgroups
    );
  
 
  &AddArticle(0, %article);
  
  print "Location: $base\n";
  
  #&print_HTML_HEAD($title);
  #print "Article: <A HREF=\"$base\">$title</A>.";
  
  print "\n";
  
}


