# add-lib.pl - HyperNews code common to adding base articles and responses.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

require "urc.pl";
require "wwwdates.pl";
require "subscribe-lib.pl";
require "email.pl";
require "hnview.pl";  # Need this to generate mail.
require "lock.pl";
require "response-lib.pl";
require 'log-lib.pl';


sub AddArticle {
  # This is only for base articles.
    local($annotationsMode, %article) = @_;
    local($debug) = 0;

    # Unpackage all the values.
    local $URL = $article{'URL'};

    $title = $article{'Title'};
    $short_title = $article{'ShortTitle'};
    $keywords = $article{'Keywords'};
    $description = $article{'Description'};
    $contentType = $article{'ContentType'};
    $body = $article{'Body'};
    $headerURL = $article{'HeaderURL'};
    $footerURL = $article{'FooterURL'};
    $bodytag = $article{'BodyTag'};
    $upURL = $article{'UpURL'};
    $responses = $article{'Responses'};
    $listAddress = $article{'ListAddress'};
    $userid = $article{'From'};
    $userName = $article{'Name'};
    $userURL = $article{'UserURL'};
    $defaultOutlineDepth = $article{'DefaultOutlineDepth'};
    $moderation = $article{'Moderation'};
    $allMember = $article{AllMember};
    $searchURL = $article{'SearchURL'};

    # &ErrorIfTainted($responses);  # tainted here.
    $responses = &untaintPath($responses);  # Shouldnt have to do this here.

    &HNMsg("add-lib.pl- defaultOutlineDepth: $defaultOutlineDepth") if $debug;

    $previousDate = $article{Date};

    if ($userid) {
	local (%member) = &getMember($userid);
	# Use the real userid, if there is one.
	$userid = $member{'UserID'} || $userid;
	$userName ||= $member{'Name'};
	$userURL ||= $member{'UserURL'};
    }
    &HNMsg ("ARTICLE URL is:$responses\n") if $debug;
    if ($annotationsMode == 1) {
	# $addArticleSF = 1;  Authentication broken for annotations in Mosaic.
    } 

    # The URL of the base document.
    if ($annotationsMode == 1) {
	$base = "$getURL$responses";
    } else {
	$base = "$getURL$responses.html";
    } 

    $date = &wwwdates::wtime(time, 'GMT');

    # Create the directory for responses.
    # May need to create parent directories first.
    $dir = "$hndocs$responses";
    &create_dir($dir);

    #for NNTP interface only
#    $NNTPServer =  $article{'NNTPServer'};
#    $frequency = $article{'frequency'};
#    $newsgroups = $article{'newsgroups'};

#    &HNMsg("nntp:$NNTPServer\n 
#frequency:$frequency\n 
#newsgroups: $newsgroups");

    &create_dir("$dir/.incoming");
    @newsgroupsList = split(/\,/, $newsgroups);
    foreach $newsgroup (@newsgroupsList){
	$newsgroup =~ s/[ \t]+//g;
	next if ($newsgroup eq "");

	&writeURC("$dir/.incoming/$newsgroup,urc",
		      'NNTPServer', $NNTPServer,
		      'frequency', $frequency);

    }



    # Save the base body
    $bodyURL = &saveBody($body, $contentType, "$responses-body.html");


    if ($annotationsMode==1) {
	$urcFileName="$hndocs$responses,urc";
    } else {
	$urcFileName="$hndocs$responses.html,urc";
    }

    # Data might have changed in the meantime, so only overwrite some fields.
    &lock($urcFileName);
    my (%urc) = &parseURCFile($urcFileName);

    $urc{Title} = $title;
    $urc{ShortTitle} = $short_title;
    $urc{Keywords} = $keywords;
    $urc{Description} = $description;
    $urc{Body} = $bodyURL;

    $urc{URL} = $base;
    # UpURL is not really used, but it could be.
    $urc{UpURL} = $upURL;
    $urc{BaseURL} = $base;
    $urc{Responses} = $responses;   # The directory for messages.
    $urc{ListAddress} = $listAddress;
    $urc{From} = $userid;
    # We should instead lookup name and URL - doing that now.
    #'Name' , $userName,
    #'UserURL' , $userURL,

    $urc{Date} ||= $date;  # Creation?
    $urc{LastMod} = $date;
    $urc{AnnotationType} = $annotationType;
    $urc{ContentType} = $contentType;
    $urc{HeaderURL} = $headerURL;
    $urc{FooterURL} = $footerURL;
    $urc{BodyTag} = $bodytag;
    #$urc{Moderation} = $moderation;
    $urc{DefaultOutlineDepth} = $defaultOutlineDepth;

    $urc{Subscribe} = $subscribe;
    $urc{SearchURL} = $searchURL;

    &_writeURC($urcFileName, %urc);
    &create_dir("$hndocs$responses"); # Always create the directory.

    # Remove forum lists, if autogenerated.
    if ($autoGenerateChildForumList) {
	unlink("$hndocs$responses/.child-forums");
    }
    if ($autoGenerateSiblingForumList) {
      # Siblings are the parents children.
	unlink("$hndocs$responses/../.child-forums");
    }

    &unlock($urcFileName);
}

sub AddResponse {
    local ($notifyFlag, $moving, %response) = @_;
    # If $notifyFlag, notify subscribers of $URL.
    # Need to set the following keys in response.
    # URL - of thing being responded to, relative to $hndocs
    # responses directory where responses are written under $URL
    # upRel relationship to UpResponse
    # title of the response.
    # keywords 
    # referenceURL of the reference, if a reference instead.
    # annotationType - either Response or Reference
    # contentType of body; see edit-response.pl.
    # bodyURL - either full or relative to $hndocs
    # userid of response author
    # userName of response author
    # userURL of response author
    # base of the thing being responded to.
    # previousNum of the URL of the previous response
    local($debug) = 0;

    local $URL = $response{'URL'};
    $title= $response{'Title'};
    $keywords= $response{'Keywords'};
    $annotationType= $response{'AnnotationType'};
    $contentType= $response{'ContentType'};
    $bodyURL= $response{'Body'};
    $upRel= $response{'UpRel'};
    $base= $response{'BaseURL'};
    $responses= $response{'Responses'};
    $userid= $response{'From'};
    $userName= $response{'Name'};
    $userURL= $response{'UserURL'};
    $previousDate = $response{'Date'};
    $messageID = $response{'MessageID'};
    #$messagePath = $response{'Path'};
    $visible = $response{'Visibility'};
    $MailMsgDate = $response{'Date'};

    $numMessages = $response{'NumMessages'};

    $title = &escapeHTML($title);
    $keywords = &escapeHTML($keywords);
    $userName = &escapeHTML($userName);
    # others?  Also see move-lib.pl

    # &ErrorIfTainted($responses);  # tainted here.
    $responses = &untaintPath($responses);  # Shouldnt have to do this here.

    &printAssoc("Response", %response) if $debug;

    local (%member) = &getMember($userid);
    local($Email) = $member{'Email'};

    local($address) = $Email || $userid; # Not used?

    if (!$responses) {
      &HNError("$hnSystemName is broken - please report this bug.
 \$responses is null.");
    }
    $responses = &untaintPath($responses);
    &ErrorIfTainted($responses);
    $responsesPath = "$hndocs$responses";

    # Lock the directory.
    &lock($responsesPath);

    # Get the URL prefix appropriate to this base article, if any.
    # $localhnget = &localhnget($base) || $getURL;
    # Always use the incoming info ... stored in $getURL;
    $localhnget = $getURL;
    if ($localhnget !~ m,^http://,) {
      # The base does not start with $http, which is OK.
      # But we need to report the full URL to the user in that case.
      # But $http might be wrong.
      $fulllocalhnget = "$http$localhnget";
    }

    # Make sure responsesPath exists.
    # This should probably just use create_dir.
    if (!-d $responsesPath) {
 	mkdir($responsesPath, "$hndirmode") || 
	    &HNError ("Problem creating $responsesPath");
        chmod ("$hndirmode", $responsesPath);
    }

    $todaysDate = &wwwdates::wtime(time, 'GMT');
    if ($previousDate) {
       $date = $previousDate
    } else{
       $date = $MailMsgDate || $todaysDate;
    }

    local($annoteName);
    if ($annotationType eq 'Reference') {
	$annoteName = 'ref' . &incAnnotationNum("$responsesPath/.referenceNum");
    }
    else {
	$annoteName = &incAnnotationNum("$responsesPath/.responseNum");
    }

    &writeAnnotation($annoteName);

    # Unlock the directory.
    &unlock($responsesPath);

    &updateLists($base, $responses, $numMessages+1); # Is $numMessages right?

    &updateDateList($base, "$responses/$annoteName") if $sortByDate;
  
    # &HNMsg("userid: $userid\nSubscription: $Subscription") if $debug;
    if ($userid && defined($Subscription) && ($Subscription ne "Notify")) {
      &replaceSubscription($relativePrefix, $userid, 'Unsubscribe');
    }
 
    if ($notifyFlag) {
        &HNMsg("Before notifySubscribers") if $debug;
	&notifySubscribers();
    }

    # Return the new response number.
    return "$annoteName";
}



sub addResponseResult {
  if ($rFrame) {
    #$qsOfFrame = "?frame=response&baseFrame=$baseFrameURL";
  } else {
    #$qsOfFrame = "?frame=outline&baseFrame=$baseFrameURL" if $oFrame;
  }

  if ($gotoForumAfterPost) {
      # Can't do this now.   Why not?
      # This redirect tends to confuse people.
      print "Location: $getURL$URL$qsOfFrame\n\n";
      exit 0;  # done
  }

  my $basePath = &url2hnpath($base);
  local $debug = 0;
  &HNMsg("basePath: $basePath  URL: $URL") if $debug;
  my $inbase;
  if ($basePath ne $URL) {
    my %urc = &parseURCFile("$hndocs$basePath,urc");
    my $baseTitle = $urc{'Title'} || "Untitled Forum";
    $inbase = " in the \"<A HREF=\"$getURL$basePath\">$baseTitle</A>\" forum";
  }
  
  &print_HTML_HEAD("Message Posted");
  my $frameCloseMsg = qq{
If this window popped up when using frames, you can simply close it now.
<P>
} if $allowFrames;

  my $beginningOrEnd = ($reverseMessages ? 'at the beginning' :
			'at the end');

  print qq{
<B>You have successfully added your message to 
 "<A HREF="$getURL$URL$qsOfFrame">$upTitle</A>"$inbase. </B>  <P>

You can click on the above link and continue reading.  You should see
your message added $beginningOrEnd, but if not, <B>please do NOT post
it again</B> or you will have posted two messages.  Instead, you must
<B>Reload</B> or <B>Refresh</B> that page with your browser to see
it. <P>

$frameCloseMsg
Thank you.
};
  # Automatically forwarding after a time delay might be fun.
}


sub saveBody {
    my ($body, $contentType, $file) = @_;

    # Save the $body in a $file, relative to $hndocs.
    # If $contentType is URL, then use body as the bodyURL.
    # Otherwise, convert the body to HTML according to $contentType.
    # Return the bodyURL.

    my $bodyURL;
    local $debug = 0;

    my $ok = 1;

    if ($body) {
	if ($contentType eq "URL") {
	    $bodyURL = "$body";
	    $bodyURL =~ s/\s+//g;  # Remove all whitespace.
	}
	else {
	    # $body not a URL.

	    # Fix up line endings.
	    $body =~ s/\015\012/\012/g; # Convert PC's CR/LF into just LF.
	    $body =~ s/\015/\012/g; # Convert Mac's CR into LF.

	    # Remove leading and trailing blank lines.
	    $body =~ s{^(\s*\n)*}{};
	    $body =~ s{(\n\s*\n)*$ }{\n}x;

	    # Could fix up funky chars here. e.g. =92 == '
	    # Could filter message content here.

	    &HNMsg ("Body file is $file\n") if $debug;
	    # &debugvar('file', $file);

	    &HNMsg ("contentType is $contentType\n") if $debug;
	    if ($contentType eq "Smart Text") {
		my $newbody = &smartText2HTML($body);
		$ok &&= open(BASE, ">$hndocs$file");
		$ok &&= print BASE "$newbody\n";
		$ok &&= close BASE;
	    }
	    elsif ($contentType eq "HTML") { 
		$ok &&= open(BASE, ">$hndocs$file");
		$ok &&= print BASE "$body\n";
		$ok &&= close BASE;
	    }
	    elsif ($contentType eq "Plain Text") {
		$ok &&= open(BASE, ">$hndocs$file") ||
		    &HNError("Cant write: $hndocs$file");
                $body = &escapeHTML($body);
		$body = &anchorize($body);
		$body = "<PRE>$body</PRE>";
		$ok &&= print BASE "$body\n" ||
		    &HNError("Cant print to: $hndocs$file");
		$ok &&= close BASE ||
		    &HNError("Cant close: $hndocs$file");
	    }
	    elsif ($contentType eq "Word Processor") {
		my $newbody = &WPText2HTML($body);
		$ok &&= open(BASE, ">$hndocs$file");
		$ok &&= print BASE "$newbody\n";
		$ok &&= close BASE;
	    }
	    chmod ("$hnfilemode", "$hndocs$file");
	    if (!$ok) {
	      &HNError("Problem opening, writing, or closing $hndocs$file");
	    }

	    # Now use the responseFile instead as the body.
	    $bodyURL = "$hndocsURL$file" if $file; # needed by hnview 
        }
    }

# Automatically remove the tmp file after 10 minutes in child process.
# Doesn't work in perl 5 on solaris 2.3.
# If you fix this, let me know.
#unless (fork) 
# I'm the child.
#  sleep(10*60); # 10 minutes
#  unlink $responseFile;
#  exit;  # the child stops executing here.
# # parent continues here

# Remove any tmp files older than a day, just in case.
    opendir HN, $hndocs ||
	  &HNError("Problem reading $hndocs");
    my @tmpfiles = readdir HN;
    closedir HN;
    @tmpfiles = grep {/^hn_tmp/} @tmpfiles;

    my $filename;
    foreach $filename (@tmpfiles) {
	$filename =~ m,^(.*)$,;
	$filename = $1;  # Bypass tainting.
	if (-M "$hndocs/$filename" > 1) {
	    unlink "$hndocs/$filename";
	}
    }

    return $bodyURL;
}


sub WPText2HTML {
  # Filter for converting "Word Processor" text into HTML.
  my ($body) = @_;
  local $debug = 0;

  $body = &escapeHTML($body);  # This breaks URLs with '&'.

  &HNMsg("body: $body") if $debug;

  $body =~ s,\n,<BR>\n,g;  # Break lines
  $body =~ s,(^|\n) ,\n&nbsp;,g;  # Replace spaces
  while ($body =~ s,\&nbsp; ,&nbsp;&nbsp;,g) {}
  $body =~ s,(^|\n)\t,\n&nbsp;&nbsp;,g; # Replace tabs
  while ($body =~ s,\&nbsp;\t,&nbsp;&nbsp;&nbsp;,g) {}
  
  # Make everything use TT font.
  $body = "<tt>\n$body\n</tt>\n";

  $body = &anchorize($body);
  &HNMsg("body: $body") if $debug;

  return $body;
}


sub smartText2HTML {
    # Filter for converting "Smart Text" into HTML.
    my ($body) = @_;
    local $debug = 0;

    # We really shouldnt be doing this here - it breaks URLs with '&'.
    $body = &escapeHTML($body);

    # Split into paragraphs.
    my @paragraphs = split(/\n[ \t\n]*\n/, $body);
    $body = '';
    my $para;
    foreach $para (@paragraphs) {
      # If any line after the first starts with whitespace, 
      # PRE-format the whole paragraph.
      &HNMsg("para: '$para'") if $debug;
      if (($para =~ m,\n[ \t],) || 
	  ($para =~ m,^[ \t].*$,)) # one line that starts with whitespace.
	{
	  $para = "\n<PRE>$para</PRE>\n";
	}
      else {
	# This is getting too smart for its own good.
	$para .= "\n"; 	# Add a new line after each paragraph.
	$para =~ 
	  s{(
	    # Recognize if two or more lines have the same prefix.
	    # A prefix is maybe whitespace or words followed by 
	    # typical quoting chars e.g. "Tom>"
	     ((^|\n)
	      ([\w \t]*([|:<>\]\*\#]|&(lt|gt);)) # the prefix
	      .*\n(\4.*(\n|\$))+)
	    )}
	    {\n<PRE>$1</PRE>\n}xg;

	# Recognize signature separator - should maybe only be last paragraph.
	#if ($para =~ m,(^|\n)--\n,) {
	#  $para = "$`\n<PRE>--\n$'</PRE>"
	#}
      }
      &HNMsg("new para: '$para'") if $debug;

      $body .= "\n$para<P>\n";
    }

    $body = &anchorize($body);
    return $body;
}

# Previously used regexp for smarttext.
	     # | 
	     # Also, recognize any lines starting with ...
	     # (This might be too tricky)
	     #(((^|\n)
	     #  (
		#[\w-]+:\ |  # any '<header-name>: '  Is this safe?
		#[|:<>\]] |  # or some prefix chars.  Is this redundant?
		#&(lt|gt);
	       # )
	       #.*)+)


$URIscheme = '(http://|ftp://|afs://|wais://|telnet://|gopher://|' .
  'news:|nntp:|mid:|cid:)';  # more? 

$URIre = $URIscheme . 
  q%[^\s\(\)\|<>\"\']*[^\.;,\"\'\|\[\]\(\)\s<>]%;


sub anchorize {
  my ($body) = @_;

  # Recognize email addresses and use 'mailto:'  
  # Do this before URIs?  Need not worry now - URIre doesnt include mailto.
  # Don't allow '%' instead of '@'.  URLs may include '%'.
  $body =~ s{([A-Za-z0-9_=+%/\.\-]+[@][A-Za-z0-9_\-][A-Za-z0-9_\.\-]*)}
  {<A HREF="mailto:$1">$1</A>}xg;

  # Make anchors of URIs.
  $body =~ s{($URIre)}{&make_anchor($1)}eg;

  return $body;
}

sub make_anchor {
    my ($str) = @_;
    my $URL = $str;
    $URL =~ s{&amp;}{&}g;
    return qq{<A HREF="$URL">$str</A>};
}

sub localhnget {
  my ($url) = @_;
  # Return the hnget prefix of $url.
  if ($url =~ m|^(.*/$get)/|) {
    return $1;
  }
}


sub localizeURL {  # not used?
    my ($url, $base) = @_;
    # Change $url to use same prefix as $base
    my ($basehnget) = &localhnget($base);
    $url =~ s|^.*/get(\.cgi)?/|$basehnget/|;
    return $url;
}


sub incAnnotationNum {
    my ($filename) = @_;
    # Get the number in the $filename and increment it for next time.
    # Assume locking is done by caller.

    #&HNMsg("Incrementing: $filename");

    open (RNFH, "<$filename"); # it need not exist.
    my $num = <RNFH>; # num will be effectively zero if no file.
    close RNFH;
    $num =~ m,([0-9]*),;
    $num = $1 + 1;  # untaint while adding.

    # Do this carefully - otherwise the .responseNum file could be lost.
    open(RNFH, ">$filename-new")  || 
	&HNError("Cannot open $filename-new");
    print RNFH "$num\n" ||
	&HNError("Cannot write to $filename-new");
    close RNFH;

    &moveItSafely($filename, "$filename-old", "$filename-new");
    return $num;
}

sub fixPreviousURCFile {
    local($nextNum) = @_;

    &HNMsg("nextNum before::$nextNum") if $debug;

    #get the parent URL to compute previous URL
    local($parentDir) = $nextNum;
    $parentDir =~ s|/[0-9]+$||;
    $nextNum =~ s|^$parentDir/||;
    &HNMsg("nextNum after: $nextNum") if $debug;


    local(@files) = &responsesOf($parentDir);
    $index = @files - 1;
    $previousURC = $files[$index];
    $previousNum = $previousURC;

    #test whether previous response exits
    if ($previousURC eq ""){
	#&HNMsg("previous response doesn't exit, previousURC: $previousURC");
        return; 
    }

    $previousURC = &untaintPath("$hndocs$parentDir/$previousURC");

    # parse the previous URC file and rewrite $nextNum;
    &lock($previousURC);
    local(%in) = &parseURCFile($previousURC);
    $in{'NextNum'} = $nextNum;

    &_writeURC($previousURC, %in);
    &unlock($previousURC);
    #&HNMsg("0 previousURC:$previousURC");
    
    # computing $previousURC and pass to new response URC file.
    $previousNum =~ s|\.html,urc$||;
    #&HNMsg("1 previousNum:$previousURC");

    return $previousNum;
}


sub writeAnnotation {
  local($name) = @_;
  # Create the file and URL for the annotation named $name.

  &ErrorIfTainted($name);
  $relativePrefix = "$responses/$name";
  $fullPrefix = "$hndocs$relativePrefix";

  if ($messageID && !$moving) {
      # Save reference in $hnmidDB if this is a new message.
      # *Any* $messageID that is a local HN ID should not be written
      # to the $hnmidDB.  A message that looks like a repost of the same
      # message, using the same message ID, should probably be aborted
      # since this looks like there is a circularity.
      # But we have tested for that already in add-response.
            
      # This is a new message.
      &logTransaction("Add Email Reply", "$relativePrefix.html");
      
      &HNMsg("messageID:s $messageID") if $debug;
      my ($midfile, $midhost) = split('@', $messageID);

      my $middir = &untaintPath(&fliphostname($midhost));
      $midfile = &untaintPath($midfile);

      &HNMsg("
midfile = $midfile
midhost = $midhost
middir  = $middir") if $debug;
      
      #----Store MID as a URC type file, assuming it doesnt exist.
      
      &create_dir("$hnmidDB/$middir");
      my @midparams = ( 'Responses' , $relativePrefix );
      my $midpath = "$hnmidDB/$middir/$midfile";

      &writeURC("$midpath,urc", @midparams);

      &HNMsg("Wrote: \"$midpath\"  Responses: \"$relativePrefix\"") 
	  if $debug;
      
      open(HEADERFILE,">$midpath.header");
      print HEADERFILE "$headerString";
      close(HEADERFILE);
      
    }
  else {
    &logTransaction("Add", "$relativePrefix.html");
  }


  # &debugvar('bodyURL', $bodyURL);
  if ($bodyURL) {
      if ($contentType eq "URL") {
	# OK as is.
      }
      else {			
	# bodyURL is really the name of a temp file 
	# where the body is stored.
	my $bodyFile = $bodyURL;
	$bodyFile =~ m,(/hn_tmp[0-9]+)$,;
	$bodyFile = $1;  # Untaint while extracting temp file name.
	&ErrorIfTainted("body: $bodyFile");
	&ErrorIfTainted("prefix: $fullPrefix");
	if (!rename("$hndocs$bodyFile", "$fullPrefix-body.html") &&
	    # Try copying in case forum is in a different file system.
	    !&copy_file("$hndocs$bodyFile", "$fullPrefix-body.html", 'delete'))
	{
	    # If error occurs here, response num is already incremented.
	    if (-e "$hndocs$bodyFile") {
		&HNError("Can't move or copy '$hndocs$bodyFile' to 
 '$fullPrefix-body.html'.");
	    }
	    else {
		# temp file is missing.
		&HNUserError("$hnSystemName can't post your message.
Perhaps you already submitted this message.");  # give URL
	    }
	}
	$bodyURL = "$hndocsURL$relativePrefix-body.html";
      }
    }

  #fix previous response URC file
  #&HNMsg("name:$name");
  if ($name != 1){
    $nextNum = $relativePrefix;
    $previousNum = &fixPreviousURCFile($nextNum);
  }    
  #&HNMsg("outer previousNum:$previousNum");

  # Create a HN MID.  Use as message ID if there is none.
  my $prefix = "$hnGroup:" if $hnGroup;
  # We could probably drop the '"'s in the mid.
  my $hnID = "\"$prefix$relativePrefix\"\@$ENV{SERVER_NAME}";
  $messageID ||= $hnID;

  # Prepend the HyperNews ID to the message path.
  # $messagePath = join (', ', ($hnID, $messagePath));

  # The base should match equivalent groups too.
  # Need other separators?
  my $relativeBaseURL = &url2hnpath($base);
  if ($newsgroups !~ m{$relativeBaseURL([, ]|$)}) {  
    my @newsgroups = split (/[, ]+/, $newsgroups);
    # Newsgroups line should include whatever is there now plus this group.
    my $fullURL = ($fulllocalhnget || $localhnget) . $relativeBaseURL;
    $newsgroups = join (', ', ($fullURL, @newsgroups));
  }
  my $newURL = "$localhnget$relativePrefix.html";

  # Build the URC file.
  my @params = 
	('Title' , $title,
	 'URL' , $newURL,
	 'MessageID' , $messageID,
	 #'Path' , $messagePath,
	 'Keywords' , $keywords,
	 'AnnotationType' , $annotationType,
	 'ContentType' , $contentType,
	 'Body' , $bodyURL,
	 #'References' , $references,
	 'UpURL' , "$localhnget$URL",
	 'UpRel' , $upRel,
	 'BaseURL' , $base,
	 'Newsgroups' , $newsgroups,
	 'Responses' , $relativePrefix,
	 'From' , $userid,
	 'Name' , $userName,
	 'UserURL' , $userURL,
	 'Date' , $date, 
	 'LastMessageDate' , $date,
	 'LastMod' , $todaysDate,
	 'PreviousNum', $previousNum);

	if ($numMessages) {
	  $param{NumMessages} = $numMessages;
	}

    if ($thearticleIsDeleteNow) {
      push (@params, 'Deleted', $thearticleIsDeleteNow);
    }

    if ($referenceURL) {
      push (@params, 'ReferenceURL', $referenceURL);
    }
	
      &writeURC("$fullPrefix.html,urc", @params);
      &create_dir($fullPrefix); # Always create the directory.
}


sub updateLists {
    my ($base, $dir, $diffCount) = @_;
    # $base forum 
    # $dir relative to hndocs.
    # $diffCount is how much were are adding to the count.
    local $debug = 0;
    &HNMsg("dir: $dir diffCount: $diffCount") if $debug;

    my $adding = $diffCount >= 0;
    # $dir =~ s,\.html$,,;

    # Regenerate the whole list.
    &clearResponseList($dir);

    $todaysDate = &wwwdates::wtime(time, 'GMT');

    #&HNMsg ("URL: $localhnget$URL\n") if $debug;
    &HNMsg ("BaseURL: $base\n") if $debug;
    my $endPath = &url2hnpath($base);
    $endPath =~ s,/[^/]+$,,; # Delete last component.
    my $path = $dir;
    &HNMsg("endPath: $endPath") if $debug;

    my $clearNextLevelUp = 0;

    # while not looking at the base, update ancestor nodes.
    while ($path && ($path ne $endPath)) {  
      &HNMsg("path: $path") if $debug;

      if ($clearNextLevelUp) {
	# This clear is to add or remove the '#include' line.
	&clearResponseList($path);
	$clearNextLevelUp = 0;
      }
      
      &lock("$hndocs$path.html,urc");
      my %urc = &parseURCFile("$hndocs$path.html,urc");
      if ($adding) { $urc{'LastMessageDate'} = $todaysDate; }
      $urc{'LastMod'} = $todaysDate;
      
      if ($urc{'NumMessages'} == 0 && -d "$hndocs$path") {
	# Accumulate diff counts.  This is a work around since
	# previous versions of HyperNews did not accumulate counts.
	if ($path eq $dir) {
	  # The initial diffCount should be adjusted to not count
	  # the message that was added or deleted.
	  $diffCount -= (($diffCount>0) ? 1 : -1);
	}
	opendir (DIR, "$hndocs$path") ||
	  &HNError("Problem reading $hndocs$path");
	my @messages = readdir DIR;
	closedir DIR;
	@messages = grep (/^[0-9]+\.html,urc$/, @messages);
	$diffCount += @messages;
      }
      
      &HNMsg("diffCount: $diffCount") if $debug;
      my $oldNumMessages = $urc{'NumMessages'};
      $urc{'NumMessages'} += $diffCount;
      if ($urc{'NumMesages'} < 0) { $urc{'NumMessages'} = 0; }
      &HNMsg("NumMessages: $urc{'NumMessages'}") if $debug;
      
      &_writeURC("$hndocs$path.html,urc", %urc);
      &unlock("$hndocs$path.html,urc");
      
      if (($oldNumMessages == 0) != ($urc{'NumMessages'} == 0)) {
	# Changed from 0 to >0 or >0 to 0.
	&clearResponseList($path) if ($path ne $dir);
	$clearNextLevelUp = 1;  # Must clear next level up in any case.
      }
      
      $path =~ s,/[^/]+$,,;	# Delete last component.
    }
}

# use IO::File;

sub updateDateList {
    my ($base, $dir) = @_;
    # Update the appropriate date list with current message.
    # Update the forum at $base, at directory $dir relative to $hndocs.

    # Todo: work with deleting and moving messages in and out.

    local $debug = 0;

    return if !$dateBoundaries;

    my $forumDir = &untaintPath(&url2hndir($base));

    &lock("$hndocs$dir");
    &HNMsg("Dir: $dir") if $debug;
    my %msgURC = &parseURCFile("$hndocs$dir.html,urc");
    my $date = $msgURC{Date};
    my $msgDate = &dateToUID($date);

    my %baseURC = &parseURCFile("$hndocs$forumDir.html,urc");

    push (@INC, "$hnbin/.scripts/lib");
    require Date::Manip; import Date::Manip;
    # or do: eval 'use Date::Manip; 1' || die $@;

    # $Date::Manip::Debug="trace";

    my $startDate = 'Aug 1, 1998'; # Earliest date - e.g. install date.
    my $lastBoundary = $baseURC{LastDateBoundary} || $startDate;
    # if ($lastBoundary < $startDate) { $lastBoundary = $startDate; }

    &HNMsg("start: '$lastBoundary'") if $debug;
    &HNMsg("end: '$date'") if $debug;
    &HNMsg("recur string: '$dateBoundaries'") if $debug;

    my @nextBoundaries = 
	&ParseRecur($dateBoundaries, $lastBoundary, $lastBoundary, $date);
    &HNError("Date boundaries missing.") if !@nextBoundaries;
    if ($debug) {
	foreach $d (@nextBoundaries) {
	    &HNMsg("boundary: $d");
	}
    }
    my $nextBoundary = pop @nextBoundaries;  # Last one is our next boundary.

    # Convert to UID and to full date.
    my $unixSeconds = &UnixDate($nextBoundary, '%s');
    $nextBoundaryDir = &dateToUID(0, $unixSeconds);
    $nextBoundary = &wwwdates::wtime($unixSeconds + 0, 'GMT');
    
    &HNMsg("nextBoundaryDir: '$nextBoundaryDir'") if $debug;

    my $currentDateDir = "$hndocs$forumDir/.byDate/$nextBoundaryDir";
    # &ErrorIfTainted($currentDateDir);
    &HNMsg("Date Dir: $currentDateDir") if $debug;
    &HNMsg("Date File: $msgDate") if $debug;

    if ($nextBoundary ne $lastBoundary) {
	&HNMsg("Create the new date directory: $nextBoundary") if $debug;
	&create_dir($currentDateDir);
	# Remember the new boundary point.
	$baseURC{LastDateBoundary} = $nextBoundary;
	&writeURC("$hndocs$forumDir.html,urc", %baseURC);
    }
    &unlock("$hndocs$dir");
    
    # &ErrorIfTainted($dir);

    $dir =~ m{((/[0-9]+)+)}; # Keep only the message number path.
    my $msgDir = $1;
    &HNMsg("msgDir: $msgDir") if $debug;

    # Add the message to the current date dir.
    symlink ("../..$msgDir", "$currentDateDir/$msgDate");
}


sub dateToUID {
    my ($date, $seconds) = @_;
    # Create a UID given the $date.  
    # Todo: append integer for more than one UID per second.
    # Todo: use '/' instead of ':' to group messages in dirs.
    my $time = $seconds || &wwwdates::get_gmtime($date);
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
	localtime($time);
    $year += 1900;
    
    sprintf("%04d:%02d:%02d:%02d:%02d:%02d",
            $year, $mon+1, $mday, $hour, $min, $sec);
}


# These should be in hnrc.

# Break large mailing lists down to several smaller lists.
# Problem: the 'To:" recipient is always the same, and gets a copy of each.
$MAX_RECIPIENTS_PER_EMAIL = 100;
$DELAY_BETWEEN_EMAIL = 5;  # in seconds

sub notifySubscribers {
  # Email notification to subscribers.
  
  local $debug = 0; # debugging here may interfere with email.
  
  &HNMsg ("In Notify Subscribers...for $URL.  notifying: $hnEmailNotify") 
      if $debug;
  return if (!$hnEmailNotify);

  my $baseFile = &url2hnpath($base);
  my %baseurc = &parseURCFile("$hndocs$baseFile,urc");
  my $listAddress = $baseurc{'ListAddress'};
  &HNMsg ("List Address: $listAddress") if $debug;
  
  # Build mailing list starting from this posting.
  my %list = &buildList("$relativePrefix.html");
  &printAssoc("After buildList", %list) if $debug;
  #print "This bug is dragging its heals...\n\n";
  #&flush(STDOUT);
  
  my @immediate_list = &filterList('Immediate', %list);
  &HNMsg ("the List is: @immediate_list\n") if $debug;
  #print "This is really painful debugging...\n\n";
  #&flush(STDOUT);

  return if !@immediate_list;  # Check if anyone wants the mail.
  
  my %member = &getMember($userid);

  my $name = &unescapeHTML($userName || $member{Name});

  my $address = &addressOf($userid);

  my $hideEmailAddressInEmail = ($member{Hide} eq 'Email') || 0;
  # Maybe don't use email address in mail if $hideEmailAddressInEmail

  my $fromAddress;

  if (!$address) {
      # '(name)' by itself causes failure for some mail programs
      # Need to say 'undisclosed-sender:;'
      $name ||= 'Unknown Author';
      $fromAddress = qq{"$name via $hnSystemName" <$mail_from>};
  }
  else {
      $fromAddress = qq{<$address>};
      $fromAddress = qq{"$name" $fromAddress} if $name;
  }
  &HNMsg ("fromAddress: $fromAddress") if $debug;
  &flush(STDOUT);
  
  # Set errorsAddress to ... what?  
  #  forum administrators, group admin, site admin, bounce-agent?
  my $errorsAddress = '';
  if ($mail_errors_to) {
    $errorsAddress = "Errors-To: $mail_errors_to";
  }
  
  # Must be local var.  Used by printHeaderLines
  local $upURL = "$localhnget$URL";
  
  # Get the parent MID.
  my %upURC = &parseURCFile("$hndocs$URL,urc");
  my $inreplyto;
  my $upMID = $upURC{'MessageID'};
  # If no mid found, we could create one.
  if ($upMID) {
    $inreplyto = "In-Reply-To: <$upMID>";
  }

  # References should include the whole chain up to the base.
  # Need to break up long lines.
  # my $referencesLine = "References: $references\n  <$upMID>";

  # This Newsgroups line messes up MS Outlook express since it tries
  # to post to a news group and fails to mail to HyperNews.
  my $newsgroupsLine = "Newsgroups: $newsgroups";

  
  &HNMsg("Before unescapeHTML") if $debug;
  my $subject = &unescapeHTML($title);
  if ($upRel && $upRel ne 'None') {
    $subject = "$upRel: $subject";
  }
  my $hnSystemAbbrev = '';  # Move to hnrc !!
  if ($hnSystemAbbrev && ($subject !~ m{$hnSystemAbbrev})) {
    # Add a reference to $hnSystemAbbrev in outgoing email.
    $subject .= " (ref: $hnSystemAbbrev)";
  }
  
  # Where to visibly send the email?  Not merely a formality.
  # The base article might have an email list.
  $mail_to = $listAddress || $mail_to;
  # The person being responded to might not be subscribed.
  $toAddress = $mail_to || $address;
  
  my $senderAddress = $mail_sender;
  if ($listAddress) {
    $senderAddress = "Owner-$listAddress";
  }

  
  my $fullURL = ($fulllocalhnget || $localhnget) . 
    $relativePrefix . ".html";

  my $replytoAddress = $listAddress || $mail_reply_to || $mail_to;
  my $replyto;
  # This is screwy, in the case of a listAddress.
  $replyto = "Reply-To: $replytoAddress" if $mail_reply_to;

  my $moreHeaders;
  foreach $header ($inreplyto, $referencesLine, $replyto, 
		   $errorsAddress, $newsgroupsLine) {
      $moreHeaders = join ("\n", $moreHeaders, $header) if ($header);
  }

  local $debug = 0;
  
  while (@immediate_list) {
      my @tosend = splice(@immediate_list, 0, $MAX_RECIPIENTS_PER_EMAIL);

      my $bccaddress = join(", ",  @tosend);
      # Need to &ParseAddrs($bccaddress);

      # Sendmail may put out an error message at this point that
      # will break the CGI protocol.  We should either avoid the error
      # or catch it.

      &HNMsg("
Subject: $subject
Bcc: $bccaddress
Message-ID: <$messageID>
X-Mailer: $hnSystemName $hnversion
X-Hn-Url: $fullURL
Precedence: list$moreHeaders
") if $debug;


      local $debug = 0;
      &sendmail(HYPERNEWS, $toAddress, $fromAddress, $senderAddress);

      &HNMsg("returned from sendmail\n") if $debug;

      # Redirect STDOUT to HYPERNEWS
      my $saveout = select(HYPERNEWS);
      $| = 1;			# make unbuffered

      &HNMsg("Before printHeaderLines") if $debug;

      # More header lines.
      &printHeaderLines('plaintext');

      &HNMsg("Returned from printHeaderLines\n") if $debug;

      # More header lines.
      # Start with newline?
      print HYPERNEWS "
Subject: $subject
Bcc: $bccaddress
Message-ID: <$messageID>
X-Mailer: $hnSystemName $hnversion
X-Hn-Loop: $fullURL
Precedence: list$moreHeaders
";

      #print HYPERNEWS "Path: $messagePath\n";

      print HYPERNEWS "\n";	# Extra blank line separating header from body

      if ($allowEmailIn) {
	  print HYPERNEWS 
	    qq{If you reply to this message at $replytoAddress, include:
In-Reply-To: <$messageID>

} if !$listAddress;
      }
      else {
	  print HYPERNEWS qq{Visit this $hnSystemName message (to reply or unsubscribe) at: 
$fullURL

};
      }
#-------------------------------------------------------------

      if ($bodyURL =~ m,$relativePrefix-body.html,) {
	  require "parse-html.pl";
	  require "html-ascii.pl";
	  &parse_html ("$fullPrefix-body.html");
      }
      elsif ($bodyURL) {
	  my $pbodyURL = $bodyURL;
	  if ($pbodyURL !~ m,^http:,) {
	      $pbodyURL = "$http$pbodyURL";
	  }
	  print HYPERNEWS "\nThe message body is at: \n  $pbodyURL";
      }

      print HYPERNEWS qq{
-------------------------------------------------------------
Visit this $hnSystemName message (to reply or unsubscribe) at: 
$fullURL
};

#Unsubscribe at: 
#  $editSubscribeURL$relativePrefix.html

      # Restore STDOUT
      $| = 0;			# make buffered again
      select($saveout);
      &HNMsg("Successful saveout\n") if $debug;

      close (HYPERNEWS);

      sleep $DELAY_BETWEEN_EMAIL if (@immediate_list);
  }

}


sub fliphostname {
    my ($hostname) = @_;
    print main::DEBUG "\nhostname =  $hostname\n" if $debug;
    $hostname =~ s,^\[([^\]]*)\]$,$1,;  # Drop [...]s from IP number.
    my @domains = split (/\./, $hostname);
    if ($hostname =~ /^[^\d]/) {
      # Only flip domain names not IP address parts.
      @domains = reverse @domains;
    }
    return join("/", @domains);
}

sub TestPermissions {
    my ($permission, $file) = @_;
    # This is a fairly bogus routine to test whether a file has 
    # the designated permissions.
    local($PermissionResult) = stat($file);
    # Expects permissions in hex form; the next line converts it to text.
    local($textResult) = sprintf("%0.4o", $PermissionResult);
    $textResult =~ s/\d\d\d$//;
    $textResult = "0$&";

    #Convert the Hex permissions to text
    local($permissionText) =  sprintf("%0.4o", $permission);

    #Error if the Permissions are incorrect
    if (!($permissionText =~ /$textResult/)) {
	&HNError ("The file $file has incorrect permissions<p>
        File permission is $textResult.<br>
        Expected permission is $permissionText.<br>");
    }
    return(1);
}


sub create_dir {
    my ($dir) = @_;
    # Create $dir, relative or absolute path.
    # The directory may already exist, or any tail of it may be missing.
    # All remaining parts are created, using $hndirmode.

    # Need to loop rather than use mkdir -p because the latter doesnt
    # work on all OSes.  Perl should have a similar option.

    # local($debug) = 1;

    my $directory;
    while ($dir =~ s,^(/?[^/]+),,) {
	if ($directory) {
	    $directory = "$directory$1";
	} else {
	    $directory = $1;
	}
        &HNMsg ("directory is $directory\n") if $debug;
	if (! -d $directory) {
	    mkdir($directory, "$hndirmode") || 
		&HNError ("Can't create directory: $directory");
            chmod("$hndirmode", $directory);  # Redundant?
            # &TestPermissions("$hndirmode", $directory);
	}
    }
    if (! -d $directory) {
	&HNError ("Unable to create directory: $dir");
    }
}


sub decode {
  my ($string) = @_;
  # Return the string decoded as if from a FORM.
  # e.g. $title = &decode($title);
  
  $string =~ s/\+/ /g ;
  $string =~ s/%(..)/pack('c',hex($1))/eg; # unescape characters
  if ($string =~ "^''\$") { $string = ""; }
  $string;
}


sub unescapeHTML {
   local($string) = @_;
 
   $string =~ s/^\s*//;  # delete leading space
   $string =~ s/\&lt;/</g ;
   $string =~ s/\&gt;/>/g ;
   # $string =~ s/\&quot;/\"/g ; don't undo what wasnt done.
   # Dont quote double quotes.  
   $string =~ s/\&amp;/\&/g ; # do this last.
   $string;
}

