#!/usr/bin/perl -T
# admin-responses.pl - Administration of response trees.
# Copyright Board of Trustees of the University of Illinois at UC, Feb 1996

# Currently only copy and delete are supported here.  But other functions,
# such as visibility, archiving, and attribution modification are possible
# using the same general interface.

# Add current directory to load path.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};
$0 =~ m{^(.*)/[^/]+$ }x;
$hnbin = $1 || '.';
unshift(@INC, $hnbin, "$hnbin/.scripts"); 


require "Admin/hnrc";
&feval('&AdminResponsesPL()');

sub AdminResponsesPL {
    require "add-lib.pl";
    require "member-lib.pl";
    require "formlib.pl";

    local $debug = 0;

    my $URL = $ENV{PATH_INFO};
    &HNMsg("URL: $URL\n") if $debug;

    my %form = &GetFormArgs();

    &printAssoc('form', %form) if $debug;

    &checkAuthenticated($form{UserID}, $form{Password},
			$form{remoteUser}, $form{adminPassword});
 
    my $whoisthis = &sustring($remoteUser);


    my %qs = &parseQueryString();

    if ($allowFrames && $qs{'frame'}) {
	$frame = $qs{'frame'};
	$bFrame = $frame eq 'base';
	$oFrame = $frame eq 'outline';
	$rFrame = $frame eq 'response';
	$baseFrameURL = $qs{'baseFrame'};
	&frameVariables();
	&HNMsg("frame: $frame") if $debug;
    }
    
    $admin = $qs{'admin'};

#    $actionQS = &appendToQS('', $qsOfFrame); # and $qsOfBaseFrame
#    $actionQS = &appendToQS($actionQS, "admin") if $admin;

    # &frameVariables() if ($rFrame || $bFrame || $oFrame);  redundant?

    
    my $responses = $form{'response'};
    my @sourceList = reverse &flatten($responses);

    &HNUserError("You must select at least one message to act on.")
	if !@sourceList;

    $asbase = $form {'asbaseyesno'} eq 'yes';  # Not used yet

    $delete = $form {'deleteyesno'} eq 'yes';
    &HNMsg ("\$delete is $delete\n") if $debug;
    $notify = $form {'notifyyesno'} eq 'yes';

    &HNMsg("sourceList: @sourceList") if $debug;

    $destination = $form {'DESTINATION'};
    if ($destination) {
	$destination =~ s,\s*,,g; # Remove whitespace
	$destination =~ s|^[^/]|/$&|; # Prepend missing '/'.
	$destination =~ s,/$,,;	# Delete trailing '/'.
	if ($destination !~ m,\.html$,) {
	    # Append missing '.html';
	    $destination .= '.html';
	}

	# Test whether destination, a relative URL, exists.
	&HNMsg ("Checking if destination exists..\n") if $debug;
	if (-e "$hndocs$destination,urc") {
	    if ($asbase) {
		&HNUserError ("Destination '$destination' already exists!");
	    }
	}
	else {
	    if (!$asbase) {
		&HNUserError ("Destination '$destination' does not exist!
 Maybe create it first.");
	    }
	    if (@sourceList > 1) {
		&HNUserError ("
 Select just a single message to be made into a new base article.
 All replies will be moved as well.");
	    }

	}
    }

    if ($destination) {
	require 'move-lib.pl';
	&HNMsg("Moving or copying messages to $destination") if $debug;
    }
    if ($delete) {
	require 'delete-lib.pl';
	&HNMsg("Deleting messages") if $debug;
    }

    # Operate on the list of messages one at a time.
    map {
	my ($source) = $_;
	my %response = &parseURCFile("$hndocs$source,urc");

	&HNMsg("Acting on $source") if $debug;
	&printAssoc("source", %response) if $debug;

	$owner = $response {'From'};
	$base = $response{'BaseURL'};

	if (!$remoteUser) {
	    $remoteUser = '';
	}

	if ($destination) {
	    &access1($moveSF, 2, $owner, $remoteUser, 
		     "User <strong> $remoteUser </strong> 
 is not permitted to copy $source owned by $owner.",
		     $authenticated, $base);

	    # This does deletion too, if needed.
	    &move($source, $destination, $delete, $notify, $expires);
	}
	elsif ($delete) {
	    &access1($deleteSF, 2, $owner, $remoteUser,
		     "User <strong> $remoteUser </strong>
 is not permitted to delete $source owned by $owner",
		     $authenticated, $base);

	    &deleteTree($source, $expires);
	}
    } @sourceList;

    #===============================
    # Return result - a redirect.
    # Loop to find first URL that exists.
    # Should be smarter when used with frames.

    local $debug = 0;

    my $tmpURL = $URL;
    &HNMsg("tmpURL:$tmpURL\n") if $debug;

    while ($tmpURL && !-e "$hndocs$tmpURL,urc") {
	# Chop off last component and try again.
	$tmpURL =~ s,/[^/]*$,.html,;
	&HNMsg("tmpURL:$tmpURL\n") if $debug;
	# Cause new frameset to be used.
	# This is only really needed when we climb higher than baseFrame.
	# $target = $targetTop;
	# $qsOfFrame = "?$resetFrame";
	# Doesn't work yet because of what follows.
    }

    if (!$tmpURL) {
	&HNError("Something horrible has happened to '$URL'.
Please report it.");
    }
    my $qs;
    $qs = &appendToQS('', $qsOfFrame, $qsOfBaseFrame);
    $qs = &appendToQS($qs, "admin") if $admin;
    
    &HNMsg("targetTop: $targetTop\n") if $debug;
    &HNMsg("qs: $qs") if $debug;

    if ($target) {
	$target =~ m,TARGET="(.*)",;
	my $windowTarget = $1;
	print "Window-target: $windowTarget\n";
    }
    print "Location: $getURL$tmpURL$qs\n";

    #&print_HTML_HEAD();
    #print "<P>Message deleted from: 
# <A HREF=\"$getURL$tmpURL$qs\" $target>
# $getURL$tmpURL$qs</A>.\n";
    print "\n";
}


sub flatten {
    my ($dottedlist) = @_;
    # Flatten elements of the dotted list: [atom, dottedlist | atom]
    # Return the flattened list.
    local $debug = 0;
    my (@resultlist) = ();
    while ($dottedlist) {
	&HNMsg("dottedlist:$dottedlist\n") if $debug;
	if (ref $dottedlist) {
	    my @cell = @$dottedlist;
	    push @resultlist, $cell[0];
	    #&HNMsg("cell[0]: $cell[0]") if $debug;
	    $dottedlist = $cell[1];
	} else {
	    push @resultlist, $dottedlist;
	    #&HNMsg("dottedlist: $dottedlist") if $debug;
	    $dottedlist = '';
	}
    }
    #&HNMsg("result: @resultlist") if $debug;
    return @resultlist;
}

