#!/usr/bin/perl
# batch-add-members.pl - adds members to HyperNews from a data file
# Copyright HyperNews.org, 1998

# Written by Daniel LaLiberte for HyperNews.org.
# Thanks to "Chris 'Floyd' Kreutzer" <floyd@news2.uww.edu> February, 18 1997

# This script is experimental.  Please have safe experiments.

# Use this program to add several members to HyperNews hnpeople directory
# automatically using data from input (or from one or more files).

# (Part of this is general purpose, for future use, and part is
# specific to the HN membership files.)

# The first line of the input is the pattern used to split apart lines
# into fields.  The second line of the input identifies the name of
# the type of data, and the names of each of the fields that will appear on
# all subsequent lines.  All the names and values are separated by the
# pattern.  

# Any fields that are blank or missing will be left blank or undefined
# (same thing here) in the data.  (It should be possible, in the
# future, to supply the first two lines by some other means.)

# For the Member data, the fields may include the following:
#	UserID
#	Password  - unencrypted
#	EmailAddress
#    	FullName
#	UserURL
#	MailAddress
#	Phone
#
#
# The only required field is the UserID.  
# Future option: a password will be generated if one is required.
#
#
# Example input (ignore leading '#  '):
#  Member, UserID, Email, FullName
#  liberte, liberte@hypernews.org, Daniel LaLiberte
#  hnadmin, hnadmin@hypernews.org, HyperNews Administrator

#################################
# NOTE:
# Before you run this program,
# you must first su to the same user who runs HyperNews
# (or arrange for common group writable permission).
#################################
# After running batch-add-member.pl, you must run 'setup.pl -P'.
# Or use the Rebuild Password file option in setup-form.pl.
#################################

#################################
# Options

# Whether current members are affected.
my $changeCurrentMembers = 0;

# If $changeCurrentMembers == 1, whether current values are changed,
#   except current password is kept.
my $overwriteCurrentValues = 0;

# The initial status of new members.
my $initialStatus = 'invited';

# Whether to rebuild the password file after adding these members.
my $rebuildNow = 0;

#################################

BEGIN {
    # Prepend HyperNews directories to load path.
    $0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
    $hnbin =~ s{/(.scripts|Admin|SECURED)}{}g;
    unshift(@INC, $hnbin, "$hnbin/.scripts"); 
}

require 'Admin/hnrc';
require 'change-member-lib.pl';

#################################

my $line;

# Get the delimiter pattern from first line.
#$line = <>;
#chop $line;
#my $delimiter = $line;

$line = <>;
chop $line;

# Get delimiter from chars between first and second symbol.
$line =~ m{\w([^\w]+)};
my $delimiter = $1;

# Get the object name and field names.
my ($objectType, @fieldNames) = split (/$delimiter/, $line);

foreach $line (<>) {  # For each input line...
    # Remove whitespace from both ends.
    $line =~ s|^\s*||;
    $line =~ s|\s*$/||;

    next if !$line;  # skip empty lines
    # print "Line: $line\n";

    my %object = ();  # someday: new $objectType;
    # Parse the lines into components.
    my @values = split /$delimiter/, $line;
    
    # Assign fields from values on line.
    my $i = 0;
    foreach $value (@values) {
	my $name = @fieldNames[$i];
	$object{$name} = $value;
#	print STDOUT "$name=$value\n";
	$i++;
    }

    # Get default values.
    $object{UserID} ||= $object{Email};
    $object{Email} ||= $object{UserID} && &addressOK($object{UserID});

    my %member = &getMember($object{UserID});

    # Maybe don't change at all if member already exists.
    next if %member && !$changeCurrentMembers;

    if (!%member) {
	$object{Status} = $initialStatus;
    }

    # Create a password if needed.
    my $randPassword;
    if (!$object{Password} && ($requirePasswords || $generatePasswords)) {
	$randPassword = &rand_passwd();
    }

    # Encrypt the password
    my $password = $object{Password} || $randPassword;
    my $cryptedpass = crypt_passwd($password);
    $object{Password} = $cryptedpass;

    print STDOUT <<EOT;
UserID: $object{UserID}  
 Email: $object{Email}
  Name: $object{FullName} 

EOT


    if (%member) {
	# Use existing values.
	my $field;
	foreach $field (keys %member) {
	    if ($overwriteCurrentValues) {
		$object{$field} ||= $member{$field};
	    }
	    else {
		$object{$field} = $member{$field};
	    }
	}
    }

    &putMember(%object);
}

 # Rebuild now, since passwords have been changed.

if ($rebuildNow) {
    &rebuildPasswordGroup();
}
