#!/usr/bin/perl
# batch-email-list.pl - return a list of email addresses of all members.
# Copyright HyperNews.org, 1998

# Written by Daniel LaLiberte for HyperNews.org.
#
# Use this program to extract a list of the email addresses of all members.



#################################
# Options - none yet.

# e.g. 
# List members who match a pattern.
# List members who have not used the system in a long time.
# List recent members.
# List invited members who have not yet joined.

# List other things about members.
# Change to batch-member-list.pl
# Merge with view-members.pl

#################################

BEGIN {
    # Prepend HyperNews directories to load path.
    $0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
    $hnbin =~ s{/(.scripts|Admin|SECURED)}{}g;
    unshift(@INC, $hnbin, "$hnbin/.scripts"); 
}

require 'Admin/hnrc';
require 'member-lib.pl';

$CACHE_MEMBERS = 0;  # Turn off caching.

#################################

&Main();

sub Main {

    # Get the members list
    opendir (DIR, "$hnPeople$hnGroup");
    my @files = readdir DIR;
    closedir DIR;

    # Select the members that match the goal.
#    if ($goal) {
#	@files = eval { grep (/$goal/i, @files); };  # ignore errors
#    }

    # Remove files not for members.
    my @members = ();
    foreach $member (@files) {
#	print "Checking: $member\n";

	if (&useridOK($member) &&
	    $member !~ m,-old$, &&
	    $member !~ m,-new$, &&
	    $member !~ m,^\., &&
#	    $member !~ m{\@} &&  email is OK
	    $member !~ m,^urc$,) {
#	    print "OK: $member\n";
	    push @members, $member;
	}
    }

    my @email_list = ();
    my %seen;
    my $member;
    foreach $member (@members) {
#	print "Getting: $member\n";
	my $email = &addressOf($member);
	next if !$email;
	if (!$seen{$email}) {
	    $seen{$email} = 1;
	    push @email_list, $email;
	}
    }
    
    @email_list = sort @email_list;

    foreach $email (@email_list) {
	print "$email" . "\n";
    }
}
