#!/usr/bin/perl
# batch-incoming-msgs.pl - Process a stream of incoming messages.
# Copyright HyperNews.org, 1998

# Written by Daniel LaLiberte for HyperNews.org

# The sequence of messages to be processed may come from a mail or usenet 
# archive, or a mail digest, or perhaps other streams.   Each message
# is added separately using mail-incoming.pl.

# The order of the messages should be partially ordered by date, so
# that newer messages always follow all older messages that they are
# in reply to.  This is so the messages may be properly threaded.
# This needs to be fixed by using placeholders somehow.

######################################################################
# The format of the message stream is a sequence of normal messages in
# mail or usenet format, each separated from the next by a string.
# The separator is stored in $messageSeparator.  Another separator
# frequently used is "\n\nFrom ".  Should be a command line option or
# other configuration method.  Or should maybe trust Content-Length.

my $messageSeparator = "\n========\n";

# The number of newlines in $messageSeparator.
my $numNL = 2;  # == $messageSeparator =~ tr{\n}{\n};  ??

# The stream comes in on standard input.  The whole input is not
# read at one time, but rather one message is read at a time,
# processed and discarded.

# The command for processing one message.  
# This should have command line options for the forum name.
my $forumName = "/test";
my $processMsgCommand = "/home/httpd/cgi-bin/HyperNews/.scripts/mail-incoming.pl -f $forumName";

#####################################################################

# We have to do some things with @INC in order for this to compile right.
BEGIN {
    # Prepend HyperNews directories to load path.
    $0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
    $hnbin =~ s{/(.scripts|Admin|SECURED)}{}g;
    unshift(@INC, $hnbin, "$hnbin/.scripts"); 
}
 
require "Admin/hnrc";

local $plaintext = 1;

# For debugging, we leave files HNprocess.out and HNprocess.old in /tmp.
$debug = 1;
if (0 & $debug) {
  rename "/tmp/HNprocess.out", "/tmp/HNprocess.old";
  open(DEBUG, ">/tmp/HNprocess.out");

  # Redirect STDOUT and STDERR to DEBUG
  open(SAVEERR, ">&STDERR");
  open(STDERR, ">&DEBUG") || die "Cant dup DEBUG";

  select(STDERR); $| = 1;     # make unbuffered
  select(DEBUG);  $| = 1;	 # make unbuffered

  $debugHandle = "main::DEBUG";
}


&feval('&MainPL()');

sub MainPL {
    my $line = <STDIN>;

    while ($line) {
	# print "Read: $line";

	# Accumulate lines for one message.
	my @lines = ();
	my @buffer = ();  # Last $numNL lines read
	my $eom = 0; # end of message flag

	while (!$eom) {
	    push @buffer, $line;
	    push (@lines, shift (@buffer)) if (@buffer >= $numNL);

	    # Check if we have reached the end of the message.
	    my $bufferLines = join ('', @buffer);
	    # print "Buffer: '$bufferLines'";
	    if ($bufferLines =~ m{ ^$messageSeparator$ }xs) {
		$eom = 1;
	    }
	    else {
		$line = <STDIN>;
		# print "Read: $line";

		if ($line eq '') {
		    #  end of file
		    $eom = 1;
		    push @lines, @buffer;
		}
	    }
	}

	# @lines contains the lines of the message.
	my $msg = join ('', @lines);

	# Print a couple of the lines
	if ($msg =~ m{^(From: .*)$ }xm) {
	    print STDERR "$1\n";
	}
	if ($msg =~ m{^(Message-ID: .*)$ }xm) {
	    print STDERR "$1\n";
	}
	if ($msg =~ m{^(References: .*)$ }xm) {
	    print STDERR "$1\n";
	}
	print STDERR "\n";

	# print "Command: $processMsgCommand";
	@processMsgCommandFields = split(' ', $processMsgCommand);

	open(MSG, "|-") || &do_exec_command(@processMsgCommandFields);

	print MSG $msg;
	close MSG;  # Wait for exec to finish?

	# Start in on the next message
	$line = <STDIN>;
    }

}

sub do_exec_command {
    my (@commandFields) = @_;
      # Redirect stdout and stderr of command to nowhere.  There
      # should only be output if the command complains.  If you need to
      # see this complaint, instead of /dev/null, you could write to
      # a file.  
      
      exec(@commandFields, '');

      &HNError ("Failed to exec command \"@commandFields\": $!");
}
