# change-member-lib.pl - utilities for membership and password management
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

require "member-lib.pl";
require "lock.pl";
require 'log-lib.pl';


sub resetPassword {
    # Reset password for $userid.
    # Parts of this should be moved back to reset-password.pl
    local ($userid, $remoteUser) = @_;

    my %user = &getMember($userid);
      
    if (!%user) {
	&HNUserError ("<P>User <strong>$userid</strong> is not known to $hnSystemName!<P> \n");
    }

    &logTransaction("Reset password of $userid");

      
      local($tempPassword) = &rand_passwd();
      local($cpassword) = 
	&change_password($userid, &crypt_passwd($tempPassword));
      
      $user{'Password'} = $cpassword;
      &putMember(%user);
      
      print "Password for <STRONG>$userid</STRONG> reset.";
      
      my $Email = &addressOf($userid);
      if ($hnEmail) {
	if (!$Email) {
	  print "<P><strong>User $userid does not have an email address.
Please inform $userid that his or her password is now: '$tempPassword'.
</strong>\n";
	  return 1;
	}
	
	require "email.pl";
	&sendmail(HYPERNEWS, $Email);

#	print HYPERNEWS "Errors-To: $mail_errors_to\n";
	
	my $editMsg = qq{
Change it to something you can remember using the form at:

  $SECUREDeditMemberURL?$userid
};
	
	if (0 && $joinSF == 3 && !&isAdmin($userid)) {
	  my $adminEmail = &addressOf(@hnAdmin[0]);
	  $editMsg = "Write to $adminEmail to change it.";
	}
	
	my $changedBy = "by '$remoteUser'" if $remoteUser;

	print HYPERNEWS "Subject: $hnSystemName Password Reset.

Your $hnSystemName password (for '$userid' at $http)
has been changed.

Your new password is: \"$tempPassword\"

$editMsg

Write to $adminEmail for assistance.
";
	
	close HYPERNEWS;
	
	print "<P><strong>The new password has been emailed to $Email.
</strong>\n";
      }
      
    return 1;
}



# This does some authentication and access control as well.

sub changeMember {
    # Store the member data for $userid
    # Return 1 if password was correctly entered, or if nothing was changed.
    local ($remoteUser, $userid, $authenticated, %form) = @_;
    # args need to be local

    # If $remoteUser is $authenticated, this could be 
    # the same or different from $userid.

    local $debug = 0;

    # $remoteUser = lc $remoteUser;
    # $userid = lc $userid;
 
    my $Password1 = $form{Password1};
    my $Password2 = $form{Password2};
 
    # %member needs to be local for checkChange.
    local (%member) = &getMember($userid);

    my $olduserid = $member{'UserID'};  # null if userid is new.

    &printAssoc("Member '$userid'", %member) if $debug;

    #===========================================

    my $Email = $form{Email};
    if ($Email && !&addressOK($Email)) {
	&HNUserError("The Email address \"$Email\" is not valid. 
 Nothing changed. Please back up and try again.");
    }

    $Email ||= $member{'Email'};  # Use existing email, if any.

    # Whether we need to create an email alias for userid.
    my $emailAliasNeeded;

    # Figure out if we are adding a nickname, 
    # and always alias from Email to userid if different.
    if ($Email) {
      # Always try looking up the Email address as an id.
      my %emailMember = &getMember($Email);
      my $emailUserid = $emailMember{'UserID'};
      &printAssoc("email member '$Email'", %emailMember) if $debug;
      
      if ($emailUserid) {
	&HNMsg("Check whether this user authenticates with $Email as the ID.")
	    if $debug;
	if ($authenticated) {
	    if (&isAdmin($remoteUser)) { goto OK; }  # admin are OK
	    if ($emailUserid eq $userid) { goto OK; } # Already aliased?

	    &HNMsg("Compare encrypted passwords:
'$member{Password}' and $emailMember{Password}'") if $debug;
	    if (!$member{Password} || 
		($member{'Password'} eq $emailMember{'Password'}))
	      { goto OK; }

	    &HNUserError("I cannot verify that you, '$remoteUser', 
 are permitted to change member info for <strong>$Email</strong>.  
 The password of $remoteUser is different from that of $Email,
 or perhaps someone else is 
 already using '$Email' as a $UserIDorHandle.  
 Ask an administrator to help you.");
	}
	elsif (&passwordMatch($Email, $form{Password})) 
	  { goto OK; }

	# Otherwise failure.

	# This may be the wrong message here.
	&HNUserError("You must supply a password to change the member
 info for <strong>$Email</strong>.");

      OK:
	$emailAliasNeeded = 1;
	if (!$olduserid) {
	  # $userid is new, so inherit info from email user info.
	  %member = %emailMember;
	  $member{'Email'} = $Email;
	  $member{'UserID'} = $userid; # But use the given userid
	  $olduserid = $emailUserid;  # Not really a new userid anymore.
	}
      }
      else {
	# Unknown email user.  Alias anyway.
	$emailAliasNeeded = 1;
      }
    }


    #===========================================

    # Compute where to send email.
    # Need to add old email address as well, if changing it.

    my $whoto = $Email || &addressOf($userid);
    &HNMsg("whoto = $whoto") if $debug;

    if ($hnEmail && $sendmail && $whoto) {
	require "email.pl";
	&sendmail(HYPERNEWS, $whoto);
    }
    else {
	# if no sendmail, throw it away.
	open (HYPERNEWS, ">/dev/null");
    }


    #===========================================

    # Check whether to generate password.
    # Should allow admin to override generation for other users.

    # Does $userid have a blank password?  Or is it a non-member?
    my $blankPassword = &passwordMatch($userid, '') || !%member;

    if ($generatePasswords && $blankPassword &&
	# Don't generate password if $userid is admin, 
	# but do even if $remoteUser is admin.
	!&isAdmin($userid)) {

	if (!$hnEmail || !$sendmail) {
	    &HNUserError("HyperNews configuration error: 
HyperNews is not configured to send you Email with your new password.  
Please notify the administrator that email must be enabled.");
	}
	if (!$whoto) {
	    &HNUserError("You cannot proceed until you provide
 an Email address since a new random password must be emailed to you.");
	}

	print "A new random password is being emailed to you.  
Use that password to change it to something you can remember. 
<P>
If you don't get the email message with the password, you must contact
an administrator to set the email address properly and to get a new password.
<P>
";

	if ($Password1) {
	    print "The new password you entered was ignored.<P>\n";
	}

	# Now generate the password.
	$Password1 = &rand_passwd();
	$Password2 = $Password1;
    }

    #===========================================

    if ($requirePasswords && 
	# Is there no password before or after?
	$blankPassword && !$Password1 &&
	# Don't require admin to provide a password yet,
	# either for themselves or other users.
	!(&isAdmin($userid) || &isAdmin($remoteUser))) {

	&HNUserError("Every member must have a non-blank password.
 Please back up and change your password at this time.");
    }

    #===========================================

    if ($Password1 ne $Password2) {
	&HNUserError ("The new passwords you entered DO NOT MATCH
 each other.  So nothing was changed.  Please back up and try again.");
    }

    #===========================================

    if ($olduserid) {
      # Existing member.
      print "Making changes to existing member '<B>$userid</B>'. <BR>\n";

      # Start email message
      print HYPERNEWS "Subject: Changes to your $hnSystemName membership info

  ***[Email replies to this message may be ignored.]***\n

Regarding your $hnSystemName membership info at:

  $SECUREDeditMemberURL?$userid

";

      if ($olduserid ne $userid) {
	print "<STRONG>$userid</STRONG> is previously known as 
 <STRONG>$olduserid.</STRONG><BR>";
	print HYPERNEWS "'$userid' is previously known as '$olduserid'.\n";
	if (!&addressOK($olduserid) && 
	    &useridOK($olduserid) &&
	    &addressOK($userid)) {
	  # In this case, the $userid is an email address and
	  # $olduserid is the preferred id.   
	  # This can happen if changing the userid to an email address.
	  print qq{
Since '$userid' is an Email address, 
your previous $UserIDorHandle, '$olduserid', will be used instead.
To change your Email address, don\'t change the $UserIDorHandle.
};
	  # The title of the result is misleading, but .. good enough.
	  # The URL sent out is also wrong - gotta fix it!!
	  $userid = $olduserid;	# Seems suspicious, but OK.
	  $member{'UserID'} = $olduserid;
	  $olduserid = '';  # Forget this ever happened.
	}
	else {
	}
      }

    }

    #========================================================
    # $userid is new

    else {
      $member{'UserID'} = $userid;
      &HNMsg("Registering new member: $userid") if $debug;
            
      &logTransaction("Register $userid"); 
      
      # Join member here.

      # Put out Subject header and start email message.
      print HYPERNEWS qq{Subject: $hnSystemName Membership

};

    # Need to make this check $plaintext var, to strip out HTML.  !!
	print "\n<P>Welcome to $hnSystemName. <P> \n";
	print HYPERNEWS "Welcome to $hnSystemName.\n\n";

	print "<BR>\n";

      if ($remoteUser) {
	print "\nYou were added by '$remoteUser'.\n <P>";
	print HYPERNEWS "\nYou were added by '$remoteUser'.\n";
      }

      if ($joinSF < 3) {
	  print qq{
 Changes can be made at any time by visiting:
<PRE>
  <A HREF="$SECUREDeditMemberURL?$userid">$SECUREDeditMemberURL?$userid</A>
</PRE>
<P>
};
	  print HYPERNEWS qq{
You can change your membership info by visiting the URL:

  $SECUREDeditMemberURL?$userid

};
      }
  
    }

    my $adminEmail;
    if (!&isAdmin($userid)) {
	$adminEmail = &addressOf(@hnAdmin[0]) || 
	  "the $hnSystemName administrator at $ENV{'SERVER_NAME'}";
	print qq{
 Write to <A HREF="mailto:$adminEmail">$adminEmail</A> for assistance. <P>
 };
	print HYPERNEWS "\nWrite to $adminEmail for assistance. \n";
    }
      

    if ($emailAliasNeeded && ($Email ne $userid) && $allowEmailUserID) {
	print qq{
<P>
 Note: Your $UserIDorHandle is "<strong>$userid</strong>",<BR>
       but you can also use your Email address: "<strong>$Email</strong>". <P>
 };
	print HYPERNEWS qq{
Note: Your $UserIDorHandle is "$userid",
but you can also use your Email address: "$Email".

};
    }

    if ($whoto && $hnEmail) {
	print qq{
 Note: If you don\'t get an email message regarding this change of your
 membership information, something is wrong with your Email address 
 or delivery to that address has failed for some other reason. <P>
 };
    }

    my %memberFields = &parseURCFile("$hnPeople/.member-fields");

    # Convert fields
    foreach $name (keys %memberFields) {
	$form{$name} =~ s{\r\n}{\n}g; # Convert PC's CR/LF into just LF.
	$form{$name} =~ s{\r}{\n}g; # Convert Mac's CR into LF.
        $form{$name} =~ s{\s+$ }{}gx;  # Remove trailing whitespace.
 	# $form{$name} =~ s{\n}{%0a}g;  # Encode LFs.
    }

 
   # ==============================================
    # Figure out if anything has changed.  Need to execute this always.
    local $somethingnew =
	(!$olduserid || ($form{UserID} ne $olduserid) ||
	 ($member{Status} eq 'invited') ||
	 # $emailAliasNeeded || # Always true if Email is set!!
	 # should make a loop out of this
	 ($form{Name} && ($form{Name} ne $member{'Name'})) || 
	 ($form{Email}  && ($form{Email} ne $member{'Email'})) ||
	 ($form{UserURL} && ($form{UserURL} ne $member{'UserURL'})) || 
	 ($form{Hide} && ($form{Hide} ne $member{'Hide'})) || 
	 ($form{Subscribe} && ($form{Subscribe} ne $member{'Subscribe'})) || 
	 ($form{SessionLength} && ($form{SessionLength} ne $member{'SessionLength'})) || 
#	 ($form{Content} && ($form{Content} ne $member{'Content'})) || 
#	 ($form{Format} && ($form{Format} ne $member{'Format'})) || 
	 $Password1);

    # Get more member fields.
    foreach $name (keys %memberFields) {
	my ($maxlength, $xsize, $ysize, $description) = 
	    split ('\|', $memberFields{$name});
	$somethingnew ||= ($form{$name} && ($form{$name} ne $member{$name}));
    }


    if ($olduserid) {
	if ($somethingnew) { 
	    print "There are some changes.<BR>\n";
	    print HYPERNEWS "\nThere are some changes.\n\n";
	    my $logaddress = " Email: $whoto" if ($whoto && $whoto ne $userid);
	    &logTransaction("Update $userid$logaddress");
	}
	else { 
	    print "Nothing was changed.<BR>\n"; 
	    print HYPERNEWS "Nothing was changed.\n"; 
	}
    }


    if ($form{Status} eq 'invited') {
	# They were specifically invited.
	print "Thank you for accepting our invitation.\n";
	print HYPERNEWS "Thank you for accepting our invitation.\n";
    }
    $member{Status} = 'registered';
    
    print "\n<UL>\n";


    if ($Password1 || !$olduserid || !$member{Password}) {
      # Special handling of password change or new user.
      if ($Password1) {
	if (!$member{Password}) {
	  print "<LI><B>Password has changed. </B>\n";
	  print HYPERNEWS 
	    qq{* Your Password is: "$Password1"\n};
        }
        else {
	  # Compare with old to see if it really did change!!
          print "<LI><B>Password has changed. </B>\n";
          print HYPERNEWS
            qq{* Password has changed to "$Password1"\n};
        }
	
	$member{'Password'} = 
	  &change_password($userid, &crypt_passwd($Password1));
      }
      else {
	# No new password; is there an existing password?
	if ($member{'Password'}) {
	  $member{'Password'} = 
	    &change_password($userid, $member{'Password'});
	}
	if (!$member{'Password'}) {
	    # This should happen each time the member changes info.
	    print "<LI><B> Warning: 
<FONT COLOR=red>$userid has blank password.</FONT></B>\n";
	    print HYPERNEWS "* Warning: $userid has blank password.\n\n";
	    # Save blank password anyway.
	    $member{'Password'} = '';
	    # Encrypt for password file.
	    &change_password($userid, &crypt_passwd(''));
	}
      }
    }


    # Check whether password change needed for $Email too.
    if (&checkChange("Email Address", 'Email', $Email)
	|| $Password1 || !$olduserid || !$member{Password}) {
      # We *could* also remove the htpassword for the old $member{'Email'}

      if ($Email && $allowEmailUserID && ($Email ne $userid)) {
	# We checked above whether we are allowed to change info for $Email.
	&change_password($Email, $member{'Password'});
	my $passwordOrNone = $member{'Password'} ? 
	  "the same password" : "NO password";
	print qq{
<LI><B> "$userid" and "$Email" have $passwordOrNone. </B>
};
	print HYPERNEWS 
	  qq{
* "$userid" and "$Email" have $passwordOrNone.
};

	print qq{
<P>
<B>Remember your $UserIDorHandle and Password.  You must enter them
when you wish to post messages or change your membership info.</B>
};

	print HYPERNEWS qq{
*** Remember your $UserIDorHandle and Password.  You must enter
*** them when you post messages or update your membership info.
};

	# There might be other aliases that need updating too.
	# But we only remember one - could cause problems.
	# Rebuilding the password file would fix that.
      }
    }
    

    if ($form{Subscribe} ne $member{Subscribe}) {
	my $freq = ($form{Subscribe} eq 'all') ? 'Immediate' :
	    (($form{Subscribe} eq 'none') ? 'Unsubscribe' : '');
	require 'subscribe-lib.pl';
	&replaceSubscription('', $userid, $freq);
    }

    print HYPERNEWS "\n";

    # Better description of changes needed.
    &checkChange("Name", 'Name');
    &checkChange("Personal home page URL", 'UserURL');
    &checkChange("Hide", 'Hide') if !$hideEmailAddresses;
    &checkChange("Subscribe", 'Subscribe');
    &checkChange("Session Length", "SessionLength") 
	if $UseCookies eq 'sometimes';
    # If SessionLength changes from something to none,
    # we should send a new cookie that expires immediately.
    # We should translate internal coded values to meaningful display.

    # &checkChange("Content option", 'Content');
    # &checkChange("Format option", 'Format');


    # Check more fields.
    foreach $name (keys %memberFields) {
	# Should probably display values with PRE, or with multi-line email.
	&checkChange($name, $name);
    }

    print "\n</UL>\n";

    if ($somethingnew) {
      &printAssoc("member URC", %member) if $debug;
      &putMember(%member);
    }

    # Add email alias if allowed and different from $userid.
    # We don't need to always do this, but figuring out when is tricky.
    # Need to for new users, or adding or changing email, or new nickname.
    my $emailAlias = (($userid ne $Email) ? $Email : '');
    # Add to Admin group, if appropriate, which also adds to non-Admin group.
    my $adminOrMemberGroup = &isAdmin($userid) ? "$hnGroup/Admin" : $hnGroup;

    &addtogroup($adminOrMemberGroup, ($userid, $emailAlias));

    close (HYPERNEWS); 

    return 1;
}

sub checkChange {
    # Do $member{$var} = $form{$var} if it is different, and return non-zero.
    # Better description of changes needed.
    my ($name, $var) = @_;
    my $val = $member{$var};
    my $newVal = $form{$var};
    &HNMsg("$name: $var=$val  new $var = $newVal") if $debug;

    if ($newVal && ($val ne $newVal)) {
	if ($val) {
	    print qq{
<LI> <B>$name</B> changed
<BR> &nbsp; from: \"$val\"
<BR> &nbsp; to: \"$newVal\"
};

	    print HYPERNEWS qq{
* $name changed 
  from: "$val" 
  to: "$newVal"
};
	}
	else {
	    print "<LI> <B>$name</B> is \"$newVal\"\n";
	    print HYPERNEWS "* $name is: \"$newVal\"\n";
	}
	$member{$var} = $newVal;
	return 1;
    }
    return 0;
}



#============================
# Group management and aliases.


sub makeUserAlias {
  my ($userid, $group, @aliases) = @_;
  # Make @aliases in $group be the same as $userid.

  &HNMsg("makeUserAlias($userid, '$group', @aliases)") if $debug;
  $userid = &untaintID($userid);
  $group = &untaintPath($group);

  my $dotdots = '';
  my $path = $group;
  while ($path =~ s,^/[^/]+,,) { $dotdots .= '../'; } # Count the '/'s

  my $alias;
  foreach $alias (@aliases) {
      #&HNMsg("$group, $alias, $userid") if $debug;
      if ($group || ($alias && $userid ne $alias)) {
	  # Link the alias to the userid file.
	  $alias = &untaintID($alias);
	  unlink ("$hnPeople$group/$alias"); # OK if not there.
	  if ($lower_case_IDs) {
	      $alias = lc $alias;
	      $alias =~ s{[_\s]+}{_}g if $allow_spaces_in_userids;
 	      unlink ("$hnPeople$group/$alias"); # OK if not there.
	  }

	  &HNMsg("symlink (\"$dotdots$userid\", \"$hnPeople$group/$alias\")")
	      if $debug;
	  # was: symlink ("$hnPeople/$userid", "$hnPeople$group/$alias");
	  symlink ("$dotdots$userid", "$hnPeople$group/$alias");
      }
  }
}


# We don't really need the server group files, and they just cause
# trouble and take more time.  So they are going away now.
# Instead, we can just check whether the member file is in the directory.
# See member-lib.pl.

# We still call addtogroup because it adds aliases to supergroups.

# The maximum number of characters in a single group line.
# Keep it relatively small, but not too small or the number of
# groups will grow too large.
# Workaround for bug in NCSA httpd 1.5.1: change to very large number.
# $MaxGroupLength = 50000;
$MaxGroupLength = 5000;

sub addtogroup {
    # Add user to a group
    local($groupName, $userid, @aliases) = @_;
    # groupName - the path of the group, relative to hnPeople
    # userid - to be added
    # aliases - others to also be added as aliases of userid.

    local($debug) = 0;
    &HNMsg("addtogroup(\"$groupName\", $userid, @aliases)") if $debug;

    return if (!$userid);

    # Make aliases for userid in this group.
    &makeUserAlias($userid, $groupName, $userid, @aliases);

    my ($supergroup, $subgroupName);

    # Check if groupFile is for a subgroup.
    if ($groupName =~ m,^(.*)/([^/]+)$,) {
	($supergroup, $subgroupName) = ($1, $2);
	# Recursive call to add user to supergroup first.
	# We need to do this because groups cannot have groups as members.
	# And because we want to share member info.
	&addtogroup($supergroup, $userid, @aliases);
	&HNMsg("Returning to $groupName") if $debug;
    }
    else {
	$subgroupName = "All";
    }

    # Do this here because we want the above makeUserAlias calls to be done.
    return if $externalSecurity; # Access control is managed outside HyperNews.

    # The rest is specific to the group files.
    return;

    local($namedivider);
    if ($serverType =~ m/CERN/) {
	# Same for group names as for user names.
	$namedivider = ", ";
    } else {
	$namedivider = " ";
    }
    my $dividersize = length($namedivider);

    my @grouplines;
    my $groupFile = "$hnPeople$groupName/$hnGroupMembers";

    if (-e $groupFile) {
	# It might not exist yet, but if it does, it should be readable.
	&HNMsg("Opening $groupFile") if $debug;

	open(FILE, "<$groupFile") || 
	    &HNError("Problems reading $groupFile");

	# Read all lines in the group file at once.
	@grouplines = <FILE>;

	&HNMsg("#group lines: $#grouplines") if $debug;
        close(FILE);
    }

    push @aliases, $userid;  # $userid is just another alias for the rest.

    my $line;

    # Find out if each userid is already in some line in the group.
    # We could avoid this outer loop if we had a linear list of all members.
    foreach $line (@grouplines) {
      my @newaliases = ();
      my $id;
      foreach $id (@aliases) {
	next if (!$allowEmailUserID && &addressOK($id));
	if ($id && $line !~ m/ $id($namedivider|\Z)/) {
	  push @newaliases, $id;
	}
      }
      @aliases = @newaliases;
      return if (!@aliases); # nothing left to add.
    }

    #============================
    # Remaining @aliases are not in the group file.
    # Rewrite the file adding @aliases.

    &HNMsg("New aliases: '@aliases'") if $debug;

    &lock($groupFile);

    open(NEWFILE, ">$groupFile$$") || 
	&HNError("Problems writing $groupFile$$");
    
    my @groups = ();   # list of numbered "subgroups" composing the one group
    
    # Write all but the last line.
    my $lastline;
    foreach $line (@grouplines) {
	# &HNMsg("group line: $line") if $debug;
	if ($line =~ m,^(.+):,) {
	    push @groups, $1;
	    &HNMsg("Got line for group: $1") if $debug;
	    if ($lastline) { print NEWFILE $lastline; }
	    $lastline = $line;
	}
    }

    my $linecount = @groups;

    # Check to see if the file needs a new line.
    if (!$lastline || length($lastline) > $MaxGroupLength) {
	# The line length is too large, or non-existent, so start new line.
	# First write last line.
	print NEWFILE $lastline if $linecount;
	$linecount++;
	&HNMsg("New group is $subgroupName$linecount") if $debug;
	@aliases = sort { length($a) <=> length($b) } @aliases;
	my $aliasesString = join($namedivider, @aliases);
	print NEWFILE "$subgroupName$linecount: $aliasesString\n";
	&HNMsg("Added '$aliasesString' on a new line") if $debug;

	# Update Access File with new Group
	push @groups, "$subgroupName$linecount";
	&createAccessFile($groupName, join($namedivider, @groups));
    }
    else {
	# User will fit on last line.
	# Need to sort all the entries on the line, as a workaround
	# for NCSA httpd bug.
	chop $lastline;
	$lastline =~ m,^([^:]+):\s*,;
	my $theGroupName = $1;
	my $membersString = $'; # The member list is everything else.
	my @members = split(/[ $namedivider]+/, $membersString);
	# Members might have a single null string, which should be ignored.
	push @members, @aliases;
	# Some older versions of Perl have problems in here somewhere.
	# @members comes out empty.
	@members = sort { length($a) <=> length($b) } @members;
	&HNMsg ("Members sorted: @members") if $debug;
	$membersString = join($namedivider, @members);
	&HNMsg("Adding '$membersString' at the end of line") if $debug;
	print NEWFILE "$theGroupName: $membersString\n";
	# There is no need to change the .htaccess file.
    }
    close NEWFILE;
    &moveItSafely($groupFile, "$groupFile-old", "$groupFile$$");
    &unlock($groupFile);
}


sub addtogroupBatch {
  # Add all users to a group file - used by setup.
  # This actually replaces the existing group file with the new list.
  local($groupName, @users) = @_;
  # groupName - the alias of the group to add
  # users - the list of users to be added
  
  local($debug) = 0;

  return if $externalSecurity; # Access control is managed outside HyperNews.
  
  # Sort the users by length, to workaround NCSA httpd bug.
  @users = sort { length($a) <=> length($b) } @users;
  
  local($supergroup, $subgroupName);
  
  # Check if groupFile is for a subgroup.
  if ($groupName =~ m,^(.+)/([^/]+)$,) {
    ($supergroup, $subgroupName) = ($1, $2);
    # makeUserAlias is not necessary here, because we only know they are
    # members since they are already listed in the directory.
    # &makeUserAlias($userid, $groupName, $userid);
    # Calling recursively for every subgroup will be expensive, and wrong.
  }
  else {
    $subgroupName = "All";
  }
  


  local($groupFile) = "$hnPeople$groupName/$hnGroupMembers";
  &lock($groupFile);
  
  open(FILE, ">$groupFile$$") || &HNError("Can't write to $groupFile$$");
  
  my $groupString = "";	# list of "subgroups" composing the one group
  
  &createAccessFile($groupName, $groupString);

  # The rest of this is for group files.
  return;


  local($linecount) = 0;
  local($linelength) = $MaxGroupLength + 1; # fake it.
  
  local($namedivider, $groupnamedivider);
  if ($serverType =~ /CERN/) {
    $namedivider = ", ";
  } else {
    $namedivider = " ";
  }
  local($dividersize) = length($namedivider);
  
  foreach $user (@users) {
    if ($user) {
      # Check to see if the file needs a new line.
      &HNMsg("Group File Size is $groupfilesize") if $debug;
      if ($linelength > $MaxGroupLength) {
	# If the file size is too large, start new line.
	print FILE "\n" if ($linecount);
	$linecount++;
	# Remember new group for access file.
	$groupString .= "$groupnamedivider$subgroupName$linecount";
	&HNMsg("New Group is $subgroupName$linecount") if $debug;
	$linelength = length("$subgroupName$linecount:");
	print FILE ("$subgroupName$linecount:");
	$groupnamedivider = $namedivider; # Switch to namedivider.
	# Append to the new line.
	$linelength += length($user) + 1;
	print FILE (" $user");
	&HNMsg("Added $user, on a new line") if $debug;
      }
      else {
	# Append to the last line.
	$linelength += ($dividersize + length($user));
	print FILE ("$namedivider$user");
      }
    }
  }
  print FILE "\n";
  close(FILE);
  &moveItSafely($groupFile, "$groupFile-old", "$groupFile$$");
  &unlock($groupFile);
  
  # Update the access file to list the groups. - not needed now.
  &createAccessFile($groupName, $groupString);
}


#=======================
# Access file management.

sub createAccessFile {
    local($groupName, $groupString) = @_;
    # groupName is the name of the group it corresponds to.
    # groupString is the string of all groups composing the group.
    # groupString is no longer used.
    local($debug) = 0;

    return if $externalSecurity; # Access control is managed outside HyperNews.

    local($filename) = "$hnPeople$groupName/$hnaccess";
    &HNMsg ("Writing access file: $filename...\n") if $debug;
    # &testReadWrite($filename, "file");

    &lock($filename);
    open (FILE, ">$filename")
	|| &HNError("Can't write to $filename");

    $hnRealm ||= "HyperNews$hnGroup";

    if ($serverType =~ m{Netscape}) {
        print FILE qq{
<Files *>
RequireAuth userfile=$hnPassword realm="$hnRealm"
</Files>
};
    }
    else {
	# Anything else - assume it is compatible with NCSA httpd.
	my $requireLine = "require valid-user";
        # was: "require group $groupString";

	# if ($serverType =~ /NCSA/)
	# Should also work for Apache (with dbm) and others.
	print FILE qq{
AuthUserFile $hnPassword
AuthName "$hnRealm"
AuthType Basic
 
<Limit GET POST>
$requireLine
</Limit>
};
    } 
    close(FILE);
    chmod("$hnfilemode", $filename);
    &unlock($filename);
}




#============================================================
# Password Routines - cleaned up by sparky@hypereality.co.uk
# rand_passwd	-	Generates a random password
# to64		-	Used by crypt_passwd
# crypt_passwd	-	Encrypt a password
# check_passwd	-	Encrypt a string with salt of a password for validation

# Modify this to create even more randomness.
srand(time() + $$);  # also add $^T ?

sub rand_passwd {
    local($temp,$pass);

    for($temp = 0; $temp < 4; $temp++) {
      $previous_rand = rand(32767);
      $pass .= &to64($previous_rand,2);
    }

    return($pass);
}

$itoa64 = 
  "23456789ABCDEFGHJKMNPQRSTUVWXYZabcdefghjkmnpqrstuvwxyz";
# Removed "./01oOiIlL" from the possible chars.
#  "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

sub to64 {
    my ($v,$n) = @_;
    my $salt;

    while (--$n >= 0) {
	$salt .= substr($itoa64, ($v % length($itoa64)), 1);
	$v >>= 6; 
    }

    return ($salt);
}



sub crypt_passwd {
    my ($passwd) = @_;
    my $salt;

    srand(time() + $previous_rand);
    $previous_rand = rand(32767);
    $salt = &to64($previous_rand, 2);

    return (crypt($passwd, $salt));
}


sub change_password {
   # Write a new password file while changing user's password.
    # It would be handy to change cookies here too, if appropriate,
    # but it is usually too late, and not strictly necessary.
   my ($user, $cpassword) = @_;

   return if !$user;
   return if $externalSecurity;  # Password file is managed outside HyperNews.

   # [jt] Added DBM code
   if ($PWFileDBM) {
     require 'passwd-dbm.pl';
     return change_dbm_password($cpassword);
   }

   &lock($hnPassword);
   open(FILE, "$hnPassword");
   open(FILE2, ">$hnPassword$$") || 
       &HNError ("Error on file open: $hnPassword$$");

   my $line;
   my $found;
   $user .= ":";
   do {
     $line = <FILE>;
     if ($line !~ m/^$user/) {
	 print FILE2 "$line";
     }
     else {
	 # Now print $user with new password.
	 print FILE2 "$user$cpassword\n";
	 $found = 1;
     }

   } while ($line);

   if (!$found) {
       print FILE2 "$user$cpassword\n";
   }

   close(FILE);
   close(FILE2);
   &moveItSafely($hnPassword, "$hnPassword-old", "$hnPassword$$");
   &unlock($hnPassword);
   # Make the password file always readable by all.
   # Group writable might be a problem for some though!!
   chmod (0664, $hnPassword);
 
   return($cpassword);
}


# Rebuild the password file from member files.
# Also rebuild top level email list.

sub rebuildPasswordGroup {

    return if $externalSecurity; # Access control is managed outside HyperNews.

    if (!$hnGroup) {
	# Rebuild password and group file.
	print &TextorHTML("<P><strong>
Rebuilding password file for \"root\" group</strong> ...");

	&testReadWrite($hnPassword, "file");

	print STDERR "Rebuilding passwords ...\n" if $debug;

	local($groupFileName) = "$hnPeople/$hnGroupMembers";
	# These files might not exist.
	unlink ("$groupFileName-old");
#	rename ($groupFileName, "$groupFileName-old");
#	&testReadWrite($groupFileName, "file");

	opendir (DIR, "$hnPeople");
	@files = readdir DIR;
	# @files should not include .htpasswd, etc.
	@files = grep (/^[^.]/, @files);
	closedir DIR;

	require 'lock.pl';
	&lock($hnPassword);
	open(FILE, ">$hnPassword$$") || 
	  &HNError("Cannot open $hnPassword$$");
	my $id;
	my @groupMembers = ();
	my %idSeen = ();
	my @subscribeAllList = ();

	foreach $id (@files) {
	    if (!$idSeen{$id}) {
		%members = ();  # Disable caching by getMember.

		my %member = &getMember($id);
		my $userid = $member{'UserID'};
		my $email = $member{'Email'};

		my $password = $member{'Password'};
		# Blank passwords need to be encrypted for .htpasswd file.
		$password = &crypt_passwd('') if !$password;

		# Dont include non-member files.
		# Dont include email addresses if not allowed as user IDs
		if (%member) {
		    print FILE ("$id:$password\n");
		    $idSeen{$id} = 1;
		    # push (@groupMembers, $id); 

		    if (($allowEmailUserID || !&addressOK($userid))
			&& !$idSeen{$userid}) {
			print FILE ("$userid:$password\n")
			    || &HNError("Cannot print to $hnPassword$$");
			# push (@groupMembers, $userid); 
		    }
		    $idSeen{$userid} = 1;
		    if (&addressOK($email) && ($userid ne $email)
			&& !$idSeen{$email}) {
			&makeUserAlias($userid, '', $email);
			if ($allowEmailUserID) {
			    print FILE ("$email:$password\n")
				|| &HNError("Cannot print to $hnPassword$$");
			    # push (@groupMembers, $email); 
			}
		    }
		    $idSeen{$email} = 1;
		    if ($member{Subscribe} eq 'all') {
			push (@subscribeAllList, $userid);
		    }
		}
	    }
	}

	close(FILE);

	&moveItSafely($hnPassword, "$hnPassword-old", "$hnPassword$$");
	&unlock($hnPassword);


	# Write the top-level list.
	&lock ("$hndocs/list");
	open FH, ">$hndocs/list";
	foreach $m (@subscribeAllList) {
	    print FH "$m:Immediate\n";
	}
	close FH;
	&unlock ("$hndocs/list");

	# Write all the group members at once.
	&addtogroupBatch("", @groupMembers);

	print &TextorHTML("<STRONG>OK</STRONG> \n");
    }
    #=================================
    # The rest is for group files.  Not needed in batch mode.
    return;

    if ($hnGroup) {
	print &TextorHTML("<P><strong>
Rebuilding group $hnGroup </strong>...");

	local($groupFileName) = "$hnPeople$hnGroup/$hnGroupMembers";
	# These files might not exist.
	unlink ("$groupFileName-old");
	rename ($groupFileName, "$groupFileName-old");
	&testReadWrite($groupFileName, "file");

	opendir (DIR, "$hnPeople$hnGroup");
	@files = readdir DIR;
	closedir DIR;
	@files = grep (/^[^.]/, @files);

	my @groupMembers = ();
	# Only include members who are also in group All.
	foreach $id (@files) {
	    %members = ();	# Disable caching by getMember.
	    my (%member) = &getMember($id);
	    my ($userid) = $member{'UserID'};
	    my $email = $member{'Email'};
	    if ($userid) {
		if ($allowEmailUserID || !&addressOK($userid)) {
		    push (@groupMembers, $userid);
		}
		if (&addressOK($email) && ($userid ne $email)) {
		    &makeUserAlias($userid, $hnGroup, $email);
		    if ($allowEmailUserID) {
			push (@groupMembers, $email); 
		    }
		}
	    }
	}
	# Write all the group members at once.
	&addtogroupBatch($hnGroup, @groupMembers);
	print &TextorHTML("<STRONG>OK</STRONG> \n");
    }
}


1;
