#!/usr/bin/perl -T
# change-member.pl - Action script to change member information of users.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

# This is used as the action for edit-member.pl.

# Add current directory to load path.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};
$0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
unshift(@INC, $hnbin, "$hnbin/.scripts"); 

require "Admin/hnrc";
&feval ('&changeMemberPL()');

sub changeMemberPL {

    require "urc.pl";
    require "formlib.pl";
    require "change-member-lib.pl";

    local $debug = 0;

    my %form = &GetPostArgs();  # Post only.
    &printAssoc('form', %form) if $debug;

    my $userid = $form{UserID};
    # Use the userid for the email address if it is known.
    # This is so we can change userids.
    my %email_member = &getMember($form{Email});
    my $email_userid = $email_member{UserID}; # Not useridOf($form{Email})

    # First remove leading and trailing spaces.
    $userid =~ s{^\s*}{};
    $userid =~ s{\s*$ }{}x;

    &HNMsg("userid: '$userid'") if $debug;

    # Don't check this here.  Check in change-member-lib.pl.
    # &check_userid_email($userid, $form{Email});  

    &checkAuthenticated($email_userid || $userid, $form{Password}, 
			$form{remoteUser}, $form{adminPassword});


    $commandFlag = $form{'Command'};
    # Delete a few things we don't need anymore.
    delete $form{Command};
    delete $form{Commandsubmit};
    delete $form{adminPassword};
    delete $form{remoteUser};

    my %member = %form;

#    $userid = $member{UserID} || $remoteUser;  # Is this right?

    $remoteUser ||= $userid || 'UnknownUser';

    &HNMsg("After authenticating<BR>
 remoteUser: $remoteUser<BR>
 userid: $userid") if $debug;
 

    # Collect requirements for the form data.
    $requirements = '';


    if (!$userid) {
	$requirements .= "<LI>You must enter a $UserIDorHandle.\n";
    }
    if ($userid =~ m{(Anonymous) | (Unknown) | (Nobody)}ix) {
	$requirements .= "<LI>That $UserIDorHandle, '$userid', is reserved.\n";
    }
    if ($userid && !&useridOK($userid)) {
	$requirements .= qq{<LI>The $UserIDorHandle, "$userid", 
is not acceptable.
It must start with a letter or digit and contain only letters, digits
or any of the characters '_.-'.
};
    }


    # Correct userids to latest version.
    $userid = &useridOf($userid);
    $remoteUser = &useridOf($remoteUser);

    &HNMsg("After normalizing<BR>
 remoteUser: $remoteUser<BR>
 userid: $userid") if $debug;
 

    my $remote_user_type;

# $authenticateInquiries - needs to be checked a bit differently.

    # This access control check is probably redundant with
    # what is done in changeMember.
    if ($commandFlag =~ m{Update|Register}) {
	$remote_user_type = 
	    &access1($joinSF, 2, $userid, $remoteUser,
		     "User '<strong>$remoteUser</strong>' 
 doesn't have permission to $commandFlag member info for
 <strong>'$userid'</strong>.
 (Or someone else is already using that $UserIDorHandle.)",
	       $authenticated, "", 1);
    }

#    if (($commandFlag ne 'Inquire') || $authenticateInquiries) {
#	my $joinOrUpdate = ($commandFlag eq 'Register or Update' 
#			    ? 'register' : 'update info');
#	if (!&access1($joinSF, 2, $userid, $remoteUser,
#		      "User <strong> $remoteUser </strong> 
# doesn't have permission $commandFlag for $userid.",
#		      $authenticated, "", 1, 1)) {
#
#	    &HNMsg("
#<H2>Error using $hnSystemName</H2>
#<B><FONT SIZE=+1>
#$userid is already registered.
#Use <A HREF=\"$SECUREDeditMemberURL\">Update Member</A> instead.
#</FONT>
#</B>
#");
#	    exit;
#	}
#    }
 
   &HNMsg("After access control<BR>
remoteUser: '$remoteUser'<BR>
userid: '$userid'") if $debug;

    if (!$userid) {
      &HNUserError("You must enter a $UserIDorHandle.  
Please back up and try again.");
    }

    if ($commandFlag eq 'Inquire') {
      # Inquire about User ID.  Maybe use viewMember here.
      print "Location: $editMemberURL?userid=$userid\n";
      print "\n";
      exit 0;  # done.
    }

    my %oldmember = &getMember($userid);
    &printAssoc('old member', %oldmember) if $debug;


    my $email = $member{Email};

    if ($email && !&addressOK($email)) {
	# Need a way to delete email address, or mark it as non-working.
	# Almost the same as unsubscribe all.

	$requirements .= qq{<LI>The Email address, "$email", is not
in the correct form.  
It must look like "login\@your.site.name".
};
    }

    my %restricted = &parseURCFile("$hnPeople/.restricted");
    &printAssoc('Restricted', %restricted) if $debug;
    if (%restricted && ($email =~ m{\@(.*)$ }x)) {
	# Check for restricted domains.
	my $domain = lc $1;
#	&HNMsg("domain: '$domain'");
#	&HNMsg("restricted{domain}: '$restricted{$domain}'  exists: " . 
#	       (exists ($restricted{$domain})));
	if (exists $restricted{$domain}) {
	    $requirements .= "<LI>We don't accept '$domain' addresses.\n";
	}
    }

    $member{Email} = $email;
    
    if (0) {
	if (!($oldmember{FirstName} || $member{FirstName})) {
	    $requirements .= "<LI>A First Name is required.\n";
	}
	if ($member{FirstName} && ($member{FirstName} !~ m,^[A-Za-z][A-Za-z],)) {
	    $requirements .= "<LI>Use your full first name please.\n";
	}

	if (!($oldmember{LastName} || $member{LastName})) {
	    $requirements .= "<LI>A Last Name is required.\n";
	}

	if (!($oldmember{Address} || $member{Address})) {
	    $requirements .= "<LI>An Address is required.\n";
	}

	if (!($oldmember{City} || $member{City})) {
	    $requirements .= "<LI>A City or township is required.\n";
	}

	$member{State} = '' if ($member{State} eq 'Select Your State/Province');

	if (!($oldmember{Country} || ($member{Country} ne 'Select Your Country Code.')
	      || $member{OtherCountry})) {
	    $requirements .= "<LI>A Country is required.\n";
	}
	$member{Country} = '' if ($member{Country} eq 'Select Your Country Code');

	if (!($oldmember{Zipcode} || $member{Zipcode})) {
	    $requirements .= "<LI>A Zipcode is required.\n";
	}

	if (!(length($oldmember{Phone}>=10) || length($member{Phone})>=10)) {
	    $requirements .= "<LI>A complete Phone Number with area code is required.\n";
	}
	if (!($oldmember{Phone} || $member{Phone})) {
	    $requirements .= "<LI>A Phone Number must be complete with area code.\n";
	}


	if (!$oldmember{UserID}) {
	    # New member
	    if(!$member{AgreeWithIt} || !$member{ReadIt}) {
		$requirements .= "<LI>You must read, understand, and agree 
with the <A HREF=\"\"> 
Terms and Conditions</A>.\n";
	    }
	}

    }				# end extras


    if ($requirements) {

	&print_HTML_HEAD("Incorrect Data");
	my $addingOrChanging = ($oldmember{UserID} ? 'changing' : 'adding');

	print qq{
<H3>Error while $addingOrChanging information for user $userid</H3>

<P>
<FONT SIZE=+1 COLOR=ff0000>
<B>You must correct some fields on the form before you can submit.</B></FONT>
<P>
<B><FONT COLOR=000FFF><U>
Use your Browser\'s BACK Button to go back and correct these entries.
</U></FONT></B>
<P> 
<FONT SIZE=+1 COLOR=ff0000>
<B>
<UL>
$requirements
</UL>
</B> 
</FONT>
<HR>
<P>
};
	&printEnd();
	exit 0;
}

    if (!$oldmember{Date}) {
	# Record date they registereded.
	require 'wwwdates.pl';
	$thistime = time;
	$date = &wwwdates::wtime($thistime, 'GMT');
	$member{Date} = $date;
    }

    $member{Hide} ||= 'Nothing';

    #-------------------------

    # make the update
    {
	# $debug = 0;
	&HNMsg("$joinSF, 2, $userid, $remoteUser, $password, $authenticated,") 
	    if $debug;

	# $authenticated ||= $remote_user_type > 1;  what is this?
	$registerOrUpdate = (!$oldmember{UserID} ? 'Registering New' : 'Updating');

	&print_HTML_HEAD("$registerOrUpdate Membership Info for $userid");

	print qq{
<H2>$registerOrUpdate Membership Info for 
 <A HREF="$SECUREDeditMemberURL?$userid">$userid </A>
</H2>
};
	$headerSent = 1;

	# Update user info.
	# This prints out stuff too:
	&changeMember($remoteUser, $userid, $authenticated, %member);

	&printEnd();
    }
}





