#!/usr/bin/perl -T
# edit-response.pl - generates a form for writing a response.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

# This is the action for the FORM at the end of HyperNews articles.

# Add current directory to load path.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};
$0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
unshift(@INC, $hnbin, "$hnbin/.scripts"); 

require "Admin/hnrc";
&feval('&editResponsePL');

sub editResponsePL {

    require "urc.pl";
    require "member-lib.pl";
    require "icon-lib.pl";
    require "edit-lib.pl";
    require "formlib.pl";

    local $debug = 0;

    my %form = &GetFormArgs();

    # Check for authentication.
    &checkAuthenticated($form{UserID}, $form{Password}, 
			$form{remoteUser}, $form{adminPassword});

    $URL = $ENV{'PATH_INFO'};

    my $thisURL = &URLencode($ENV{REQUEST_URI} || 
			     "$editResponseURL$URL");
    my $thisUserID = "userid=$remoteUser" if $remoteUser;
    my $redirectURL = "$loginURL?$thisUserID&url=$thisURL";

    my $isAdmin = $authenticated && &isAdmin($remoteUser, $base);
    $whoisthis = &sustring($remoteUser);
    if ($whoisthis) { 
	$whoisthis = qq{
Your message will be posted using the $UserIDorHandle:
'<STRONG>$remoteUser</STRONG>'.};
	if ($manualSecurity) {
	    $whoisthis .= qq{
If you wish to use a different $UserIDorHandle, you\'ll need to
<A HREF="$redirectURL">login</A> again.
<P>
};
        }
    }


    $remoteHost = "$ENV{'REMOTE_HOST'}";
    $remoteUser ||= ($ENV{'REMOTE_IDENT'} || $ENV{'FROM'} );
    #  || "unknown\@$remoteHost"
   
    $annotateOrRespond = $ENV{'QUERY_STRING'};


    if ($allowModeration) {
	$invisibleResponse = '
  <INPUT TYPE="hidden" NAME="visibility" VALUE="Invisible">';
    }
   
# Also need: $addOrEdit


    if (!$URL || $URL =~ m,^/[0-9],) {
	&HNError( qq{
 Congratulations! You have been bit by a rare HyperNews bug. <P>

 For some reason, the message you are trying to post would end up in a "null"
 forum with no name.  I want to find out how this is happening, but as
 of yet, I know of no pattern.  So please provide as much information
 as you can that might be relevant, including the data listed below.
 Mail to <A
 HREF="mailto:liberte\@hypernews.org">liberte\@hypernews.org</A>.  Here
 are a few questions. <P>

Thanks.
 
});
    }
    
    my %in; 

    # edit-response.pl is called as a form action
    if ($ENV{'REQUEST_METHOD'} eq 'POST') {
	%in = &GetFormArgs();
	$URL = $in{'URL'} || $URL;
	$baseFrameURL = $in{'baseFrame'};
	$frameString = $in{'frame'};
    }

    if ($frameString){
	&frameVariables();

	$frameURLOption = qq{
 <INPUT TYPE="hidden" NAME="baseFrame" VALUE="$baseFrameURL">};
	$frameOption = qq{
 <INPUT TYPE="hidden" NAME="frameString" VALUE="$frameString">};
    }

    $annotationType = $in{'annotationType'} || "Message";
    $annotationtype = lc($annotationType);

    # Parse the URC file before checking for access
    %urc = &parseURCFile("$hndocs$URL,urc");

    $upTitle = $urc{'Title'};
    $upURL = $urc{'URL'};
    $upURL = &convertref($upURL);   

    $base = $urc{'BaseURL'};
    $base = &convertref($base);
    my $baseDir = &url2hndir($base);

    if (!$authenticated && $addResponseSF > 1) {  # was also: $manualSecurity
	# Try logging in first.  Need to avoid login loop.
	&printRedirect($redirectURL);
    }

    &print_HTML_HEAD("Edit $annotationType to: $upTitle");
  
    if ($addResponseSF > 1) {
	&access1($addResponseSF, 2, '', $remoteUser, 
		 "User <strong> $remoteUser </strong> is not permitted
 to add a message.<p>",
		 $authenticated, $base);
    }


    # Inherit keywords from parent.
    $keywords = $urc{'Keywords'};


    if ($joinSF == 1 && !$authenticated) {
	# This doesnt make sense if edit-member.pl is restricted.
	$memberNotice = "If you are a frequent contributor to forums at
this site, you should <A HREF=\"$editMemberURL\">become a member</A>.";
    }
    
    $testMsg = qq{
If you just want to test this system, please do it on the <A
HREF="$hntestURL">Test Forum</A>.};

    &HNMsg("policy file: '$forumPolicyFile'") if $debug;
    $forumPolicyFile ||= '';  # see getDefaults;
    my $forumPolicy = '';

#    my $dir = $baseDir;
#    while ($dir && !$forumPolicyFile) {
#      # Find policy file, if any.
#      if (-e "$hndocs$dir/.policy.html") {
#        $forumPolicyFile = "$hndocs$dir/.policy.html";
#      }
#     # Chop off last component for the next round
#      $dir =~ s{/[^/]*$ } {}x;
#    }

    if ($forumPolicyFile) {
      # Read the policy statement, if any.
        open PH, $forumPolicyFile;
        my @policyLines = <PH>;
        close PH;
        $forumPolicy = join ('', @policyLines);
    }
 
    elsif  ($hntestURL =~ m{\Q$baseDir\E} && ($baseDir ne '/hypernews')) {
	$forumPolicy = qq{
This is a <strong>Test Forum for $hnSystemName</strong>.  All messages
in this forum will soon be <strong>deleted</strong>.  Only add test messages
that you do not care about preserving.  Please don\'t do tests in other forums.
}
    }

    elsif ($http eq 'http://www.hypernews.org' && $baseDir =~ m,^/hypernews,) {
	$forumPolicy = qq{
<FORM>
<STRONG>Is your message about HyperNews itself? </STRONG>
<INPUT TYPE="radio" NAME="fake_form" VALUE="yes">Yes
<INPUT TYPE="radio" NAME="fake_form" VALUE="no" CHECKED>No

<p> If you answered "No", then please <strong>DO NOT add your message
here</strong> - first find the appropriate forum (maybe go Back to
where you were previously) and add it there.  If you answered "Yes",
and your message is about "$upTitle", then please do continue.
</FORM>
} . $testMsg;
    }

    else {
	if ($instructionsURL && $hntestURL) {
	    $forumPolicy = qq{
Your message should be related to the subject named above. 
If it is not, please <strong>do not add your message here</strong>.
Instead, first find the appropriate page and add your message there.
} . $testMsg; 
	}
    }


    my $instructionsNotice;
    $instructionsNotice = qq{
If you are unfamiliar with $hnSystemName, please see the <A
HREF="$instructionsURL">
instructions</A> for details.  
} if $instructionsURL;


    print qq{
<H2> Edit $annotationType</H2>

You are adding a $annotationType to:
<STRONG>"<A HREF="$upURL">$upTitle</A>" </STRONG><P>

$forumPolicy<P>

$memberNotice<P>

$instructionsNotice<P>

<HR>

<FORM ACTION="$previewResponseURL$URL?$annotateOrRespond"
  METHOD="POST"  $targetBottom>

<INPUT TYPE="hidden" NAME="annotationType" VALUE="$annotationType">

$frameURLOption
$frameOption
$invisibleResponse
};

    # If you want to limit the formatting options, see messageFormats in hnrc.
    local $formatOption = "";

    {
	local $CHECKED = 'CHECKED';

	if (@messageFormats == 1) {
	    print qq{
<INPUT TYPE="hidden" NAME="contentType" VALUE="@messageFormats">
};
	}

	else {
	    # More than one possible format, so let user choose.
	    # Default is first one that we see below.
	    $formatOption .= qq{
<STRONG>Choose a format for your message:</STRONG> <BR>

<DL>
};
	}
	my %messsageFormats = ();
	foreach $f (@messageFormats) 
	{ $messageFormats{$f} = 1; }

	&printFormatOption('Smart Text', qq{
 Format paragraphs separated
 by blank lines.  If you begin any lines with spaces or tabs, 
 the whole paragraph will not be formatted.  Lines beginning with a 
 common prefix (for instance, <NOBR><TT>" > "</TT></NOBR>) are
 also unformatted.  URLs and email addresses will be made into links.
 });

	&printFormatOption('Word Processor', qq{
 Wrap long lines, break lines at
 hard Returns, preserve leading blanks and tabs, and use a
 fixed-width font.  URLs and email addresses will be made into links.
 });

	&printFormatOption('Plain Text', qq{
 Your formatting will not be changed. 
 Be sure to break up your lines with Returns.
 URLs and email addresses will be made into links.
 });
 
	&printFormatOption('HTML', qq{
 Enter any HyperText that goes between &lt;BODY&gt;
 and &lt;/BODY&gt;
});

	&printFormatOption('URL', qq{
    # URL format is not enabled by default.
 Enter only an http URL that points to an accessible
 HTML document.
 });

	$formatOption .= qq{\n</DL>\n} if (@messageFormats > 1);
        $formatOption .= "<P>\n";
     }


    #=================================================================
    print qq{
$whoisthis

};
    

    if ($authenticated) {
	if ($userid) {
	    print qq{
<INPUT TYPE="hidden" NAME="UserID" VALUE="$userid">
<INPUT TYPE="hidden" NAME="Password" VALUE="$form{Password}">
};
	}
	elsif ($remoteUser) {
	    print qq{
<INPUT TYPE="hidden" NAME="remoteUser" VALUE="$remoteUser">
<INPUT TYPE="hidden" NAME="adminPassword" VALUE="$remotePassword">
};
	}
    }

    if ($joinSF > 0 && (!$authenticated || $isAdmin)) {
	$emailid = " or Email address" if $allowEmailUserID;
#	print qq{
#<HR> <P>
#};

	if ($isAdmin) {
	    print qq{
<BR> (Since you are an Administrator, you may post a message for
another user - just enter the $UserIDorHandle or Email address below.)
<P>
 };

	}

	my $passwordOption = qq{
<STRONG>Password at $hnSystemName</STRONG>: <BR>
<INPUT NAME="Password" TYPE="password" SIZE=8 MAXLENGTH=8> <P>
} if !$isAdmin;


	my $ifNonMember;
	$ifNonMember = qq{
If you are <B>not</B> a <A HREF="$editMemberURL">member of $hnSystemName</A>,
please enter the following instead.<P>
	} if $joinSF > 0;

	my $nonMemberOption;
	$nonMemberOption = qq{
<HR>
$ifNonMember

<STRONG>Email Address</STRONG> (required if you want to be notified): <BR>
<INPUT NAME="Email" SIZE=45 MAXLENGTH=45 VALUE=""> <P>

<STRONG>Your name</STRONG> (optional): <BR>
<INPUT NAME="UserName" SIZE=60 MAXLENGTH=60> <P>
} if ($addResponseSF == 1);  # && !$authenticated  - don't do this?

    # Dont put $remoteUser in the UserID value field, it's not useful.

        print qq{
<STRONG>From</STRONG> ($UserIDorHandle$emailid, 
if you are a member of $hnSystemName):
<BR>
<INPUT NAME="UserID" SIZE=35 MAXLENGTH=65 VALUE=""> <P>

$passwordOption

$nonMemberOption

<HR> <P>
};

    }
    else {
    }


    print qq{
<STRONG>Title</STRONG> (describe your message, no HTML tags allowed): <BR>
<INPUT NAME="title" SIZE=60 MAXLENGTH=120> <P>

};

    if ($annotationType eq 'Reference') {
	print qq{
<STRONG>Reference URL</STRONG> (any URL): <BR>
<INPUT NAME="referenceURL" SIZE=60 MAXLENGTH=120> <P>
};
    }

    local($rows) = 25;
    if ($annotationType eq 'Reference') {
	$rows = 5;
    }

    print $formatOption;

    print qq{
<STRONG>Enter your $annotationtype here$onlyFormat: </STRONG> <BR>
<TEXTAREA NAME="body" ROWS=$rows COLS=62 WRAP="SOFT"></TEXTAREA>
<P>
<!--
<STRONG>Keywords</STRONG> (list of words or phrases): <BR> 
<INPUT NAME="keywords" SIZE=60 MAXLENGTH=120 VALUE="$keywords"> <P>
-->
};

    # Variables to specify an alternative format.
    $TABLE = '';  $TABLEEND = '';  # 'DL';
    $TR = ''; $TREND = '';
    $TD1 = ''; $TD1END = '';
    $TD2 = '<BLOCKQUOTE>'; $TD2END = '</BLOCKQUOTE>';

    print "$TABLE";

    if ($annotationType eq "Message") {
      # References need a different set of relations.
      my (@rels) = &ParseIconsFile($hniconsTable);

      # Filter out some rels here if you want.
      # @rels = ...

      my(%icons) = &parseURCFile($hniconsTable);
      # Add None if needed.
      if (!$icons{'None'}) { $icons{'None'} = $icons{'Note'}; }

      my (%relChecked) = ();
      $relChecked{'None'} = 'CHECKED';

      my ($relationOption);
      #$relationOption = "<SELECT NAME=\"UpRel\">" .
      # &buildOptions($icons[0], @icons) . "</SELECT>";

      my ($numRows) = (1+$#rels) / 4;  # How many rows in 4 columns?
      #&HNMsg("numRows: $numRows");
      my ($i) = 0;
      # Start first col.
      $relationOption .= "<TABLE>\n<TD><DL>\n";
      foreach $rel (@rels) {
	#&HNMsg("i: $i");
	if ($i < $numRows) { $i++; }
	else {
	  # Start a new col
	  $i = 1;
	  $relationOption .= "</DL></TD>\n<TD><DL>\n";
	  #&HNMsg("start new col");
	}
	#&HNMsg("add row $rel");
	$relationOption .= qq{
<DT>
<INPUT TYPE="radio" NAME="UpRel" VALUE="$rel" $relChecked{$rel}>
<IMG align=absmiddle border=0 src="$hniconsURL/$icons{$rel}">
$rel
</DT>\n
};
      }
      # Finish last col.
      $relationOption .= "</DL></TD>\n</TABLE>\n";
      
      if ($useHNIcons) {
	  print "
$TR
$TD1

<STRONG>Kind of Message</STRONG>:
$TD1END
$TD2
$relationOption
$TD2END
$TREND
";

#$TD2
#Select the appropriate relationship which 
#nwill be displayed with one of several
#<A HREF=\"$displayIconsURL\">icons</A>.
#$TD2END

      }

    }

    if ($hnEmailNotify) {
	if ($defaultNotification) {
	    $defaultNotification = "CHECKED";
	}
	else {
	    $defaultNotification = "";
	}

	print qq{
$TR
$TD1
<INPUT TYPE="checkbox" NAME="notify" VALUE="Notify" $defaultNotification> 
<STRONG>Notify me when people respond to this message:</STRONG>
<br>
$TD1END
$TD2
<font size="-1">
 (If checked, each reply to your message will be sent to you as email.)
</font>
$TD2END
$TREND
<p>
};
    }

    print "$TABLEEND\n";


    if ($allowDirectPost) {
	$subscriberMsg = qq{
Remember that this message 
will be sent to all the subscribers. 
} if $hnEmailNotify;

	$postOption = qq{
$TR
$TD1
<INPUT TYPE="submit" NAME="command" VALUE="Post your Message"><P>
$TD1END

$TD2
Posting your message will bypass the preview stage.  Only do
this if you are sure you are done editing. 
$subscriberMsg 
Don\'t forget to include a <i>Title</i>.<p>

Posting may take some time, so please be patient.  Please don\'t post
the same message more than once.
$TD2END

$TREND
};
    }

    print qq{
$TABLE
$TR
$TD1
<INPUT TYPE="submit" NAME="command" VALUE="Preview your $annotationType"><P>
$TD1END

$TD2 
Please preview your $annotationtype before posting. The preview
page shows your message the way it will appear to readers. You may
post your message from the preview page or return to this page to edit
it (use your browser <b>Back</b>
button to get back here), then preview again and send.
To cancel your $annotationtype, just don\'t submit it.  <P>
$TD2END
$TREND

$postOption

<INPUT TYPE="hidden" NAME="URL" VALUE="$URL"> 
$TABLEEND

</FORM>
};

    if ($instructionsURL) {
	print <<EOT;
<HR> <H3> |
<A HREF="$instructionsURL#Respond">
Help for $hnSystemName</A>
| </H3>
EOT
}
    &printEnd();
}


sub printFormatOption {
    my ($formatName, $description) = @_;
    if ($messageFormats{$formatName}) {
	if (@messageFormats > 1) {
	    $formatOption .= qq{
		<DD> <INPUT TYPE="radio" NAME="contentType" VALUE="$formatName" $CHECKED>
		    <B>$formatName</B>: 
		    };
	}
	$CHECKED = '';
	$formatOption .= $description;
    }
}

