#!/usr/local/bin/perl
# email.pl - utilities for emailing.

# The following code is modified from the Majordomo distribution.

# $sendmail = "/usr/lib/sendmail";  # Configured in hnrc

sub do_exec_sendmail {
    if ($sendmail) {
      # Redirect stdout and stderr of sendmail to nowhere.  There
      # should only be output if sendmail complains.  If you need to
      # see this complaint, instead of /dev/null, you could write to
      # a file.  The -oeq option should also be sufficient.
      # This causes some duplicate output for some reason.
      #open(SAVEOUT, ">&STDOUT");
      #open(SAVEERR, ">&STDERR");
      #open(STDOUT, ">/dev/null") || die "Can't redirect stdout";
      #open(STDERR, ">&STDOUT") || die "Can't dup stdout";
      
      # I'd like to put this in the background, so we don't have to
      # wait until sendmail is done before finishing the http request.
      exec(@_, "");

      &HNError ("Failed to exec mailer \"@_\": $!");
    }
}

sub sendmail {
    local($MAIL, $to, $from, $sender) = @_;
    # Open a sendmail process.
    # Caller may print more header lines to $MAIL, then a blank line,
    # then the body of the message.
    # Caller should then close the $MAIL filehandle.

    local $debug = 0;

    $from ||= $mail_from;
    $sender ||= $mail_sender;

    my $errorsAddress = '';
    if ($mail_errors_to) {
      $errorsAddress = "\nErrors-To: $mail_errors_to";
    }

    &HNMsg("Parameters to sendmail
            MAIL: $MAIL
            To: $to
            From: $from
            Sender: $sender") if $debug;    
    # force unqualified filehandles into callers' package
    local($package) = caller;
    $MAIL =~ s/^[^\']+$/$package\'$&/;

    # clean up the addresses, for use on the sendmail command line
    local(@to) = &ParseAddrs($to);
    for (@to) {
	$_ = join(", ", &ParseAddrs($_));
    }
    $to = join(", ", @to);
    &HNMsg ("\$to field is $to\n") if $debug;

    #if (!$allowEmailIn) {
    #  $mail_reply_to = $from;     This is redundant.
    #}
 
    local(@mailer, $mailer);

    $mailer = "$sendmail -t -f$sender";
    &HNMsg("mailer: $mailer") if $debug;

    @mailer = split(' ', $mailer);

    $| = 1;			# make output unbuffered - caller undoes this?

    local $debug = 0;
    # Do we need to redirect errors??  Yes, but how?
    open($MAIL, "|-") || &do_exec_sendmail(@mailer);

    # generate the header.
    &HNMsg ("mail sent to:$to\n") if $debug;

    print $MAIL 
"To: $to
From: $from$errorsAddress
Content-Type: text/plain; charset=iso-8859-1
";

    # Don't close $MAIL yet.  Caller puts out the rest of the message.
    return;
}


sub ParseAddrs {
  local($_) = shift;
  1 while s/\([^\(\)]*\)//g; 		# strip comments
  1 while s/\"[^\"]*\"//g;		# strip comments
  split(/,/);				# split into parts
  foreach (@_) {
    1 while s/.*<(.*)>.*/\1/;
    s/^\s+//;
    s/\s+$//;
  }
  
  @_;
}

1;
