#!/usr/local/bin/perl
#
#   bjellis perl library for www
#
#   copyright (c) 1994 Brigitte Jellinek
#   e-mail: bjelli@cosy.sbg.ac.at
#   URL:    http://www.cosy.sbg.ac.at/people/bjelli.html
# 
#   available under the GNU General Public License
#
#   Version: 0.0  -  Wed Dec 29 1993
#   Modified: by Daniel LaLiberte, liberte@hypernews.org
#
#   Description:  there subroutines that help you parse the information
#                 passed from a FORM
#
#   Usage:  call one of the subroutines to parse the information, then
#           read the resulte from %in: e.g. the enty in an input-field
#           with name 'color' can be found as $in{'color'}.  The total
#           number of argument found can be read from $noin.
#
#    &GetFormArgs: for both GET and POST
#    &GetPostArgs: for Method POST
#    &GetGetArgs:  for Method GET
#
#    &GetPathArgs: parses a string of the form name=value/name=value/...
#           
############################################################################

# Global var to return results.  Bad.  Also returns same result.
%in = ();

sub GetFormArgs
#
#  decides whether GET or POST was used, otherwise gives an error code
#
    {
    if (   $ENV{REQUEST_METHOD} eq 'POST' 
        && (($ENV{CONTENT_TYPE}  =~ m{^application/x-www-form-urlencoded})))
	# No file upload yet.
        {
	    return &GetPostArgs();
        }
    elsif ($ENV{REQUEST_METHOD} eq 'GET')   #    $#ARGV == -1  ??
	# && $ENV{QUERY_STRING}
        {
	    return &parseQueryString();
        }
    else
        {
        &HNUserError("This script needs to be called from a FORM,
                    either by POST or GET.");
        };
    return ();
    }

sub GetPostArgs
#
#  reads $ENV{CONTENT_LENGTH} bytes from STDIN, (assumed to be urlencoded)
#  and parses them with &parse
#
    {
    unless( $ENV{REQUEST_METHOD} eq 'POST' 
        &&  ($ENV{CONTENT_TYPE} =~ m{^application/x-www-form-urlencoded}))
          { 
	      &HNUserError("This script needs to be called from a FORM,
                    using method POST.");
          };

    # my $str;
    # This depends on an accurate CONTENT_LENGTH.  
    # read(STDIN, $str, $ENV{CONTENT_LENGTH});
    
    my @lines = <STDIN>;

    my $str = join ('', @lines);
    $str =~ s/^\s*//;  # Strip leading whitespace.
    $str =~ s/\s*$//;  # Strip trailing whitespace.
    &HNMsg("Read content ($ENV{'CONTENT_LENGTH'}): '$str'") if $debug;

    return &parse($str, '&');
    };

sub parseQueryString
#
#  reads the Arguments from the string $ENV{QUERY_STRING}, assumes that they
#  are urlencoded, and parses them with &parse
#
{
    my $str = $ENV{QUERY_STRING};
    if ($str) {
      return &parse($str, '&');
    }
    return ();
};

sub GetPathArgs
#
#  parses its one argument, a string of the from {/name=value}
#
    {
    my ($pathstring) = @_;
    $pathstring =~ s!^/!!;
    return &parse($pathstring,'/');
    };

sub parse
#
# parses urlencoded input its only argument (a string)
# returns values in the global variables %in and $noin
#
    {
    my ($argstring, $splitcharacter) = @_;
    my @args = split("$splitcharacter", $argstring);

    my %in = ();

    foreach (@args) {
	# The following parses a single name=value pair.
	# For browsers that don't encode \n, we have to allow
	# the value to contain \n.
      my $namval = $_;
      # &HNMsg("Parsing: '$_'");
	my $splitpos = index($namval, "=");
	if ($splitpos >= 0) {
	  my $nam = substr($namval, 0, $splitpos);
#	    $nam =~ s/^\s*//;  # Strip leading whitespace.
#	    $nam =~ s/\s*$//;  # Strip trailing whitespace.
	  my $val = substr($namval, $splitpos + 1);
	    # Decode the value.
	    $val =~ s/\+/ /g ;
	    $val =~ s/%(..)/pack('c',hex($1))/eg; # unescape characters
	    if (!$in{$nam}) {
		$in{$nam} = $val;
		&HNMsg("set in{$nam} = '" . &escapeHTML($val) . "'") if $debug;
	    } else {
		# Cons up a Lisp-like dotted list.
		my $oldval = $in{$nam}; # is this needed?
		$in{$nam} = [$val, $oldval];
		&HNMsg("cons in{$nam} = '" . &escapeHTML($val) . "'") if $debug;
	    }
	}
	else {
	    # No '='.
	    $in{$namval} = 1;
	};
    };
    # We are not using noin anyway.
    #@k = keys(%in);                # I couldn't find a better way of counting
    #$noin = $#k; $noin++;          # entries in %in.  do YOU know one?

    return %in;
    };



# The following don't belong in this file, but it is convenient for now.

sub frameVariables {
  
  if ($rFrame){
    $qsOfFrame = 'frame=response';
    $target = 'TARGET="responseFrame"';
  } elsif ($bFrame){
    $qsOfFrame = 'frame=base';
    $target = 'TARGET="baseFrame"';
  } elsif ($oFrame) {
    $qsOfFrame = 'frame=outline';
    $target = 'TARGET="outlineFrame"';
  }
  
  if ($rFrame || $bFrame || $oFrame) {
    
    $qsOfBaseFrame = "baseFrame=$baseFrameURL"
      if $baseFrameURL;
    
    $qsOfRFrame = 'frame=response';
    $resetFrame = 'frame=newset';
    $bottomFrame = 'frame=bottom';
    
    $targetB = 'TARGET="baseFrame"';
    $targetO = 'TARGET="outlineFrame"';
    $targetR = 'TARGET="responseFrame"';
    $targetBottom = 'TARGET="bottomFrame"';
    
    $targetSelf = 'TARGET="_self"';
    $targetParent = 'TARGET="_parent"';
    $targetTop = 'TARGET="_top"';
    $targetBlank = 'TARGET="_blank"';
  }
}


sub appendToQS {
  # Append each of the non-empty parameters to the query string.
  # If $qs is '&', just append.
  # If $qs is empty, start with '?'.
  my ($qs, @params) = @_;
  my $newqs, $param;
  for $param (@params) {
    if ($param) {
      # &HNMsg("param: '$param'");
      $newqs .= '&' if $newqs;
      $newqs .= $param;
    }
  }
  # &HNMsg("qs: '$qs'  newqs: '$newqs'");

  if ($qs eq '&') {
    $newqs = ('&' . $newqs) if $newqs;
  } elsif (!$newqs) {
    $newqs = $qs;
  } elsif ($qs) {
    $newqs = $qs . '&' . $newqs;
  } else {
    $newqs = '?' . $newqs;
  }
  return $newqs;
}

1;
