# hn-lib.pl - A few highly used HyperNews subs.
# Copyright Board of Trustees of the University of Illinois at UC, April 1996


# Hard code PATH, to avoid possibly tainted environment var.
$ENV{PATH} = '/bin:/usr/bin';

# The script to look up system info.  - should be in hnrc, or lookitup.
$uname = '/bin/uname';


sub repair_SCRIPT_NAME {
    # Use REQUEST_URI, if provided, to repair SCRIPT_NAME.
    # This is necessary for running under some versions of cgiwrap.
    if ($ENV{REQUEST_URI}) {
	my $qs = quotemeta $ENV{QUERY_STRING};
	$qs = ($qs ? "\\?$qs" : "\\??");
	# SCRIPT_NAME should be everything on URI before 
	# PATH_INFO and QUERY_STRING.
	if ($ENV{REQUEST_URI} =~ m{^(.*)\Q$ENV{PATH_INFO}\E$qs$ }x) {
	    $ENV{SCRIPT_NAME} = $1;
	}
	if ($ENV{SCRIPT_NAME} =~ m{/$ }x) {
	    chop $ENV{SCRIPT_NAME};
	    $ENV{PATH_INFO} = '/' . $ENV{PATH_INFO};
	}
    }
}

sub compute_hnprefix {
    # Compute $hnprefix from $ENV{SCRIPT_NAME}
    # which must be a URL prefix, not a filesystem path.
    if ($ENV{SCRIPT_NAME}) {
	$ENV{SCRIPT_NAME} =~ m{([^/]+$ )}x;
	my $file_name = $1; # Last component is the file name.
	$hnprefix = $ENV{SCRIPT_NAME};
	# Remove everything from first occurance of the file name to the end.
	$hnprefix =~ s{/$file_name.*$ }{}x;

	# If in a known subdir, remove that too.
	$hnprefix =~ s{/(.scripts|Admin|SECURED)}{}g;
    }
}


sub compute_hnSECUREDURL {
    $hnSECUREDURL = $hnbinURL;
    $hnAdminURL = $hnbinURL;
    if ($hnprefix =~ m{/cgiwrap}) {
	# Authenticated use of cgiwrap requires that it be inside an
	# access controlled directory.  So PATH_INFO after cgiwrap repeats
	# the path to get to the script. If not using ScriptAlias to
	# hide all this, we need to make it explicit here.
	$hnSECUREDURL =~ s{/cgiwrap}{/SECURED/cgiwrap};
	$hnAdminURL =~ s{/cgiwrap}{/Admin/cgiwrap};
	$cgiwrap = 'yes';  # signal to setup.pl
    }
    $hnSECUREDURL .= "/SECURED";
    $hnAdminURL .= "/Admin";
}

# flush.pl should be in your Perl library.
require "flush.pl";

# Things used only when debugging should be autoloaded.  E.g. Fatal 
# errors only happen once per execution, and so should be autoloaded.

#sub printflush {
#  my($FH, $string) = @_;
#  # Non flushing printflush
#  print $FH $string;
#}
#sub flush {
#  # Non flushing flush
#}



sub print_Content_type {
  local $last_cookie = '';
    if (!$headerSent) {
	$headerSent = 1;  # Remember that header is-being/has-been sent.
        &printflush ( STDOUT, "Content-type: text/html\n");
	if ($HNStatus) {
	  &printflush ( STDOUT, "Status: $HNStatus\n");
	  # &printflush ( STDOUT, "Realm: $hnRealm\n"); this is wrong.
	}
	if ($HNLastModDate) {
	  &printflush ( STDOUT, "Last-Modified: $HNLastModDate\n");
	}	  
	if (@HNOtherHeaders) {
	  &printflush ( STDOUT, (join "\n", @HNOtherHeaders), "\n" );
	}
	&printCookie($HNUserPassCookie, $CookieLifespan);
        &printflush ( STDOUT, "\n");  # Extra blank line.
	# Now the content can follow.

	#local $debug = 0;  # = 1 to see the last cookie that was sent.
	if ($last_cookie) {
	  &HNMsg("Sent Cookie: $last_cookie") if $debug;
      }
    }
}

sub printRedirect {
    my ($URL) = @_;
    # This must be the first output, otherwise it won't work.
    # Permanent redirect status: Show $URL.
    print "Status: 302\n";
    &printCookie($HNUserPassCookie, $CookieLifespan);
    # Need to indicate non-cachability of this redirect.
    print "Cache-Control: must-revalidate, maxage=0\n";
    print "Location: $URL\n";
    &printflush ( STDOUT, "\n");  # Extra blank line.

    if ($headerSent) {
	# Too late for redirect, but end output anyway.
	&printEnd();
    }
    exit 0;  # done.
}


sub printCookie {
  my ($cookie, $max_age) = @_;
  # Print the $cookie with $max_age relative expiration time, in days.
  return if !$cookie;

#  &HNMsg("printing cookie: $cookie  max-age: $max_age") if $debug;
  my $path_attr = 'path=' . ($hnrootprefix || $hnprefix) . '/';

  # Compute string for max age (expiration).
  # Absolute time uses 'expires'.  Relative time uses 'max-age'.
  my $expire_attr = '';
  my $max_age_attr = '';
  if ($max_age) {
    my $time = time;
    my $max_age_seconds = $max_age *   3600 * 24;
    $time += $max_age_seconds;
    require 'wwwdates.pl';
    $expire_attr = 'expires=' . &wwwdates::cookie_time($time);
    #  $max_age_attr = 'Max-Age=' . $max_age_seconds;
  }
  # my $version_attr = 'Version="1"';
  my $av_pairs = join (' ;', 
		       ($cookie, ($expire_attr), $path_attr));

#  &HNMsg("Cookie pairs: $av_pairs") if $debug;
  # Print the set-cookie now.
  &printflush ( STDOUT, "Set-Cookie: $av_pairs\n");
  # Prevent caching of set-cookie header.
  &printflush ( STDOUT, "Cache-control: no-cache=\"set-cookie\"\n");

  $last_cookie = "Set-Cookie: $av_pairs";
}

sub print_HTML_HEAD {
    my ($title, $other) = @_;
  
    &print_Content_type ();
    if (!$HTML_HEAD_SENT) {
      $HTML_HEAD_SENT = 1;

    $other .= qq{
<SCRIPT LANGUAGE="JavaScript">
<!-- hide this
function help(message) {
  self.status = message;
  return true;
}
// stop hiding -->
</SCRIPT>
};

      require 'hnview-lib.pl';
      &printHeadStuff($title, $other);
    }
    else {
	# Too late for the title
	print &TextorHTML ($other);
    }
}


#-------------------------
# Untainting routines.

sub untaintPath {
    my ($path) = @_;
    # Encode any weird chars.
    $path =~ s{[^\w\-\.\,/:=+\~]} {'%' . sprintf('%2x', ord($&))}xeg;

    $path =~ s{(^|/)[^/]+/\.\.}{}g; # Collapse all 'dir/..'.
    &HNUserError ("No relative paths allowed: '$path'") 
	if $path =~ m,(^|/)\.\.,;  # if any '..' remains
    $path =~ m{.*};  # Final untaint of the rest.
    $path = $&;
    return $path;
}

sub is_tainted {
    # from perlsec
    return ! eval {
	join('',@_), kill 0;
	1;
    };
}

sub ErrorIfTainted {
    &HNError("Tainted values: '@_'") if &is_tainted(@_);
}

#----------------------------------
# For Trapping Errors - used to call the main function of each script.
sub feval {
  my ($parameter) = @_;
  eval($parameter);
  $errorMsg = &escapeHTML($@);
  if ($@) {
      &HNError ("<PRE>Error evaluating: $parameter
$errorMsg
</PRE>");
  }
}


#----------------------------------
# For converting URLs

sub url2hnpath {
    my ($url) = @_;
    # Convert full 'get' URL to hypernews full path, including the '.html'.
    # It is generally safe to do this more than once.
    $url =~ s{^.*/get(\.cgi)?/}{/};
    return $url;
}

sub url2hndir {
    my ($url) = @_;
    # Return the directory name for the URL.
    $url = &url2hnpath($url);
    $url =~ s{\.html$}{},;
    return $url;
}

sub getBaseURL {
    my ($url) = @_;
    # Return the path of base article of $url, without the .html
    # It may or may not have the http://.../get prefix.
    $url =~ s,(/[0-9]+)*.html$,,;
    return $url;
}


sub convertref {
   my ($str) = @_;
   # Convert a string containing a HN 'get' URL to use the $getURL prefix.
   # The $getURL is derived from what the user enters
   if ($str =~ m|/get(\.cgi)?/|) {
       $str = "$getURL/$'";
   }
   return($str);
}


#--------------------------------
# Escaping and encoding strings to be safe in HTML docs.

sub escapeHTML {
    my ($string) = @_;
    $string =~ s/\&/\&amp;/g ;  # do this first
    $string =~ s/</\&lt;/g ;
    $string =~ s/>/\&gt;/g ;
    $string;
}

sub HTMLize_quotes {
  my ($string) = @_;
  # Return $string with double quotes converted to &quot; entities.
  # String will be used inside attribute values. e.g.  NAME="string"
  $string =~ s{\"}{\&quot;}g;
  return $string;
}

sub URLencode {
   my ($string) = @_;
   # Encode chars that might cause confusion in a URL or form field.
   # Do something like encoding for a FORM.
   $string =~ s{[^\w]} {'%' . sprintf('%2x', ord($&))}xeg;

   # Used to be:
# This is missing several things e.g. the '+' stuff.
#   $string =~ s/%/%25/g;  # First do this.
#   $string =~ s/\"/%22/g;
#   $string =~ s/\&/%26/g;
#   $string =~ s/</%3c/g;
#   $string =~ s/>/%3e/g;

   return($string);
}


sub URLdecode {
    # decode the URL encodings.
    my ($val) = @_;
    # $val =~ s/\+/ /g ; not this though
    $val =~ s/%(..)/pack('c',hex($1))/eg; # unescape characters
    return ($val);
}



#--------------------------------------------------
# Debugging
#
# The rest of this file should be autoloaded.

$TextHR = "-" x 72 . "\n";

#    if ($setupMode ne "web") {

sub TextorHTML {
    my ($HTMLstring) = @_;
    if ($plaintext) {
	# Just remove HTML tags for plain text display.
	# We could instead *render* the HTML in plain text.
        $HTMLstring =~ s/<[pP]>/\n/g;
        $HTMLstring =~ s/<[hH][rR]>/\n$TextHR/g;
	$HTMLstring =~ s|</?[^>]+>||g;
	# Need to remove SCRIPT tags entirely.
	# Remove extra spaces and leading spaces.
        # $HTMLstring =~ s|  | |g;
        # $HTMLstring =~ s|^ *||;
    }
    return "$HTMLstring\n";
}


$errorInMemberLib = 0;  # Set this if an error is occuring in member-lib.pl

sub DumpUserInfo {
    my ($text) = @_;

    my ($errno) = $!;

    # print ("plaintext: $plaintext\n");

    $hnSystemName ||= 'HyperNews';

    if (!$plaintext) {
      &print_HTML_HEAD ("Error using $hnSystemName");
    }

    print &TextorHTML ("<H3>Error using $hnSystemName</H3>\n");

    if (!$errorInMemberLib && $hnAdmin[0]) {
	require 'member-lib.pl';
	# If an error occurs during getMember, 
	# the next line will cause an infinite loop. !!!
	my(%admin) = (defined &getMember ? &getMember($hnAdmin[0]) : '');
	my($adminEmail) = $admin{'Email'} || $hnAdmin[0] ||
	  $ENV{'SERVER_ADMIN'};
	if ($adminEmail) {
	    print &TextorHTML ("
If you need help, please contact the administrator of
$hnSystemName: <A HREF=\"mailto:$adminEmail\">$adminEmail</A>.  <BR>
Please include the following information.
<HR> "); 
	  } 
      }

    print &TextorHTML (qq{
<FONT COLOR="red" SIZE="+2">
$text
</FONT><P>
});

    # Usually this is the wrong error message: "No such file or directory"
    # print &TextorHTML ("<B>Error: $errno</B><P>") if $errno;

    &HNENV();
}


sub HNENV {
  my $qstring = $ENV{'QUERY_STRING'} ? "?$ENV{'QUERY_STRING'}" : '';
  my $URL = "$http$ENV{'SCRIPT_NAME'}$ENV{'PATH_INFO'}$qstring";
  my $remoteUserString = $ENV{'REMOTE_USER'} || $remoteUser || '[Unknown]';
  $remoteUserString .= " (Authenticated $authenticated)" if $authenticated;
  my $operatingSystem = `$uname -a`;
  chop $operatingSystem;
  my ($user_name, $group_name) = &user_group_names();

  # Careful - don't call getMember unless already defined.
  my(%admin) = (defined &getMember ? &getMember($hnAdmin[0]) : '');
  my($adminEmail) = $admin{'Email'} || $hnAdmin[0];

  print &TextorHTML ("<TABLE WIDTH=100% BGCOLOR=lightyellow BORDER=0 CELLSPACING=0>");

  foreach $pair
    ((['URL', $URL], 
      ['Request URL', $ENV{REQUEST_URI}],
      ['HTTP User Agent', $ENV{'HTTP_USER_AGENT'}],
      ['Remote User', $remoteUserString],
      ['Server Software', $ENV{'SERVER_SOFTWARE'}],
      ['Server OS', $operatingSystem],
      ['Server User:Group', "$user_name:$group_name"],
      ['Server Administrator', $ENV{'SERVER_ADMIN'}],
      ['Perl Version', $] ],
      ['HyperNews Version', $hnversion],
      ["HyperNews Administrator", $adminEmail])) {
      my ($name, $value) = @$pair;
      $value ||= '[Unknown]';
      print &TextorHTML("  <TR><TD><b>$name</b>: </TD><TD>$value</TD></TR>\n");
    }

  print &TextorHTML ("</TABLE><P>\n");

}



sub printAssoc {
    my ($string, %assoc) = @_;
    # Print $string followed by the associative array, or hash table, %assoc.
    my $key;

    if ($plaintext) {
	print "$string:\n";
	foreach $key (sort(keys %assoc)) {
	    print "'$key' = '$assoc{$key}'\n";
	}
    }
    else {
        &print_HTML_HEAD("HyperNews Debugging Message");
	print  ("<p>$string: 
<TABLE WIDTH=100% BGCOLOR=lightyellow BORDER=0 CELLSPACING=0>\n");
	foreach $key (sort(keys %assoc)) {
	    print (qq{
  <TR><TD>'<B>$key</B>'</TD> <TD>= '$assoc{$key}'</TD></TR>
});
	}
	print ("</TABLE><P>\n");
    }
}


sub HNMsg {
    my ($text) = @_;

    if (!$plaintext) {
	&print_HTML_HEAD("$hnSystemName Message");
	$text =~ s{\s*$ }{<BR>}sx;
	print "
<TABLE WIDTH=100% BGCOLOR=lightyellow><TR><TD>$text</TD></TR></TABLE>\n";
    }
    else {
	print "\n" . &TextorHTML($text);
    }

    &flush (STDOUT);
    # Printing to STDERR should not be necessary.
    # &printflush (STDERR, "$text\n") if $debug;
}



sub HNUserError {
    # The user has screwed up.  Dump enough for user or admin to figure out.
    &DumpUserInfo(@_);
    require 'hnview-lib.pl';
    &printEnd() if (!$plaintext);
    &flush(STDOUT);
    exit 1;
}


sub HNError {
    # This is an internal error - dump out more stuff.

    &DumpUserInfo(@_);

    &tracedump();

    require 'urc.pl';  # It might be a problem to require this.
    &printAssoc('ENV', %ENV);

    require 'hnview-lib.pl';
    &printEnd() if (!$plaintext);
    &flush(STDOUT);
    exit 1;
}


#sub debug_var {
#    my ($var) = @_;
#    my ($package, $filename, $linenumber,
#	$subroutine, $hasargs, $wantargs) = caller(1);
#    $filename =~ s{^([^/]*/)*}{};  # Remove all but last component.
#    
#    &HNMsg("$package\::$subroutine in $filename at $linenumber: \$$var: $val);
#}

sub tracedump {
    my ($text) = @_;
    print &TextorHTML ("<P><B>Traceback: $text</B><P>\n");

    my $depth = 0;
    while ($depth++ < 10) {
	my ($package, $filename, $linenumber,
	    $subroutine, $hasargs, $wantargs) = caller($depth);
	return if !$filename;
	print &TextorHTML (qq{<PRE>
<b>Calling</b>: $subroutine
<b>In File</b>: $filename
<b>At Line</b>: $linenumber
</PRE>});
    }
}


sub user_group_names {
   my $gid = $); # $EGID
   my @groupids = split(/ /, $gid);
#   &HNMsg("groups: @groupids"); 
   my $groupid = $groupids[0]; # what about multiple groups
#   &HNMsg("groupid: '$groupid'");
   my $group_name = getgrgid ($groupid) if $groupid;   #broken?
   my $user_name = getpwuid ($>);  # $EUID
   return($user_name, $group_name);
}


1;
