# Library of view related functions.
# required in hn-lib.pl, sub print_HTML_HEAD.


sub printHeadStuff {
    my ($title, $other) = @_;

    &getDefaults($base);

    return if $plaintext;

    $bodytag ||= '<BODY>';

    print qq{
<HTML>
<HEAD>
<TITLE>$title</TITLE>
$other
</HEAD>
$bodytag
};

    if (!$rFrame && !$oFrame) {
      &printURLorText($headerURL || $headerText, $headerBase);
    }

    if (@parentForums) {
      shift @parentForums;
	$parentsString = join(": \n", reverse @parentForums);
	print "\n <H3>\n";
	print $parentsString;
	print " </H3> \n";
    }

}

@siblingForums = ();  # list of sibling forums.
@childForums = ();  # list of child forums.

sub printEnd {
  # Print end of HTML document.


  if ((@siblingForums > 1) || @childForums) {
    print "<P> <HR>\n";
    
    
    # Print list of sibling forums, if any.
    if (@siblingForums > 1) {
      print "\n Peer Forums: [";
      print join("] \n[", @siblingForums);
      print "] <BR>\n";
    }
    
    # Print list of child forums, if any.
    if (@childForums) {
      print "\n Subforums: [";
      print join("] \n[", @childForums);
      print "] <BR>\n";
    }
    
    print "<HR> <P>\n";
  }

  if (!$rFrame && !$frameHead && !$oFrame) {
    &printURLorText($footerURL || $footerText, $footerBase);
  }

#    &printURL($footerURL, $footerBase) ||
#      ($footerText =~ m,^http://, && &printURL($footerText)) ||
#	($footerText && print $footerText);

  if (!$plaintext) {
      print qq{
</BODY>
</HTML>
};
  }

}


sub getDepthDefaults {
    # Check if there is inlineDepth. If not, then use
    # the forum's defaultInlineDepth, or system wide value.
    $defaultInlineDepth = $baseArticle{'DefaultInlineDepth'} ||
	$globalDefaultInlineDepth;
    &HNMsg("defaultInlineDepth is: $defaultInlineDepth") if $debug ;
    $inlineDepth = $defaultInlineDepth if !defined $inlineDepth;
    &HNMsg("inlineDepth is: $inlineDepth") if $debug;

    # Check if there is outlineDepth. If not, then use
    # the base article defaultOutlineDepth, or system wide value.
    $defaultOutlineDepth = $baseArticle{'DefaultOutlineDepth'} || 
	$globalDefaultOutlineDepth;
    &HNMsg("defaultOutlineDepth is: $defaultOutlineDepth") if $debug ;
    $outlineDepth = $defaultOutlineDepth if !defined $outlineDepth;
    &HNMsg("outlineDepth is: $outlineDepth") if $debug;
}

  
sub getDefaults {
    # Climb up forums looking for defaults until root is reached.
    # Remember the first headerURL, footerURL, and searchURL found.
    # Also remember the baseURL for the header and footer.
    # Accumulate list of parent forums while climbing up.
    my ($base) = @_;
    # return if !$base;

    my $dir = &url2hndir($base);
    local $debug = 0;

    @siblingForums = &getSiblings($dir);
    @childForums = &getChildren($dir);
  
    @parentForums = ();
    $forumPolicyFile = '';

    while ($dir) {
	&HNMsg("dir: $dir") if $debug;
	my (%urc) = &parseURCFile("$hndocs$dir.html,urc");
	&printAssoc("urc", %urc) if $debug;

	if (%urc) {
	    if ($displayParents) {
		my $parentURL = &convertref($urc{'URL'});
		my $shortTitle = $urc{'ShortTitle'} || $urc{'Title'};
		push @parentForums, "<A HREF=\"$parentURL\">$shortTitle</A>";
	    }

	    if (!$headerURL && $urc{'HeaderURL'}) {
		$headerURL = $urc{'HeaderURL'};
		# URL might be relative, so remember base.
		$headerBase = ($headerURL =~ m,^((http:)|/),) ? '' : $dir;
	    }
	    if (!$footerURL && $urc{'FooterURL'}) {
		$footerURL = $urc{'FooterURL'};
		$footerBase = ($footerURL =~ m,^((http:)|/),) ? '' : $dir;
	    }

	    if (!$bodytag && $urc{'BodyTag'}) {
		$bodytag = $urc{'BodyTag'};
		&HNMsg("Found bodytag: $bodytag") if $debug;
	    }

	    # &HNMsg("SearchURL: $urc{'SearchURL'}");
	    # Assume this is absolute or rooted.
	    $searchURL ||= $urc{'SearchURL'}; 
	    # $searchBase too?

	    &HNMsg("Checking for policy at: $hndocs$dir/.policy.html") 
	      if $debug;
	    if (!$forumPolicyFile && -e "$hndocs$dir/.policy.html") {
	      &HNMsg("Found policy file") if $debug;
	      $forumPolicyFile = "$hndocs$dir/.policy.html";
	    }
	}

	# Chop off last component for the next round
	$dir =~ s,/[^/]*$,,;
    }

    # Use global defaults, if needed.
    if (!$headerURL) {
      $headerURL = $headerText;
      $headerBase = '';
    }

    if (!$footerURL) {
      $footerURL = $footerText;
      $footerBase = '';
    }

    $bodytag ||= $global_body_tag;
    &HNMsg("Use global body tag: $global_body_tag") if $debug;

    $searchURL ||= $globalSearchURL;  # Default is in hnrc.
  # Default header and footer are headerText and footerText in hnrc.
}


sub getSiblings {
    # Get sibling forums list
    my ($dir) = @_;
    my $parentDir = $dir;
    $parentDir =~ s,/[^/]*$,,;
    local $debug = 0;

    @forumList = ();

    if ($displaySiblings) {
	&HNMsg("building sibling list for '$dir'") if $debug;
	if (-e "$hndocs$dir/.sibling-forums") {
	    &HNMsg("Found $dir/.sibling-forums") if $debug;
	    @forumList = &formatForums($dir, "$hndocs$dir/.sibling-forums");
	}
	elsif ($dir) {
	    if (-e "$hndocs$parentDir/.child-forums") {
		@forumList = &formatForums($parentDir, 
					   "$hndocs$parentDir/.child-forums");
	    }
	    elsif ($autoGenerateSiblingForumList) {
		# Generate them using getChildren in parentDir.
		@forumList = &getChildren($parentDir);
	    }
	}
    }
    return @forumList;
}


sub getChildren {
    # Get the list of child forums.
    # Optionally generate child forums list.
    # We need to add something compatible to edit-article.pl
    my ($dir) = @_;
    local $debug = 0;

    my @forumList = ();
    &HNMsg("building child list for '$dir'") if $debug;

    if ($displayChildren) {
	if (-e "$hndocs$dir/.child-forums") {
	    @forumList = &formatForums($dir, "$hndocs$dir/.child-forums");
	}
	elsif ($autoGenerateChildForumList) {
	    # Create the child-forums file.
	    opendir (DIR, "$hndocs$dir") ||
		&HNError("Problem reading $hndocs$dir");
	    my @files;
	    @files = readdir DIR;
	    closedir DIR;

	    # Look at the forum urc files in alphabetical order.
	    # Need a pattern for all legitimate forum names.
	    @files = grep (/^[^0-9][\w-]+\.html,urc$/, @files);
	    @files = sort {$a <=> $b} @files;
	    &HNMsg("child forums: @files") if $debug;

	    my @childForumLines = ();
	    foreach $file (@files) {
		next if ($file !~ m{(.*)\.html,urc$ }x);
		my $root = $1;
		my %urc = &parseURCFile("$hndocs$dir/$root.html,urc");
		next if (!%urc);
		next if $urc{Redirect} || $urc{Hidden};
		# Add this child.
		my $childTitle = $urc{'ShortTitle'} || $urc{'Title'};
		my $childURL = $urc{'URL'};
		my $relativeURL = &url2hnpath($childURL);
		$childURL = convertref($childURL);
		$childURL =~ s,^$dir/,,; # Make it relative to $dir
		&HNMsg("child forum: $childTitle: $childURL") if $debug;
		push @forumList, "<A HREF=\"$childURL\">$childTitle</A>";
		push @childForumLines, "$root: $childTitle";
	    }
	    # Write the child-forums file.
	    require 'lock.pl';
	    &HNMsg("Locking: '$hndocs$dir'") if $debug;
	    &lock("$hndocs$dir");
	    open (FH, ">$hndocs$dir/.child-forums");
	    chmod "$hnfilemode", "$hndocs$dir/.child-forums";
	    print FH join ("\n", @childForumLines) . "\n";
	    &unlock("$hndocs$dir");
	}
    }
    return @forumList;
}


sub formatForums {
    my ($dir, $file) = @_;
    # Read a list of forums from the file.
    # URLs in the file may be relative to the dir.
    # Would be nice to not hyperlink the forum we are currently in, maybe.
    local $debug = 0;
    &HNMsg("format forums in '$file' relative to '$dir'") if $debug;
    open (FH, "<$file") || &HNError("Can't open $file");
    my @lines = <FH>;
    &HNMsg("Found: @lines") if $debug;

    my @forumList = ();
    foreach $line (@lines) {
	if ($line =~ m{^([^:]*): (.*)}) { 
	    my $forumURL = $1; 
	    my $forumName = $2;
	    &HNMsg("Forum name: $forumName") if $debug;
	    if ($forumURL =~ m,^/,) { 
		# relative to $hn root
		$forumURL = "$getURL$forumURL.html"; 
	    }
	    elsif ($forumURL !~ m,^http:,) { 
		# relative to $dir 
		$forumURL = "$getURL$dir/$forumURL.html"; 
	    }
	    push @forumList, "<A HREF=\"$forumURL\">$forumName</A>";
	}
    }
    return @forumList;
}


sub printURLorText {
  # Print the argument if it is an http URL by fetching it
  # Otherwise print it as text without conversion.
  my ($URLorText, $prefix) = @_;
  return if !$URLorText;

  if (!&printURL($URLorText, $prefix)) {
    print $URLorText;
  }
}


sub printURL {
    my ($URL, $prefix) = @_;
    # General routine to get the URL and print it.
    # Checks whether the URL is local so it can bypass http.
    # $prefix is used to support relative URLs.

    local $debug = 0;

    $URL =~ s{\s+}{}g;  # Remove all whitespace.
    return 0 if !$URL;
    &HNMsg ("Getting: '$URL' <BR>Prefix: $prefix<BR>") if $debug;

    my $page;
    my $file;

    # Is it a full URL starting with a scheme name?
    if ($URL =~ m{^\w+:}) {
	# It must be an http URL.   "file:" is not supported yet.
	if ($URL !~ m,^http:[^\s<\">()]+,) {
	  &HNUserError("<P><STRONG>This is not an http URL: '$URL'</STRONG>\n");
	}
	# Ignore anything before $prefix or /hn_tmp.
	# This is not particularly reliable, but
	# we want to allow people to change their server or port,
	# so this overrides external URLs that match the same pattern.
	# We should have a list of equivalent $http prefixes.

	# This problem will be gone when we always use relative path
	# for internally stored bodies.
	if (($URL =~ m,($prefix(-body|-content)?\.html)$,)
	    # For previewing, we generate a tmp file.
	    || ($URL =~ m,(/hn_tmp[0-9]+)$,)) {
	  # We can get this file directly, relative to $hndocs.
	  $file = $1;
	}
	else {
	  # We could look for hndocsURL prefix.
	}
    }

    # Is it an absolute path URL, i.e. starting with '/'?
    elsif ($URL =~ m,^/,) {
      # Does it look like it should be relative to the $hndocs dir?
      # Note, this overrides absolute URL relative to server root.
      # This problem will be mostly gone when we always use relative path
      # for internally stored bodies.
      if (($URL =~ m,^($prefix(-body|-content)?\.html)$,)
	  || ($URL =~ m,^(/hn_tmp[0-9]+)$,)) {
	$file = $1;
      }
      else {
	$file = $URL;
      }
    }

    # Is it a relative URL, i.e. not starting with '/'?
    # Note, we disallow relative URLs entered by users
    # for messages.  But HyperNews itself uses them.
    elsif ($URL =~ m{^[^/]}) {
      # URL is a relative path, relative to the parent of the $prefix 
      # Should we test whether it exists?
      $prefix =~ m{/[^/]+$ }x;	# Get parent.
      $file = "$`/$URL";
      &HNMsg("relative URL in $file") if $debug;
    }

    # Did we get a file by any of the above tests?
    if ($file) {
      # Bypass http - just copy the local file, if it exists.
      &HNMsg("get file: $file") if $debug;
      $file = &untaintPath($file);
      if (!-e "$hndocs$file" || -d "$hndocs$file") {
	&HNMsg("'$hndocs$file' doesn't exist or is a directory") if $debug;
	# We could look for look for hndocsURL prefix.
	$file = '';  # Give it up
      }
    }

    if ($file) {
      &HNMsg("Copying $hndocs$file") if $debug;
      open (FH, "<$hndocs$file");
      my @lines = <FH>;
      $page = join('', @lines);
      close (FH);
      # Fix relative URLs in $page.
      require "html.pl";
      $page = &html::abs ("$getURL$URL", 'http', '', '', '/', $page);
      print $page;
      return 1;
    }

    elsif ($URL =~ m{^\w+:}) {
      # It's still a URL, so we need to fetch it.
      &HNMsg("Getting URL: $URL") if $debug;
      # Too bad this doesn't work: print "\n<EMBED SRC=\"$URL\">\n";
      require "url.pl";
      # Need to pass other environment vars: Accepts, Agent.
      $page = &url::get
	  ($URL, 'http', $ENV{SERVER_NAME}, $ENV{SERVER_PORT});
      if (!$page) { warn "$!\n"; }
      else {
	require "html.pl";
	$page = &html::abs ($URL, 'http', '', '', '/', $page);

	if ( $page =~ m{^(.*?) # anything up to BODY
			(<\s*BODY[^>]*>.*)$ }isx ) {
	    # Cut out the extra HTML and HEAD tags - Jon
            my $head = $1;
            my $bodyRest = $2;

            # Cut out the duplicate stuff - there might be more.
            $head =~ s{<html>}{}i;
            $head =~ s{</?head>}{}ig;
            $head =~ s{<title>.*?</title>}{}i;

#	    $bodyRest =~ s{</body>}{}i;  this is ok
	    $bodyRest =~ s{</html>}{}i;

            print qq{
$head
$bodyRest
};
	}
        else { 
	    # bodyless pages only - Jon
	    print $page;   # print "no match";
	}
       }
      return 1;
    }
    
    return 0;
  }

# $textButtons = 1;

sub commandLink {
  # Print a command link.
  my ($commandURL, $target, $msg, $gif, $width, $height, $alt, $align) = @_;
  my $alignment = ($align ?  'ALIGN="absmiddle"' : '');
  my $anchor = "<A HREF=\"$commandURL\" $target
 onMouseOver=\"return window.help('$msg');\"
 onMouseOut=\"return window.help('');\">";
  if ($gifsOK && !$textButtons) {
    # Netscape 2.0 needs the width and height with the javascript.
    print "
$anchor
 <IMG SRC=\"$hniconsURL/$gif\" BORDER=0  
 WIDTH=$width HEIGHT=$height
 ALT=\"$alt\"
 $alignment></A>\n";
      }
  else {
    # The $TD$B is not always right.
    print "$TD$B
$anchor
 $alt</A>$endB$endTD \n";
  }
}

1;

