# Routines for HTML to ASCII.
# (fixed width font, no font changes for size, bold, etc) with a little

# BUGS AND MISSING FEATURES
#  font tags (e.g. CODE, EM) cause an extra whitespace 
#   e.g. <TT>foo</TT>, -> foo ,

# Jim Davis July 15 1994
# modified 3 Aug 94 to support MENU and DIR

# Daniel LaLiberte - Jan 11, 1996
# modified to handle PRE and paragraphs "correctly"
# and to add a list of URL references at the end.

# Daniel LaLiberte - Apr 29, 1998
# Fix BR to always print a new line.

require "tformat.pl" || die "Could not load tformat.pl: $@\nStopped";

# Can be set by command line arg
if (! defined($columns_per_line)) {
    $columns_per_line = 72;}

if (! defined($flush_last_page)) {
    $flush_last_page = 1;}

if (! defined($line_prefix)) {
  $line_prefix = '';  # No default.
}

# amount to add to indentation
$indent_left = 4;
$indent_right = 4;

# ignore contents inside HEAD.
$ignore_text = 0;

# Set variables in tformat
$left_margin = 1;
$right_margin = $columns_per_line;
$bottom_margin = 0;

# application-specific initialization routine
sub html_begin_doc {
    @list_stack = ();
    $list_type = "bullet";
    $list_counter = 0;
    print $line_prefix;
  }

## Routines called by html.pl
$Begin{"HEAD"} = "begin_head";
$End{"HEAD"} = "end_head";

sub begin_head {
    local ($element, $tag) = @_;
    $ignore_text = 1;}

sub end_head {
    local ($element) = @_;
    $ignore_text = 0;}

$Begin{"BODY"} = "begin_document";

sub begin_document {
    local ($element, $tag) = @_;
    &start_page();}

$End{"BODY"} = "end_document";

sub end_document {
    local ($element) = @_;
    &fresh_line();}

## Headers

$Begin{"H1"} = "begin_header";
$End{"H1"} = "end_header";

$Begin{"H2"} = "begin_header";
$End{"H2"} = "end_header";

$Begin{"H3"} = "begin_header";
$End{"H3"} = "end_header";

$Begin{"H4"} = "begin_header";
$End{"H4"} = "end_header";

$Skip_Before{"H1"} = 1;
$Skip_After{"H1"} = 1;

$Skip_Before{"H2"} = 1;
$Skip_After{"H2"} = 1;

$Skip_Before{"H3"} = 1;
$Skip_After{"H3"} = 0;

sub begin_header {
    local ($element, $tag) = @_;
    &skip_n_lines ($Skip_Before{$element}, 5);}

sub end_header {
    local ($element) = @_;
    &skip_n_lines ($Skip_After{$element});}

$Begin{"BR"} = "line_break";

sub line_break {
    local ($element, $tag) = @_;
    &print_blank_lines(1);}

$Begin{"P"} = "begin_paragraph";

# if fewer than this many lines left on page, start new page
$widow_cutoff = 5;

sub begin_paragraph {
    local ($element, $tag) = @_;
    # Skipping is not correct in general. dan
    &skip_n_lines (1, $widow_cutoff);}

$Begin{"BLOCKQUOTE"} = "begin_blockquote";
$End{"BLOCKQUOTE"} = "end_blockquote";

sub begin_blockquote {
    local ($element, $tag) = @_;
    $left_margin += $indent_left;
    $right_margin = $columns_per_line - $indent_right;
    &skip_n_lines (1);}

sub end_blockquote {
    local ($element) = @_;
    $left_margin -= $indent_left;
    $right_margin = $columns_per_line;
    &skip_n_lines (1);}

$Begin{"PRE"} = "begin_pre";
$End{"PRE"} = "end_pre";

sub begin_pre {
    local ($element, $tag) = @_;
    &skip_n_lines (1);  # Added to be more consistent with evil PRE rendering
    $whitespace_significant = 1;}

sub end_pre {
    local ($element) = @_;
    &skip_n_lines (1);  # Added to be more consistent with evil PRE rendering
    $whitespace_significant = 0;}

$Begin{"INPUT"} = "form_input";

sub form_input {
    local ($element, $tag, *attributes) = @_;
    if ($attributes{"value"} ne "") {
	&print_word_wrap($attributes{"value"});}}

$Begin{"HR"} = "horizontal_rule";

sub horizontal_rule {
    local ($element, $tag) = @_;
    &fresh_line ();
    &print_n_chars ($right_margin - $left_margin, "-");
    &fresh_line ();}

# Add code for IMG (use ALT attribute)
# Ignore I, B, EM, TT, CODE (no font changes)

## List environments

$Begin{"UL"} = "begin_itemize";
$End{"UL"} = "end_list_env";

$Begin{"OL"} = "begin_enumerated";
$End{"OL"} = "end_list_env";

$Begin{"MENU"} = "begin_menu";
$End{"MENU"} = "end_list_env";

$Begin{"DIR"} = "begin_dir";
$End{"DIR"} = "end_list_env";

$Begin{"LI"} = "begin_list_item";


sub push_list_env {
    push (@list_stack, join (":", $list_type, $list_counter));}

sub pop_list_env {
    ($list_type, $list_counter) = split (":", pop (@list_stack));
    $left_margin -= $indent_left;}

sub begin_itemize {
    local ($element, $tag) = @_;
    &skip_n_lines (1);
    &push_list_env();
    $left_margin += $indent_left;
    $list_type = "bullet";
    $list_counter = "*";}

sub begin_menu {
    local ($element, $tag) = @_;
    &skip_n_lines (1);
    &push_list_env();
    $left_margin += $indent_left;
    $list_type = "bullet";
    $list_counter = "*";}

sub begin_dir {
    local ($element, $tag) = @_;
    &skip_n_lines (1);
    &push_list_env();
    $left_margin += $indent_left;
    $list_type = "bullet";
    $list_counter = "*";}

sub begin_enumerated {
    local ($element, $tag) = @_;
    &skip_n_lines (1);
    &push_list_env();
    $left_margin += $indent_left;
    $list_type = "enumerated";
    $list_counter = 1;}

sub end_list_env {
    local ($element) = @_;
    &pop_list_env();
    &skip_n_lines (1);
}

sub begin_list_item {
    local ($element, $tag) = @_;
    $left_margin -= 2;
    &fresh_line();
    print("$list_counter ");
    if ($list_type eq "enumerated") {$list_counter++;}
    $left_margin += 2;}

$Begin{"DL"} = "begin_dl";
$End{"DL"} = "end_list_env";

sub begin_dl {
    local ($element, $tag) = @_;
    &skip_n_lines(1,5);}
    
$Begin{"DT"} = "begin_defined_term";
$Begin{"DD"} = "begin_defined_definition";
$End{"DD"} = "end_defined_definition";

sub begin_defined_term {
    local ($element, $tag) = @_;
    &fresh_line();}

sub begin_defined_definition {
    local ($element, $tag) = @_;
    $left_margin += $indent_left;
    &fresh_line();}

sub end_defined_definition {
    local ($element) = @_;
    $left_margin -= $indent_left;
    #&fresh_line();
}

$Begin{"META"} = "begin_meta";

# a META tag sets a value in the assoc array %Variable
# i.e. <META name="author" content="Rushdie"> 
# sets $Variable{author} to "Rushdie"
sub begin_meta {
    local ($element, $tag, *attributes) = @_;
    local ($variable, $value);
    $variable = $attributes{name};
    $value = $attributes{content};
    $Variable{$variable} = $value;}

$Begin{"A"} = "begin_anchor";
$End{"A"} = "end_anchor";

local($anchor_href) = 0;
local(@anchor_hrefs) = ();

# The last html_content string since the last begin_anchor
local($last_content) = ''; 

sub begin_anchor {
    local ($element, $tag, *attributes) = @_;
    if ($anchor_href = $attributes{"href"}) {
	push (@anchor_hrefs, $attributes{"href"});
    }
    $last_content = '';
}

sub end_anchor {
    local ($element, $tag) = @_;
    if ($anchor_href) {
      my ($anchor_re) = $anchor_href;
      $anchor_re =~ s,^mailto:,,i;
      $anchor_re = quotemeta($anchor_re);  # Protect against bad chars
      if ($last_content =~ m,^$anchor_re$,i) {
	# We don't need this one.
	pop (@anchor_hrefs);
      }
      else {
	local ($num_hrefs) = @anchor_hrefs + 0;
	local ($whitespace_significant) = 1; # Turn off temporarily.
	&print_word_wrap ("[$num_hrefs]");
      }
    }
    $anchor_href = 0;  # Turn off until next href
    # We need to disable whitespace before and after anchors,
    # but not within the anchor content.  print_word_wrap does it wrong.
}

$Begin{"IMG"} = "begin_img";
$End{"IMG"} = "end_anchor";

sub begin_img {
    local ($element, $tag, *attributes) = @_;
    if ($anchor_href = $attributes{"src"}) {
	push (@anchor_hrefs, $attributes{"src"});
    }
    &print_word_wrap (($attributes{"alt"} ne "") ? 
		      $attributes{"alt"} : "[IMAGE]");}

sub html_end_doc {
    # Finish up last line, if needed.
    &new_line();

    if (@anchor_hrefs) {
	# Print references
	&horizontal_rule();
	local ($i, $href);
	for ($i = 1; @anchor_hrefs; $i++) {
	    $href = shift (@anchor_hrefs);
	    &print_word_wrap("[$i] $href");
	    &fresh_line();
	}
    }
}

# Content and whitespace.

sub html_content {
    local ($string) = @_;
    unless ($ignore_text) {
      $last_content = $string;
      &print_word_wrap ($string);}}

sub html_whitespace {
    local ($string) = @_;
    if (! $whitespace_significant) {
	die "Internal error, called html_whitespace when whitespace was not significant";}
    local ($i);
    for ($i = 0; $i < length ($string); $i++) {
	&print_whitespace (substr($string,$i,1));}}

# called by tformat.  Do nothing.
sub do_footer {
}

sub do_header {
}


1;
