# http.pl       --- recognize, parse and retrieve URLs
#
# http::get:     perform an http GET request and return the result
# http::post:    perform an http POST request and return the result
#
# Oscar Nierstrasz 26/8/93 oscar@cui.unige.ch
#
# 25/3/94 -- moved to separate package
# 28/3/94 -- added stripping of MIME headers (code by Martijn Koster)
# -- other changes by Daniel LaLiberte and Roberto Melo
#

package http;

$timeout = $main::timeout || 60; # Set in hnrc

$version = 'HTTP/1.0';   # The type of requests we can make.
# We are not 1.1 compliant for a couple reasons.  e.g. Status code of 100.
# This library should be replaced by libwww.pl.

use Socket;
# Alternatively, try:  require "sys/socket.ph";
# sys/socket.ph should be in your perl library.
# If it's not there, complain to your system admin!
# If it has errors, try fixing them by commenting out the junk.

# Get the hostname of this host, not the server we are connecting to.
# Using $ENV{'SERVER_NAME'} is wrong in the case that your mail host
# is not the same as your web host.
$thishost = $ENV{SERVER_NAME};
$thishost =~ m{(.*)};
$thishost = $1;   # untaint
# &main::ErrorIfTainted($thishost);

# Could use hostname, but that may be wrong for virtual hosts.
#chop($thishost = `/bin/hostname`);


($name, $aliases, $proto) = getprotobyname("tcp");
($name, $aliases, $type, $len, $thisaddr) = gethostbyname($thishost);

$sockaddr = 'S n a4 x8';
$thissock = pack($sockaddr, &AF_INET, 0, $thisaddr);

$useragent = "User-Agent: HyperNews $main::hnversion http.pl\r\n";

#&main::HNMsg("After http.pl top stuff.");


sub timeout {
  print "attempting timeout";
  die ("Timeout");
}

# Code adapted from Marc van Heyningen
sub get {
  local($host, $port, $request) = @_;
  # Perform an http GET request and return the result.

  local $debug = 0;
  &main::HNMsg("In http::get: $host, $port, $request") if $debug;
  &main::ErrorIfTainted($proto);
  
  my $serverHost = $httpProxyHost || $host;
  my $serverPort = ($httpProxyHost && $httpProxyPort) || $port;
  ($fqdn, $aliases, $type, $len, $thataddr) = gethostbyname($serverHost);
  return "<B>Host unknown: $serverHost</B>" if !$thataddr;  # host unknown?
  $that = pack($sockaddr, &AF_INET, $serverPort, $thataddr);
  #&main::ErrorIfTainted($proto);
  #&main::ErrorIfTainted(&AF_INET);
  #&main::ErrorIfTainted(&SOCK_STREAM);
  socket(FS, &AF_INET, &SOCK_STREAM, $proto) || return undef;
  bind(FS, $thissock) || return undef;
  local($/);
  unless 
    (eval {
      $SIG{'ALRM'} = "http::timeout";
      alarm($timeout);
      connect(FS, $that) || return undef;
      select(FS); $| = 1; select(STDOUT);
      # MIME header treatment from Martijn Koster
      if ($version) {
	my $httpPrefix = "http://$host:$port" if $useFullHTTPURL;
	&main::HNMsg("httpPrefix: $httpPrefix") if $debug;
	print FS "GET $httpPrefix$request $version\r\n"; 
	print FS "Host: $host\r\n"; 
	print FS "$useragent"; 
	print FS "\r\n";  # End headers
	undef($page);
	$/ = "\n";
	my $line = <FS>;
	&main::HNMsg("first line: $line") if $debug;
	if ($line =~ m:HTTP(/1.[01])?\s+\d+\s+:) { #HTTP/1.0 or 1.1
	  while($line = <FS>) {
	    &main::HNMsg("header: $line") if $debug;
	    last if $line =~ m/^[\r\n]+$/; # end of headers
	  }
	  undef($/);
	  $page = <FS>;  # everything else is the body
	}
	else {			# old style server reply
	  undef($/);
	  $page = $line;
	  $line = <FS>;            
	  $page .= $line;
	}
      }
      else {			# old style request
	print FS "GET $request\r\n";
	$page = <FS>;		# gives old-style reply
      }
      $SIG{'ALRM'} = "IGNORE";
    }) {
      return ($@ ? "Making http request caused error: \"$@\"" : "TimeOut");
      }
  $| = 0;
  close(FS);
  $page;
}



sub timeout_post_mail {
    print "timeout while posting";
    # Sendmail will queue the mail for later when it gets a status of 75.
    exit(75);
}

# This is only used by mail-incoming.pl
sub post {
  local($host, $port, $request,
	$debug_file, $user, $pass, %args) = @_;
  # Perform an http POST request and return the result.
  # Must use local for vars to be seen in eval?
  local $debug = 0;

  print $debug_file ("In http::post: $host, $port, $request\n") if $debug;
  
  require "base64.pl";
  local $credentials = &base64::b64encode("$user:$pass");

  local $from = "From: $user@$thishost\r\n";
  
  my $serverHost = $httpProxyHost || $host;
  my $serverPort = ($httpProxyHost && $httpProxyPort) || $port;

  ($fqdn, $aliases, $type, $len, $thataddr) = gethostbyname($serverHost);
  return "<B>Host unknown: $serverHost</B>" if !$thataddr;  # host unknown?
  $that = pack($sockaddr, &AF_INET, $serverPort, $thataddr);
  socket(FS, &AF_INET, &SOCK_STREAM, $proto) || return undef;
  bind(FS, $thissock) || return undef;
  local($/);
  
  my @content;
  foreach $key (keys(%args)) {
    my $val = &encode($args{$key});
    push @content, "$key=$val";
  }
  
  local $contentStr = join('&', @content);
  local $len = length($contentStr) + 1;  # calc length - include newline before.
  
  print $debug_file ("content = '$contentStr'\n") if $debug;
  
  unless 
    (eval {
      #    print "post_type is $Post_type\n";
      # 	if ($Post_type) { }
      $SIG{'ALRM'} = "http::timeout_post_mail";	# Hack
      alarm($timeout);
      #        while (1 == 1) {  }  # loop for testing.
      
      connect(FS, $that) || return undef;
      select(FS); $| = 1; select(STDOUT);
      # MIME header treatment from Martijn Koster
      my $httpPrefix = "http://$host:$port" if $useFullHTTPURL;
      print FS "POST $httpPrefix$request $version\r\n";
      print FS "Host: $host\r\n"; 
      print FS $useragent;
      print FS $from;
      print FS "Content-Type: application/x-www-form-urlencoded\r\n";
      print FS "Content-length: $len\r\n";
      print FS "Authorization: Basic $credentials\r\n";
      print FS "\n";		# End headers  # proper ending????
      print FS "$contentStr\n";
      undef($page);
      $/ = "\n";
      my $line = <FS>;
      #this line here can also be used to report the status codes...
      print $debug_file "\nhttp reply: $line\n" if $debug;        
      print $debug_file ("first line: $line") if $debug;
      if ($line =~ m:HTTP(/1.[01])?\s+(\d+)\s+:) { #HTTP/1.0 or 1.1
	$status = $2;
	while($line = <FS>) {
	  last if $line =~ m/^[\r\n]+$/;	# end of headers
	}
	undef($/);
	$page = <FS>;
      }
      else {			# old style server reply
	undef($/);
	$page = $line;
	$_ = <FS>;            
	$page .= $line;
      }
      $SIG{'ALRM'} = "IGNORE";
    }) {
      return undef;
    }
  close(FS);
  return ($page, $status);
}


sub encode {
    local($string) = @_;
# Usage: $string = &encode($string);
#
# Return the string escaped as if for a FORM.
# Maybe use URLencode.

    $string =~ s/%/%25/g;  # Do this first.
    $string =~ s/\"/%22/g;
    $string =~ s/\&/%26/g;
    $string =~ s/\</%3c/g;
    $string =~ s/\>/%3e/g;
    $string =~ s/\+/%2b/g;
    $string =~ s/\n/%0a/g;
    
    # Do this last - convert any remaining whitespace.
    $string =~ s/\s/\+/g ; 

    # If string is null, convert it to ''.  - wrong.
    #if (!$string) { $string = "''"; }
    $string;
}


1;

