#!/usr/local/bin/perl

# PERL implementation of Erik E. Fair's 'shlock' (from the NNTP distribution)
# Ported by Brent Chapman <Brent@GreatCircle.COM>

#Implementation modified to include lock breaking techniques by Roberto Melo
#<rmelo@ncsa.uiuc.edu> on March 1995.

# Lock breaking was motivated by previous experiences with a locking
# mechanism that frequently became jammed up due to some error in the
# software (the client of the lock), thus filling our mailboxes with
# extraneous warnings.  It seemed reasonable to recover from the
# situation by automatically breaking the lock after some preset time
# (established by the guy who died after creating the lock) has passed.
# The breaking is done robustly too, with locks that can be broken.

# $Source: /sources/cvsrepos/majordomo/shlock.pl,v $
# $Revision: 1.1.1.2 $
# $Date: 1993/10/27 16:20:10 $
# $Author: rouilj $
# $State: Exp $
#
# $Locker:  $
# 

package shlock;

# The following is the time specified in seconds before a process can break 
# a lock held by another process.  
# Note: This should be a parameter of the lock rather than a static value.
$cons_difference = 45;

# The following is a flag for the option that the process breaking the lock 
# will kill the locked process.
$kill_processes = 1;

# Some error codes.
$EPERM = 1;
$ESRCH = 3;
$EEXIST = 17;
$ENOSPC = 28;


$shlock_debug = 0;


sub main'lock_tree {
    # Recursively lock the directory and every subdirectory.
    # Pass in the directory without the trailing /
    local($dir) = shift;
    local($counter);
    local(@dirFiles)=();

    &main::HNMsg (("locking:$dir\n")) if $shlock_debug;
    &main'lock(&main::untaintPath($dir));  
    if (-d $dir) {
    opendir(DIR,$dir) || warn ("Problems opening $dir");
    @dirFiles = readdir(DIR);
    closedir(DIR);
    }
    for($counter=0; $counter < @dirFiles ;$counter++) {
       if ((-d "$dir/@dirFiles[$counter]") &&
         (@dirFiles[$counter] ne "..") &&
           (@dirFiles[$counter] ne ".")) {
          &main'lock_tree("$dir/@dirFiles[$counter]"); 
       }
    }
    return (0);    
}

sub main'unlock_tree {
    # Pass in the directory without the trailing /
    local($dir) = shift;
    local($counter);
    local(@dirFiles)=();

    &main::HNMsg (("unlocking:$dir\n")) if $shlock_debug;
    if ($dir ne '0') {
       &main'unlock(&main::untaintPath($dir));
       if (-d $dir) { 
       opendir(DIR,$dir) || warn ("Problems opening $dir");
       @dirFiles = readdir(DIR);
       closedir(DIR);
       }
       for($counter=0; $counter < @dirFiles ;$counter++) {
          if ((-d "$dir/@dirFiles[$counter]") &&
            (@dirFiles[$counter] ne "..") &&
              (@dirFiles[$counter] ne ".")) {
             &main'unlock_tree("$dir/@dirFiles[$counter]");
          }
         }
    }
    return (0);
}

sub main::lrec {
  local($file) = shift;
  # $fm is what will actually get passed to open()

  # create name for lock file
  local($lockfile) = &main::lock_file_name($file);

    # force unqualified filehandles into callers' package
    local($package) = caller;
    local($curtime,$base_time,$elapsed);
    local(@stat_array, $curtime);
    $FH =~ s/^[^']+$/$package'$&/;
    @stat_array=stat("$lockfile");
    $base_time = $stat_array[2];
    $elapsed = 0;
    &main::HNMsg ("OUTPUT STAT=$stat_array[8],$stat_array[9],$stat_array[10]\n") if $shlock_debug;
    &main::HNMsg ("Current time=$curtime\n") if $shlock_debug;
    &main::HNMsg ("ENTERING THE RECURSIVE LOCKBREAK LOOP\n") if $shlock_debug;
     while (1) {
      &main::HNMsg ("Elapsed=$elapsed\n") if $shlock_debug;
      @stat_array=stat("$lockfile");
      if ($base_time==$stat_array[2]) {
        &main::HNMsg ("base-time is the same\n") if $shlock_debug;
        if ($elapsed > $cons_difference) {
           &main::HNMsg ("Can break lock here?\n") if $shlock_debug;
           if(&main'lrec("$lockfile")) {
                  #Got lock on lockfile to break lock
                  @stat_array=stat("$file");
                  if ($kill_processes) {
                    if (open(FILE, "$file")) {
                     $buf = <FILE>;
                     $buf =~ m,([0-9]+),; $buf = $1;  # Untaint.
                     if (int($buf) > 0) {
                         close(FILE);
                         &main::HNMsg ("killing $buf\n") if $shlock_debug;
                         kill(9,$buf);
                     }
                    }
                  }
                  local($procid);
                  $procid = fork();
                  if ($procid == 0) {
                      exec("cp","$lockfile","$file");
                  }
                  waitpid ($procid, 0);
                  # Print this out?
                  &main::HNMsg ("LOCK of $file held by $buf broken by $$\n");
                  if (&ownlock("$lockfile")) {
                      unlink("$lockfile");
                      return (1);
                  }
                  return(0);
           }
           else {
               return (0);
           }
        }#of elapsed
        #else for $elapsed
        else {
          &main::HNMsg ("Performing shlock now\n") if $shlock_debug;
          if (&main'shlock("$lockfile")) {
            #ready to break lock 
            if ($shlock_debug) {
                &main::HNMsg ("Press Enter to break lock...\n") if $shlock_debug;
                local($test);
            }#of if
            if ($kill_processes) {
            if (open(FILE, "$file")) {
             $buf = <FILE>;
	     $buf =~ m,([0-9]+),; $buf = $1;  # Untaint.
             if (int($buf) > 0) {
                 close(FILE);
                 &main::HNMsg ("killing $buf\n") if $shlock_debug;
                 kill(9,$buf);
              }
             }
            }
            local($procid);
            $procid = fork();
            if ($procid == 0) {
                  &main::HNMsg ("This is Child process\n") if $shlock_debug;
                  exec("cp","$lockfile","$file");
            }#of if
            waitpid ($procid, 0);
#            &main::HNMsg ("LOCK of $file held by $buf broken by $$\n");
            if (&ownlock("$lockfile")) {
                   unlink("$lockfile");
                   return 1;
            }#of if
            return(0); #This means lock broken by another process
          }#of if 
        }#of else elapsed 
      }#of if base
      #else for base time. 
      else {
        $elapsed=0;
        $base_time=$stat_array[8];
      }#of else if base

      &main::HNMsg ("Waiting 1 Second....\n") if $shlock_debug;
      sleep(1);
      $elapsed++; 
     }#of while?
    # If we get this far, there is an error
    return undef;
}#of function

sub main'shlock	## Public
{
    local($file) = shift;
    local($tmp);
    local($retcode) = 0;
    local($redo_loop);

    &main::HNMsg ("trying lock \"$file\" for pid $$\n") if $shlock_debug;
    if (!($tmp = &xtmpfile($file))) {
	return(undef);
    }

    do {
	$redo_loop = 0;
 
	if (! link($tmp, $file)) {
	    if ($! == $EEXIST) {
                &main::HNMsg ("TMP file is:$tmp\n") if $shlock_debug;
		&main::HNMsg ("lock \"$file\" already exists\n") if $shlock_debug;
		$redo_loop = 1;
		if (&cklock($file)) {
		    &main::HNMsg ("extant lock is valid\n") if $shlock_debug;
		    $redo_loop = 0;
		} else {
		    &main::HNMsg ("lock is invalid; removing\n") if $shlock_debug;
		    if (unlink($file) <= 0) {
			warn("RMlock: unlink(\"$file\"): $!");
		    }
		}
	    } else {
		&HNError ("lock: link(\"$tmp\", \"$file\"): $!");
	    }
	} else {
	    &main::HNMsg ("got lock \"$file\"\n") if $shlock_debug;
	    $retcode = 1;
	}
    } while ($redo_loop);

    if (unlink($tmp) <= 0) {
	warn("lock: unlink(\"$file\"): $!");
    }
    return($retcode);
}

sub p_exists {
    local($pid) = shift;

    &main::HNMsg ("process $pid is ") if $shlock_debug;
    if ($pid <= 0) {
	&main::HNMsg ("invalid\n") if $shlock_debug;
	return(0);
    }
    if (kill(0, $pid) <= 0) {
	if ($! == $ESRCH)
	    { &main::HNMsg ("dead\n") if $shlock_debug; return 0; }
	elsif ($! == $EPERM)
	    { &main::HNMsg ("alive\n") if $shlock_debug; return 1; }
	else
	    { &main::HNMsg ("state unknown: $!\n") if $shlock_debug; return 1; }
    }
    print "alive\n" if $shlock_debug;
    return 1;
}

sub ownlock {
    local($file) = shift;
    local(*FILE, $len, $pid, $buf);

    &main::HNMsg ("CHECKING OWNERSHIP OF LOCK  \"$file\"\n") if $shlock_debug;
    if (!open(FILE, "$file")) {
        &main::HNMsg ("Should this be a Fatal Error: lock file not found\n")
	            if $shlock_debug;;
	return (0);
    }
    $buf = <FILE>;
    $buf =~ m,([0-9]+),; $buf = $1;  # Untaint.
    if (int($buf) <= 0) {
        close(FILE);
        &main::HNMsg ("lock file format error\n") if $shlock_debug;
        return 0;
    }
    close(FILE);
    if ($$ == int($buf)) {
       return(1);} 
    local($tmpgrp)=$$;
    &main::HNMsg ("GRP=$tmpgrp File=$buf") if $shlock_debug;
    &main::HNMsg ("PROBLEMS IN OWNERSHIP\n") if $shlock_debug;
    return(0);
}

sub cklock {
    local($file) = shift;
    local(*FILE, $len, $pid, $buf);

    &main::HNMsg ("checking extant lock \"$file\"\n") if $shlock_debug;
    if (!open(FILE, "$file")) {
	warn("lock: open(\"$file\"): $!");
	return 1;
    }
    $buf = <FILE>;
    $buf =~ m,([0-9]+),; $buf = $1;  # Untaint.
    if (int($buf) <= 0) {
	close(FILE);
	&main::HNMsg ("lock file format error\n") if $shlock_debug;
	return 0;
    }
    close(FILE);
    return(&p_exists(int($buf)));
}

sub xtmpfile {
    local($file) = shift;
    local(*FILE);
    local($tempname);
    local($redo_loop);
    local($error_code);

    $tempname = $file;
    if ($tempname =~ m,/,) {
      if ($tempname !~ m{/$ }x) {
	# Make the lock in the same directory as $tempname.
	$tempname =~ s,/[^\/]*$,/,;
      }
      # else $tempname *is* a directory, so put the lock inside it.
      $tempname .= ".shlock.$$";
    } else {
      # No directory in file name, so assume cwd is correct.
      $tempname = ".shlock.$$";
    }
    &main::HNMsg ("temporary filename \"$tempname\"\n") if $shlock_debug;

    do {
	$redo_loop = 0;
#	&main::Debug('tempname', $tempname);
#	&main::tracedump("while locking with temp file: $tempname");
	&main::ErrorIfTainted($tempname);
	if (!open(FILE, ">$tempname")) {
            $error_code = $!;
	    if ($error_code==$EEXIST) {
		&main::HNMsg ("file \"$tempname\" exists\n") if $shlock_debug;
		if (unlink($tempname) <= 0) {
		    warn("lock: unlink(\"$tempname\"): $!");
		    return(undef);
		}
		$redo_loop = 1;
	    } else {
		&main::HNError ("lock: open(\">$tempname\"): $!");
	    }
	}
	chmod ("$main::hnfilemode", "$tempname");
    } while ($redo_loop);

    if (! print FILE "$$\n") {
	warn("lock: write(\"$tempfile\", \"$$\"): $!");
	close(FILE);
	unlink($tempname) || warn("lock: unlink(\"$tempname\"): $!");
	return(undef);
    }
    if (! print FILE "1\n") {
        warn("lock: write(\"$tempfile\", \"$$\"): $!");
        close(FILE);
        unlink($tempname) || warn("lock: unlink(\"$tempname\"): $!");
        return(undef);
    }
    close(FILE);
    return($tempname);
}

sub main'lcommit {
   local($file) = shift;
   local($tmp);
   local(@data);
   $tmp = getpgrp(0);
   $tmp = "$file.$tmp";
   if (! (open(LFH, "$tmp"))) {
      &main::HNMsg ("Problems Opening Temporary File\n") if $shlock_debug;
      unlink($tmp);
      return(0);
   }
   if (! (@data = <LFH>)) {
       &main::HNMsg ("Error Reading Temporarry FIle\n") if $shlock_debug;
       unlink($tmp);
       return(0);
   }
   if (! (close LFH)) {
      &main::HNMsg ("Error Closing Temporary File\n") if $shlock_debug;
      unlink($tmp);
      return(0);
   }
   local($lockfilename) = &main::lock_file_name($file);

   &main::HNMsg ("Calling ownlock $lockfilename\n") if $shlock_debug;;
   if (&ownlock("$lockfilename")) {
     &main::HNMsg ("File Content:@data") if $shlock_debug;
     if (!(&main'lreopen(CFILE,">>",$file))) 
     {   &main::HNMsg ("Problems Reopening File\n") if $shlock_debug;
     }
     print CFILE "@data";
   } else
   {  &main::HNMsg ("Can't Commit Lock not owned\n") if $shlock_debug;
      unlink($tmp);
      return(0);
   }
  unlink($tmp);
  return(1); 
}

sub main::lock_file_name {
  my ($file) = @_;
  my $lock_file_name = $file;
  if (-d $file) {
    # $file *is* a directory, so put the lock inside it.  Is this safe?
    $file .= '/' if $file !~ m{/$ }x;
    $lock_file_name = $file . '.HN_lock';
  }
  else {
    # else make the lock in the same directory as $file.
    $lock_file_name =~ s{([^/]*)$ } {.HN_lock_$1}x;
  }
  return $lock_file_name;
}

sub main'lwrite {
   local($file) = shift;
   local(@data) = shift;
   local($tmp);

   $tmp = getpgrp(0);
   $tmp = "$file.$tmp";
   if (! (open(LFH, ">>$tmp"))) {
      &main::HNMsg ("Problems Opening Temporary File\n") if $shlock_debug; 
      return(0);
   }
   if (! (print LFH "@data")) {
      &main::HNMsg ("Problems Writing to Temporary File\n") if $shlock_debug;
      return(0); 
   }
   if (! (close LFH)) {
      &main::HNMsg ("Error Closing Temporary File\n") if $shlock_debug;
      return(0); 
   }
   return(1);
}

sub main'readnum {
     local($lockfile) = @_;
     open(FILE,$lockfile);
     $buf1=<FILE>;
     $buf2=<FILE>;
     &main::HNMsg ("READNUM read: $buf2\n") if $shlock_debug;
     close(FILE);
     return($buf2); 
}

sub main'doexec {
    local(@exarray) = @_;
    &main::HNMsg ("In doexec @exarray\n") if $shlock_debug;
    local($procid) = fork();
    if ($procid == 0) {
      exec(@exarray);
    } else {
      waitpid ($procid,0);
    }
    return 1;
}


sub main'writenum {
    local($lockfile,$num) = @_;
    &main::HNMsg ("writenum is: $num\n") if $shlock_debug;
    local($lockfiletmp) ="$lockfile.$$";
    open(FILE,">$lockfiletmp") || &HNError ("Can't open $lockfiletmp: $!");
    print FILE ("$$\n$num\n"); 
    close(FILE);
    &main::HNMsg ("In writenum PID=$$ \n") if $shlock_debug;
    &main'doexec("cp","$lockfiletmp","$lockfile");
    unlink("$lockfiletmp");
}

sub main'lock {
    local($file)= shift;
    local($status);
    local($lockfile) = &main::lock_file_name($file);

    local($package) = caller;
    local($curtime);
    local(@stat_array, $curtime);
    $FH =~ s/^[^']+$/$package'$&/;
    local($elapsed);
    @stat_array=stat("$lockfile");
    $base_time=$stat_array[8];
    local($return_v)=0;

    local ($shlock_debug) = 0; # $main::debug;

    while(1) {
       if (&main'shlock("$lockfile")) {
          #&main::HNMsg ("$$ Sleeping here\n") if $shlock_debug;
          #sleep(10000);
          return (1); 
       }
       else {
             # didn't get the lock; wait 1 second and try again.
             if (&ownlock("$lockfile")) {
               local($numlock)=&main'readnum("$lockfile");
               $numlock++;
               &main'writenum("$lockfile",$numlock);
               &main::HNMsg ("Number of calls to lock by pid $$=$numlock\n")
		                  if $shlock_debug;
               return(1);
             }
             if ($elapsed > $cons_difference)
               {&main::HNMsg ("Can break lock here?\n") if $shlock_debug;
                 if (&main'lrec("$lockfile")) {
                        return(1); 
                 }
                 else {
                       $elapsed=0;
                 }
               }
       }
       $elapsed++;
       &main::HNMsg ("Still trying to lock file...waiting\n") if $shlock_debug;    
       sleep(1);
    }
}

sub main'lopen {
    local($FH) = shift;
    local($mode) = shift;
    local($file) = shift;
    # $fm is what will actually get passed to open()
    local($fm) = "$mode$file";
    local($status);

    # force unqualified filehandles into callers' package
    local($package) = caller;
    $FH =~ s/^[^']+$/$package'$&/;

    if (&main'lock("$file")) {
        # Got the lock; now try to open the file
        $status = open($FH, $fm);
        if (defined($status)) {
        } else {
             # File wasn't successfully opened; delete the lock
             unlink("$lockfile");
        }
        # return the success or failure of the open
        return($status);
    } 
}

# reopen a file already opened and locked (probably to change read/write mode).
# We remember the name of the lock file, so that we can delete it when
# we close the file

sub main'lreopen {
    local($FH) = shift;
    local($mode) = shift;
    local($file) = shift;
    # $fm is what will actually get passed to open()
    local($fm) = "$mode$file";

    # create name for lock file
    local($lockfile) = &main::lock_file_name($file);

    # force unqualified filehandles into callers' package
    local($package) = caller;
    $FH =~ s/^[^']+$/$package'$&/;
    &main::HNMsg ("LREOPEN: now reopening file\n") if $shlock_debug;
    # close the old file handle, and delete the lock reference
    if (&ownlock("$lockfile")) {
	close($FH);
    } else {
	# the file wasn't already locked
	# unlink("$lockfile");		### Do we really want to do this?
	return(undef);
    }

    # We've already got the lock; now try to open the file
    &main::HNMsg ("LREOPEN: now reopening file\n") if $shlock_debug;
    $status = open($FH, $fm);
    if (defined($status)) {
	# File successfully opened; remember the lock file for deletion
	$lock_files[fileno($FH)] = "$lockfile";
    } else {
	# File wasn't successfully opened; delete the lock
	unlink("$lockfile");
    }
    # return the success or failure of the open
    return($status);
}


# Close a locked file, deleting the corresponding .lock file.
sub main'lclose {
    local($FH) = shift;
    local($file) = shift;
    # force unqualified filehandles into callers' package
    local($package) = caller;
    $FH =~ s/^[^']+$/$package'$&/;

    close($FH);
    if (!(&main'unlock("$file"))) {
          &main::HNMsg ("Lock Broken by another Process\n") if $shlock_debug;
    }

}

sub main'unlock {
   local($file) = shift;
   local ($shlock_debug) = 0; # $main::debug;
   local($lockfile) = &main::lock_file_name($file);

   &main::HNMsg ("Trying to unlink $lockfile\n") if $shlock_debug;
   if (&ownlock($lockfile)) {
      local($numlock)=&main'readnum("$lockfile");
      if ($numlock > 1) {
         $numlock--;
         &main'writenum("$lockfile",$numlock); 
         &main::HNMsg ("PID $$ Still holds lock due to multiple lock calls!!\n")
             if $shlock_debug;
         return(1);
      } else {
         unlink($lockfile);
         return(1);
      }
    } else
       {
	   # Print this out?
	   &main::HNMsg ("Lock Broken by another Process\n") if $shlock_debug;;
	   return(0);
        }

}

1;

