#!/usr/local/bin/perl
# log-lib.pl - HyperNews transaction logs.
# Copyright Board of Trustees of the University of Illinois at UC, April 1996



sub logTransaction {
    my ($message, $URL) = @_;
    # Add the $message to the appropriate log file, dependent on $URL.
    # $URL may be a relative or absolute URL.

    if (!$hnLog) { return; }

    my $logProgram =  $ENV{'SCRIPT_NAME'};
    my $authType = $ENV{'AUTH_TYPE'};
    my $remoteHost = $ENV{'REMOTE_HOST'} || $ENV{'REMOTE_ADDR'} ||
      'Command-line';
    my $remoteUser = $ENV{'REMOTE_USER'} || $remoteUser; # From global var

    &HNMsg("logProgram: $logProgram
remoteHost: $remoteHost
remoteUser: $remoteUser 
authType: $authType\n") if $debug;

    #check and lock the log directory
    my $logdir;
    if (!$URL){
	$logdir = $hnPeople; 
    } else {
	$logdir = $hndocs . &chopper($URL);
	$URL =~ s,.*$logdir,,;
    }

    $logFile = "$logdir/$hnLog";
    &HNMsg("logFile: $logFile") if $debug;

#    require 'wwwdates.pl';  # See below.

    my $date = &logFileTime(time, 'GMT');

#return; # too late

    $message .= ": $URL" if $URL;

    my $logLine = &makeLogLine($remoteHost, $remoteUser, $date, $message);

    $logFile = &untaintPath($logFile);
    &ErrorIfTainted($logFile);

#return; # too late

    require 'lock.pl';
    &lock($logFile);
    if (!-e $logFile){
	open(LOGH, ">$logFile")
#	   || &HNError("Problem writing to $logFile");
    } else {
	open(LOGH, ">>$logFile") 
#	  || &HNError("Problem appending to $logFile");
    }

    print LOGH ("$logLine\n");
    close LOGH;
    chmod ("$hnfilemode", "$logFile");	   
    &unlock($logFile);
}


sub chopper {
    my ($url) = @_;
    my $dir = $url;
    $dir =~ s,^.*/get(\.cgi)?/,/,;
    $dir =~ s,\.html$,,;

    if ($dir =~ m,/[0-9]+$,){
      # Get the directory of the base article of $url
      $dir =~ s,(/[0-9]+)*$,,;
    } else {
      # Find the directory of the parent of the base article.
      # Hmm, this is wrong except for create and delete of base itself.
      # And logging of those actions can be directed at the parent log.
      # $dir =~ s,/[\w]+$,,;
    }

    return $dir;
}

sub makeLogLine {
    my ($remoteHost, $remoteUser, $date, $message) = @_;
    my $logLine;
    $remoteUser ||= '-';

    $logLine .= "$remoteHost";
    $logLine .= " -"; # ident field.
    $logLine .= " $remoteUser";
    $logLine .= " [$date]";
    $logLine .= "  \"$message\"" if $message;

    return $logLine;
}


# This needs to be here rather than wwwdates.pl to avoid
# some bug in either Perl 5.004_03 or on Solaris.
# Causes parseURCFile to read the wrong file or not read enough.

sub logFileTime {
    local($time) = @_;
    local($[) = 0;

    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
        localtime($time);

    my ($sec0,$min0,$hour0,$mday0,$mon0,$year0,$wday0,$yday0,$isdst0) = 
	gmtime($time);

    # Correct for the first day of a year.
    if (abs($yday - $yday0) > 1) {
      if ($yday < $yday0) 
	{ $yday = $yday0 + 1; }
      else 
	{ $yday0 = $yday + 1; }
    }

    my $tzdiff = 
      ((($yday*24 + $hour) * 60 + $min) -
       (($yday0*24 + $hour0) * 60 + $min0)) 
	* (100/60); # Convert to 0#00 form.  But is it decimal?

    # was: if ($year > 99) { $year %= 100; }
    $year += 1900;

    sprintf("%04d:%02d:%02d:%02d:%02d:%02d %+.4d",
      $year, $mon+1, $mday, $hour, $min, $sec, $tzdiff);
}

1;
