#!/usr/bin/perl
# mail-incoming.pl - Handler of mail incoming to HyperNews.
# Copyright Board of Trustees of the University of Illinois at UC, April 1996

#==================================

# The mail-incoming feature is out of beta testing - but please
# be cautious in its use.  For installation instructions see:
#
# http://www.hypernews.org/HyperNews/get/hypernews/email.html

# The command line for invoking mail-incoming is:
#  .../HyperNews/.scripts/mail-incoming.pl
# or
#  .../HyperNews/.scripts/mail-incoming.pl -f forum-dir

#      exit(75);  # This causes sendmail to queue the message and try later.

# We do attempt to avoid mail loops and
# spurious postings, but there may still be ways to cause loops. 
# A loop can occur if the .midDB is not correct for some reason,
# because then HyperNews can't tell that you have already posted a
# message.  It checks whether the message ID has already been seen,
# and if not, then it assumes this is a new message and proceeds to
# post it.  Make sure you have no filter on your mail that changes the
# message ID before mail-incoming.pl sees it. 

###################################
# Uncomment the next line to quickly stop an infinite mail loop in progress.
# exit 0;
###################################


# We have to do some things with @INC in order for this to compile right.
BEGIN {
    # Prepend HyperNews directories to load path.
    $0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
    $hnbin =~ s{/(.scripts|Admin|SECURED)}{}g;
    unshift(@INC, $hnbin, "$hnbin/.scripts"); 
}

require "Admin/hnrc";

require "flush.pl";
 
# Strip any HTML from output.
local $plaintext = 1;

# For debugging, we leave files HNmail.out and HNmail.old in /tmp.
$debug = 1;
if ($debug) {
  rename "/tmp/HNmail.out", "/tmp/HNmail.old";
  open(DEBUG, ">/tmp/HNmail.out");

  # Redirect STDOUT and STDERR to DEBUG
  open(SAVEERR, ">&STDERR");
  open(STDERR, ">&DEBUG") || die "Cant dup DEBUG for STDERR";
  open(STDOUT, ">&DEBUG") || die "Cant dup DEBUG for STDOUT";

  select(STDERR); $| = 1;     # make unbuffered
  select(STDOUT);  $| = 1;     # make unbuffered
  select(DEBUG);  $| = 1;	 # make unbuffered

  $debugHandle = "main::DEBUG";
}

$allowMIMEmail = 0;  # not working yet... and should be in hnrc


&feval('&MailIncomingPL()');

sub MailIncomingPL {

  local($header, $MimeEntity, %fields);
  local %l2o;
  local %l2o2;

  # Need to convert this to Getopt::Long
  require 'getopt.pl';
  &Getopt('f');
  $forum = $opt_f;
  print $debugHandle "Forum: $forum\n" if $debug;
  
  $handle = 'STDIN';
  
  # Get the body and the header... 
  $header = &MAILread_file_header("main::$handle", *fields, *l2o);
  my $MsgBody, @MimeParts;

  # Check for mail loop.
  print main::DEBUG "x-hn-loop is $fields{'x-hn-loop'}\n" if $debug;
  if ($fields{'x-hn-loop'}) {
	# Sent from HN already.  
	# This will need to be generalized to allow multiple HN sources.
    exit 0;
  }

  $userName;
  $from = $fields{'from'};
  ($fromAddress, $userName) = &ParseAddressName($from);
  $fromAddress ||= 'Unknown';  # Need some name.
  print $debugHandle "fromAddress = $fromAddress\n" if $debug;  
  print $debugHandle "userName = $userName\n" if $debug;  

  $envelopeSender = $fields{'from '};
  # Use only the first field - drop the date
  $envelopeSender =~ s,^\s*(\S+).*$,$1,;

  # Check to see if this is reject mail
  # We have to do this!
  my ($error) = &IsBogusMail($headerString);
  if($error) {
    # Send mail to administrator with rejected or bogus mail.
    &RedirectBogusEmail($mail_errors_to, $error, 
			$headerString, $body);
  }

  # Compute who gets the message.
  $whoto = $envelopeSender || $fields{'errors-to'} || 
    $fields{'sender'} || $mail_errors_to || 
      $fromAddress;   # If all else fails.
  if ($whoto eq $mail_to) {
    # Mailing to the recipient of the incoming message could cause a loop.
    exit 0;
  }

  # If it looks like an unknown header line, reject back to sender.
  #  This is being too conservative.  Allow the message to pass.
  # if ($body =~ m,^([^: ]+):\s*.*(\n|\$),) {
  #  &RedirectBogusEmail($whoto, "Unknown header in body of message: $1.", 
  #			$headerString, $body);
  #}


  # If MIME, call MIME extractor subroutine, else call old one.
  if ($allowMIMEmail && $header =~ m/Mime-Version:/i) { 
    require 'mail-mime.pl';
    ($MsgBody, @MimeParts) = &MimeExtractBody("main::$handle", $header); 
  }
  else { 
    $MsgBody = &MailExtractBody("main::$handle"); 
  }

  # Extract additional known header lines from the body.
  # This is to support mailers that do not allow user to add header lines.

  $MsgBody =~ s,^[ \t]*(\n|\$),,;  # Remove initial blank line
  print main::DEBUG 
    "---- Attempting to find additional headers in body\n" if $debug;

  while ($MsgBody && $MsgBody !~ m,^\s*(\n|\$),) {  # while not a blank line
    # print main::DEBUG "--inside matching loop\n" if $debug;
    if ($MsgBody =~
	# Allow leading " > " stuff.
	m,^([\s>]*(In-Reply-To|References|Newsgroups|Control):.*(\n|\$)),i) {
      my $line = $1;
      substr($MsgBody, 0, length ($line)) = '';  # delete it

      $line =~ s{^[\s>]*}{};  # remove leading junk.
      print main::DEBUG "---- matched the line: $line \n" if $debug;
      &MailAddField($line, *fields, *l2o);
    }
    else { last; }
  }

  # Additional Processing of the Body must be done here if To: or CC: are
  # HyperNews-request@hypernews.org. The contents will be Majordomo like
  # and after processing should be included in the fields array to continue
  # normally.

  $MsgBody =~ s,^[ \t]*(\n|\$),,;  # Remove initial blank line

  if ($debug) {
    print main::DEBUG ("\n---\nHeader String: \n$header\n");
  
    print main::DEBUG "---- Header Fields ---\n";
    local ($key);
    foreach $key (keys %fields) {
	print main::DEBUG "$key: '$fields{$key}'\n";
    }
    #print main::DEBUG "------\n";
  }

  print main::DEBUG "---- Body ---\n$MsgBody\n------\n" if $debug;
  print main::DEBUG "hnprefix is $hnprefix\n" if $debug;

  # Process MIME attachments
  if(@MimeParts) { 
    $MimePartsTarFile = &ProcessMIMEParts(@MimeParts); 
  }

  
  # Post the response to HyperNews via http
  &PostResponseFromEmail($header, $MsgBody, *fields);
  
}


sub MailExtractBody {
  # Read the remainder of the message.
  local($handle) = @_;
  my $data;
  while (my $line = <$handle>) {
      print main::DEBUG "BodyLine: $line\n";
      $data .= $line;
  }
  return($data);   
}


# This code is gross!!! Needs to be replaced proto!!

sub MAILread_file_header {
  local($handle, *fields, *l2o) = @_;
  #---modified from MHonArc - some old version
  ##-----------------------------------------------------------------------
  ## MAILread_file_header reads (and strips) a mail message header from the
  ## filehandle $handle.  *mesg is a pointer to the mail message.  *fields
  ## is a pointer to an associative array to put field values in indexed
  ## by field labels that have been converted to all lowercase.  *l2o
  ## is an associative array to get the original label text from the
  ## lowercase field label keys.  The return value is the original
  ## header text.
  ##	
  

  local($label, $tmp, $header);
  local($d) = ($/);
  
  $/ = "\n";  
  $label = '';
  $header = '';  
  # %fields = ();  %l2o = ();  Don't reinitialize here.

  # Process the "From " line specially, if there is one.
  $tmp = <$handle>;
  $header .= $tmp;
  local ($olabel, $value);
  if ($tmp =~ m/^From /) {
      ($olabel, $value) = $tmp =~ /^(From )\s*([\s\S]*)$/i;
      ($label = $olabel) =~ tr/A-Z/a-z/;
      $l2o{$label} = $olabel;
      $value =~ s/\s$//g;
      $fields{$label} = $value;
      $tmp = <$handle>;
  }

#  &printflush (DEBUG,  "tmp in headers = $tmp \n") if $debug; 

  # While there is another non-blank line.
  while ($tmp && $tmp !~ /^\s*$/) {
    $header .= $tmp;
    $tmp =~ s/\n/ /g;
    
    ## Check for continuation of a field
    if ($tmp =~ /^\s/) {
      $fields{$label} .= $tmp  if $label;
#      &printflush (DEBUG, "tmp in continued line = $tmp \n") if $debug; 
    }
    else {
	&MailAddField($tmp, *fields, *l2o);
    }
    # &printflush(DEBUG, "Finished with line $label\n") if $debug;
    $tmp = <$handle>;
  }

  $/ = $d;
  $header;
}


sub MailAddField {
  local ($headerline, *fields, *l2o) = @_;
  
#  &printflush (DEBUG, "headerline: $headerline\n") if $debug;
  ## Separate head from field text
  local ($olabel, $value) = $headerline =~ /^([^: ]+):\s*([\s\S]*)$/;
  $value =~ s/\s$//g;
  ($label = $olabel) =~ tr/A-Z/a-z/;
  $l2o{$label} = $olabel;
#  &printflush (DEBUG, "header{$label} = $value\n") if $debug;
  if ($fields{$label}) {
    $fields{$label} .=  ' ' . $value;
  } else {
    $fields{$label} = $value;
  }
}


sub ParseAddressName {
  my ($from) = @_;
  # Parse out the name and email address from a "From:" line.
  my $address;
  my $name;

  if ($from =~ 
      s{
	  ([A-Za-z0-9][A-Za-z0-9_=+%/\.\-]+ # userid part.
	   [@%]
	   [A-Za-z0-9_\.\-]+) # domain name part.
	  }
      {}x) 
  { $address = $1; }
  
  # Remove <> or () which might have surrounded address.
  $from =~ s{<>}{};
  $from =~ s{\(\)}{};

  $from =~ s{^\s*}{};     # remove leading spaces
  $from =~ s{\s*$ }{}x;   # remove trailing spaces

  $from =~ s{\"}{}g;           # remove all "s
  $from =~ s{^[\"\(\s*]}{};    # remove leading ( or space
  $from =~ s{\s*[\"\)]$ }{}x;  # remove trailing space or )

  # What remains is the name.

  $name = $from;

  return ($address, $name);
}



sub IsBogusMail {
  # This function returns an error message if the mail headers looks 
  # like rejected or suspicious mail.  Returns 0 otherwise.
  my ($headers) = @_;

  if ($headers =~ m{
    ^ (((Resent-)?(From|Sender) | X-Envelope-From):\s* |
       >?From\ )  # Is this right?
      (([^@]*[^\(.%\@a-z0-9])? # Is this right?
       ((Post(ma?(st(e?r)?|n))|office) |
	(send)?Mail(er)? |	# e.g. Mail Delivery Subsystem, MAILER-DAEMON
	daemon|mmdf|root|n?uucp|smtp|bounce|
	serv(ices?|er) |
	Admin(istrator)? |
	vacation |
	message\s*agent
       )
       ([^\).!:a-z0-9].*)?)
		   }mix)
    {
      return "The message was sent from (or sent by) '$5' which
 suggests this is an administrative message.";
    }

  if ($headers =~ m{
	 ^(Precedence:\s*(junk|bulk))  # Allow this? How about list?
		   }mix) 
    {
      return "The Precedence is '$2' which
 suggests this is an administrative message.";
    }

  if ($headers =~ m{
	 ^(Message-Type:\s*(Delivery\ Report))
		   }mix) 
    {
      return "The Message Type is '$2' which
 suggests this is an administrative message.";
    }

  if ($headers =~ m{
	^(Content-Type:\s*(multipart/report;\ report-type=delivery-status))
		   }mix) 
    {
      return "The Content Type is '$2' which
 suggests this is an administrative message.";
    }

  if ($headers =~ m{
	^(Auto-Submitted:\s*(auto-generated\ \(failure\)))
		   }mix) 
    {
      return "The Auto-Submitted header is '$2' which
 suggests this is an administrative message.";
    }

  if ($headers =~ m{
	  ^(Subject:\s*
	   ((RCPT:\b) |
	    (Receipt\ Confirmation\b) |
	    (Non-Receipt\ Notification\b) |
	    (Undeliverable\ (Message|Mail)\b) |
	    (unable to deliver mail\b) |
	    (Message\ not\ deliverable\b) |
	    #  e.g. "Host unknown", "User unknown", "Undeliverable",
	    #  "Local configuration error", " - nameserver error report",
	    #  "Cannot send message", "All recipients suppressed", "Bad usage",
	    #  "Deferred:"
	    (Warning:\ could\ not\ send\ message) |
	    (Returned\ mail\b) |
	    (Mail\ System\ Error\ -\ Returned\ Mail) |
	    (mail\ fail(ed|ure)\b) |
	    (Failed\ mail\b) |
	    (Delivery\ failure\ notification) |
	    (Delivery\ problems\ with\ your\ mail\b) |
	    (Delivery(-| )Report\b) |
	    (Delivery\ of\ Certified\ mail\b) |
	    (REJECTED:) |
	    (Invalid\ destination\b) |
	    (Service Message\b) |
	    (Waiting Mail\b) |
	    (AUTO-REPLY\b) |
	    # Vacation bounces
	    (Out\ of\ the\ office)
	    (.*\baway from my mail\b)
	   ))
      }mix) {
      # Reject it.
      return "The subject, '$2',
 suggests this is an administrative message.";
    }
  return 0;
}


sub RedirectBogusEmail {
  # Send the mail to HyperNews $mail_errors_to
  local($whoto, $msg, $header, $body) = @_;
  require "email.pl";  

  &sendmail(HYPERNEWS, $whoto);
  print HYPERNEWS "Subject: $hnSystemName Posting Error

$msg

   ----- Unsent message follows -----
$header

$body";
  
  close(HYPERNEWS);
  exit 0;
}


sub PostResponseFromEmail {
  local ($headerString, $body, *fields) = @_;

  require "http.pl";

  # If To: or Cc: is HyperNews-request, then command in body is OK.
  if ($body =~ m,^(un)?(sub?)?scr?ibe?,i) {
    &RedirectBogusEmail($whoto, 
"It appears you have sent a subscribe or unsubscribe
command to the wrong place.  We don't handle these requests via
email yet anyway.  Please do these commands via $hnSystemName on the web.", 
			$headerString, $body);
    # We should offer a URL.
  }

  if (!$allowEmailIn) {
    &RedirectBogusEmail($whoto, "Incoming email to $hnSystemName is currently 
deactivated.  Please post via $hnSystemName on the web.", 
$headerString, $body);
    # We should offer a URL.
  }

  print DEBUG "Got past IsBogusMail\n" if $debug;


  local(%articleInfo, %MemberInfo);
  local($midhost, $password);
    
  #-----Assign Values from the Fields array to local variables

  my $title = $fields{'subject'};
  print $debugHandle ("title1: '$title'\n") if $debug;
  if ($title =~ "^[ \t\n]*\$") { $title = "Untitled"; }
  $title = &URLencode($title);
  print $debugHandle ("title3: '$title'\n") if $debug;
  if ($hnSystemAbbrev && $title =~ m,$hnSystemAbbrev,) {
	# Remove reference.
	$title =~ s,\(ref: $hnSystemAbbrev\),,;
  }

  $upmsg = $fields{'in-reply-to'};  #  || $fields{'references'};
# if ($upmsg =~ m,<\s*([^\s<>]+)\s*>,) {
#    $upmsg = $1;
#    print $debugHandle "upmsg is $upmsg\n" if $debug;
#  }

  my $mid = $fields{'message-id'} || $fields{'resent-message-id'};
  #my $path = $fields{'path'};
  
  print $debugHandle "mid = $mid\n" if $debug;

   
  #MID(Incoming)
  #if ($mid =~ m/<\s*([^@]+)@([A-Za-z0-9_\.\-]*)\s*>/) {
  #  $mid = "$1\@$2"; }

  $articleInfo{'userid'} = $fromAddress;
  $articleInfo{'userName'} = $userName;
  print $debugHandle "userName: '$userName'\n" if $debug;
 
  $articleInfo{'remoteUser'} = $mail_admin;
  $articleInfo{'adminPassword'} = $mail_admin_password;

  #$articleInfo{'URL'} = $URL;
  $articleInfo{'title'} = $title;
  $articleInfo{'upRel'} = 'None';

  $contentType = 'Plain Text';  # Could be Smart Text.
  $articleInfo{'content-type'} = $fields{'content-type'};
  
  $articleInfo{'contentType'} = $contentType;
  $articleInfo{'annotationType'} = 'Message';

  $articleInfo{'body'} = $body;

  $articleInfo{'Keywords'} = "";

  $articleInfo{'notify'} = 'Notify';

  $articleInfo{'messageID'} = $mid;
  #$articleInfo{'path'} = $path;
  $articleInfo{'inreplyto'} = $fields{'in-reply-to'};
  # Pass the whole references field.
  $articleInfo{'references'} = $fields{'references'};
  
  $articleInfo{'headerString'} = $headerString;
  $articleInfo{'sender'} = $fields{'sender'};
  $articleInfo{'envelope-sender'} = $envelopeSender;
  $articleInfo{'errors-to'} = $fields{'errors-to'};
  $articleInfo{'to'} = $fields{'to'};
  $articleInfo{'cc'} = $fields{'cc'};
  $articleInfo{'newsgroups'} = $fields{'newsgroups'};
  $articleInfo{'forum'} = $forum;  # may be null
  $articleInfo{'Date'} = $fields{'date'};
  $articleInfo{'control'} = $fields{'control'};
  
  #----Additional fields are processed here
  if ($fields{'x-hn-relationship'}) {
    $articleInfo{'upRel'} = $bodyfields{'x-hn-relationship'};
  }
  
  #----Posting Message via HyperNews http
  local($addResponseScriptPath) = $addResponseURL;
  # Dont need http and server prefix.
  $addResponseScriptPath =~ s,$http($ENV{SERVER_NAME}(:$ENV{SERVER_PORT})?)?,,;

  if ($fields{'control'}) {
      &PostEmailCommand(%articleInfo);
  } else {
    print main::DEBUG "addResponseURL = $addResponseURL\n" if $debug;

    print main::DEBUG "http = $http\n" if $debug;
    print main::DEBUG qq{
POSTing to $ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}, $addResponseScriptPath
} if $debug;
  
    $articleinfo{attachment} = [ $MimePartsTarFile ] if $MimePartsTarFile;

    require "base64.pl";
    my $credentials = &base64::b64encode("$mail_admin:$mail_admin_password");

    # Get the hostname of this host, not the server we are connecting to.
    # Using $ENV{'SERVER_NAME'} is wrong in the case that your mail host
    # is not the same as your web host.
    $thishost = $ENV{'SERVER_NAME'};
    #chop($thishost = `/bin/hostname`);

#    use LWP::UserAgent;
#    use HTTP::RequestMaker;
#    use HTTP::Response;
#    use LWP::Debug qw(level); level('+');

#    &LWP::Debug::conns("Debugging $hnSystemName");
#    &LWP::Debug::debug("Where does this go?");

#    my $addURL = "http://$ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}$addResponseScriptPath";
#    print main::DEBUG "addURL: $addURL\n";

#    my $ua = LWP::UserAgent->new;

#    my $response = $ua->request
#      (POST $addURL,
#       From => "$mail_admin@$thishost",
#       Authorization => "Basic $credentials",
#       Content_Type => 'form-data',
#       Content      => \%articleInfo
#      );


    ($page, $status) = &http::post
      ($ENV{'SERVER_NAME'}, $ENV{'SERVER_PORT'}, $addResponseScriptPath,
       "main::DEBUG", $mail_admin, ($mail_admin_password || ''), 
       %articleInfo);    

    $page = &TextorHTML($page);  # Convert to plaintext by stripping it.
    
    #my $status = $response->code();

    print main::DEBUG "status: $status\n";

    if ($status =~ m{408|500|503|504}) {
      #Delay posting because HyperNews server is unavailable
      #408-Request timeout
      #500-Internal Server Error
      #503-Service Unavailable
      #504-Gateway Timeout
      print $debugHandle "Status: $status" if $debug;
      close($debugHandle) if $debug;
      exit(75);  # This causes sendmail to queue the message and try later.
    }
    
    if (!$status || ($status !~ m{2..|3..})) {
      # This is an error that the user and admin should know about.
      # Returning something to the sender is dangerous because it 
      # might cause a mail loop if the sender replies again.
      if ($whoto ne $mail_errors_to) {
	$whoto = join (', ', ($whoto, $mail_errors_to));
      }
      if ($whoto != m,$fromAddress,) {
	$whoto = join (', ', ($whoto, $fromAddress));
      }
      &RedirectBogusEmail($whoto,
"$page\n", $headerString, $body, "\nStatus '$status' returned.\n");
    }
    
  }
  #----Mostly debugging stuff here
  print $debugHandle (" Returned from post Status= $status \n$page") 
    if $debug;
  close($debugHandle) if $debug;
}


