# move-lib.pl - HyperNews move (or copy) message tree utility.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

# This code is extremely tedious.  Something needs to be done to
# improve its clarity and power.  I'm thinking that we should just
# construct HTTP requests to post the pieces at the other end.
# This would take longer, but it would allow us to do replication,
# or general moving between servers which we also need.
# The time factor can be reduced by using keep-alive.
# We could also queue requests to be handled as time permits.

require 'log-lib.pl';
require 'add-lib.pl';
require 'lock.pl';

sub move {
    local($sourceURL, $destURL, $deleteFlag, $notifyFlag, $asBase) = @_;
    # Move a tree of responses from sourceURL to destURL.
    # sourceURL and destURL should be hn relative.
    # If deleteFlag, delete the source tree leaving a placeholder.
    # If notifyFlag, notify subscribers at destination.
    # If $asBase, then if $destURL does not exist, then make
    # a new base article, if allowed.  Not supported yet.

    local $move_debug = 0;

    require "delete-lib.pl";
    &HNMsg ("source: $sourceURL\n") if $move_debug;
    &HNMsg ("dest: $destURL\n") if $move_debug;

    if (!-e "$hndocs$sourceURL,urc") {
      &HNUserError ("Source '$sourceURL' doesn't exist!");
    }

    if (!-e "$hndocs$destURL,urc") {
      if (!$asBase) {
	&HNUserError ("Destination '$destURL' doesn't exist!
Maybe <A HREF=\"$editArticleURL$destURL\">create it first</A>.");
      }
      # &responseAsBase($sourceURL, $destURL);    Not done yet.
    }

    # Get info about sourceURL.
    my %source = &parseURCFile("$hndocs$sourceURL,urc");

    my $sourceBase = $source{'BaseURL'};

    if ($sourceBase eq $sourceResponses) {
	&HNUserError("Currently, HyperNews can NOT move base articles.");
    }

    &logTransaction("Copy from $sourceURL to", $destURL);

    my $sourceResponses = $source{Responses};
    $sourceResponses = &untaintPath($sourceResponses);
    my $sourceResponsesPath = "$hndocs$sourceResponses";

    my $newResponsePath = $sourceResponsesPath;
    $newResponsePath =~ s/[^\/]*$//;

    my $sourceParentPath = $newResponsePath; # to prevent copying to same dir.
    $sourceParentPath =~ s,/$,,; 

    $newResponsePath .= "newResponse$$"; 

    &HNMsg ("source responses path: $sourceResponsesPath\n") if $move_debug;

    # Put body in tmp file as AddResponse expects.  (kludge!!)
    my $bodyURL = $source{'Body'};
    my $responseFile;
    if ($bodyURL =~ m,$sourceResponses-(body|content).html$,) {
      $responseFile = "hn_tmp$$";
      $source{'Body'} = "/$responseFile";
      # $contentType = 'HTML'; not necessarily - use what was there.
      #open(FILE, ">$hndocs/$responseFile");  # What's this for?
      #close(FILE);
    } else {
      $source{'ContentType'} = 'URL';
    }
    
    &HNMsg ("body: $bodyURL\n") if $move_debug;

    # Get info about destURL.
    my %dest = &parseURCFile("$hndocs$destURL,urc");
    &HNMsg ("Assigned assosiative array...\n") if $move_debug;

    $destResponses = $dest{Responses};
    $destResponses = &untaintPath($destResponses);
    $destResponsesPath = "$hndocs$destResponses";

    &HNMsg ("$sourceParentPath, $destResponsesPath\n") if $move_debug; 
    if ($destResponsesPath eq $sourceParentPath) {
      &HNUserError("<b>Cannot copy source to the same place</b>: $destResponsesPath");
    }
    if ($destResponsesPath =~ m,^$sourceResponsesPath(/[0-9]+)*$,) {
      &HNUserError("<b>Cannot copy source to a subdirectory of itself</b>: <BR>
 $destResponsesPath is in $sourceParentPath");
    }
      
    local ($relativeUpURL) = $dest{'URL'};
    $relativeUpURL =~ s,^.*/get(\.cgi)?/,/,;
    $source{'URL'} = $relativeUpURL;

    $source{'BaseURL'} = $dest{'BaseURL'};
    $source{'Responses'} = $destResponses;
    $source{'Moved-From'} = $sourceURL;  # This gets lost by AddResponse.

    # Unescape things that are escaped, so AddResponse works.
    $source{'Title'} = &unescapeHTML($source{'Title'});
    $source{'Keywords'} = &unescapeHTML($source{'Keywords'});
    $source{'Name'} = &unescapeHTML($source{'Name'});

    &HNMsg ("dest path: $destResponsesPath\n") if $move_debug;

    # Should check after creating.
    if (! -d "$destResponsesPath/newResponse$$") {
        if (!-d "$destResponsesPath") {
          mkdir ("$destResponsesPath", "$hndirmode");
	  chmod ("$hndirmode", "$destResponsesPath");
          &HNMsg ("making $destResponsesPath...\n") if $move_debug;
        }
	mkdir("$destResponsesPath/newResponse$$", "$hndirmode");
	chmod ("$hndirmode", "$destResponsesPath/newResponse$$");
        &HNMsg ("made dir: $destResponsesPath/newResponse$$...\n") 
	  if $move_debug; 
    }

    # Lock the source-URL tree
    &lock_tree($sourceResponsesPath);

    # Lock the dest-URL directory
    &lock($destResponsesPath);

    &HNMsg ("Source ResponsesPath is: $sourceResponsesPath\n") if $move_debug;
    if (-d $sourceResponsesPath) {
	# Copy the source-URL tree to the dest-URL 
        &HNMsg ("About to rename $sourceResponsesPath to $newResponsePath\n")
	            if $move_debug;
	rename ($sourceResponsesPath, $newResponsePath);
        &HNMsg ("About to copy $newResponsePath to $destResponsesPath\n")
	            if $move_debug; 
	&doexec("cp", "-r", $newResponsePath, "$destResponsesPath");
        &HNMsg ("Renaming $newResponsePath back to $sourceResponsesPath\n")
	            if $move_debug;
	rename ($newResponsePath, $sourceResponsesPath);
    }	
    if ($responseFile) {
      if (-e "$sourceResponsesPath-body.html") {
	&doexec('cp', "$sourceResponsesPath-body.html", 
		"$hndocs/$responseFile");  
      } elsif (-e "$sourceResponsesPath-content.html") {
	&doexec('cp', "$sourceResponsesPath-content.html", 
		"$hndocs/$responseFile");
      }
    }

    # Make the new tree a new response.
    $newResponseNum = &AddResponse($notifyFlag, 'moving', %source);
    &HNMsg ("new response: $newResponseNum\n") if $move_debug;

    unlink ("$hndocs/$responseFile");     # should be gone already.

    if (-d "$destResponsesPath/newResponse$$") {
	rename ("$destResponsesPath/newResponse$$", 
		"$destResponsesPath/$newResponseNum");
    }
    &fixURLs("$destResponses/$newResponseNum.html",
             "$destResponses/$newResponseNum", $dest{'BaseURL'},
             "$getURL$destResponses/$newResponseNum.html");

    # &updateResponseList($destResponses);
    &clearResponseList($destResponses);
    &unlock($destResponsesPath);
    &unlock_tree($destResponsesPath/$newResponseNum);

    # Don't leaving locks.
    if (-d $sourceResponsesPath) {  # Is this check right?
	&unlock_tree($sourceResponsesPath);
    }
    else {
      &unlock($sourceResponsesPath);
    }

    if ($deleteFlag == 1) {
        #Check access control for delete
        &access1($deleteSF, 2, $owner, $remoteUser, 
		 "User can't delete the response after copying.", 
		 $authenticated, $base);
	&deleteTree($sourceURL, '1 month', 
		    "$getURL$destResponses/$newResponseNum.html");
	# Need to delete the old body too, if any!!
	unlink ("$sourceResponsesPath-body.html");
    }
}



sub responseAsBase {
  my ($responseURL, $distination) = @_;
  # This code creates a new base article from a response.
  # Doesnt work yet.

  # Make sure user is allowed to create a new base,
  # by checking access to existing base of same
  # name, or access to containing article found by
  # dropping off last component of name.
  
  # Loop to find first URL above $destination that exists.
  my $tmpURL = $destination;
  while ($tmpURL && !-e "$hndocs$tmpURL,urc") {
    # Chop off last component and try again.
    $tmpURL =~ s,/[^/]*$,.html,;
  }
  
  if (!$tmpURL) {
    # &HNUserError ("Can't find anything above $destination that exists!");
    # This could be OK anyway, if remoteUser is Admin, to create new
    # base article off the root.  Disallow for now.
    &HNUserError("Can't create new base article at root level yet.");
  }
  
  # Check that we are not within responses.
  my $root;
  if ($tmpURL =~ m,/([^/]+)$,) {
    $root = $1;
    if ($root =~ m,^[0-9]+$,) {
      # This is a response, so ...
      &HNUserError("Response '$destination' does not exist");
    }
  }
  

  my %article;
  my $originalOwner;
  my $base;
  
  if ($tmpURL) {
    %article = &parseURCFile("$hndocs$tmpURL,urc");
    
    $originalOwner = $article{'From'};
    $base = $article{'BaseURL'};
    &HNMsg("Article URL:$hndocs$tmpURL,urc
 Owner: $originalOwner\n") if $debug;
  }
  
  &access1($addArticleSF, 2, $originalOwner, $remoteUser,
	   "Article $tmpURL already exists, owned by 
 <strong> $originalOwner </strong>.
 <strong> $remoteUser </strong> can't overwrite!",
	   $authenticated, $base);
  
  # OK, now create the new article, using
  # the parent article, if any, for default values.
  
  my %response = &parseURCFile("$hndocs$responseURL,urc");
  my %newarticle = %response;  # Copy defaults from response.
  $newarticle{'HeaderURL'} = $article{'HeaderURL'},
  $newarticle{'FooterURL'} = $article{'FooterURL'},
  $newarticle{'UpURL'} = '';	# could be $article{'URL'}
  $newarticle{'Responses'} = $article{'Responses'} . $root;
  $newarticle{'Moderation'} = $article{'Moderation'};
  $newarticle{'DefaultOutlineDepth'} =
    $article{'DefaultOutlineDepth'};
  
  &AddArticle(0, %newarticle);
  # Now the destination should exist.
  
}


sub fixURLs {
    local($relativeTopURL, $responsesDir, $base, $url) = @_;
    # Recursively update the URL, UpURL, BodyURL, Base, Responses
    # NumMessages, etc, for every message in $relativeTopURL.
    # This will typically be called before there are multiple users of the
    # new tree, so locking is not so crucial.
    local $move_debug = 0;
    &HNMsg ("\$relativeTopURL is $relativeTopURL\n") if $move_debug;
    &HNMsg ("Call to Fix Ref for:$hndocs$relativeTopURL,urc\n") if $move_debug;
    &HNMsg ("parsing: $hndocs$relativeTopURL,urc\n") if $move_debug;
    local($responsesPath) = &untaintPath("$hndocs$responsesDir");

    &HNMsg ("response:$responsesDir\n") if $move_debug;
    &HNMsg ("Base: $base\n") if $move_debug;
    &HNMsg ("URL: $url\n") if $move_debug;

    # Scan response directory for files
    &HNMsg ("Opendir $responsesPath\n") if $move_debug;

    # Checking content of directory
    opendir (DIR, $responsesPath);
    local(@files) = readdir DIR;
    closedir DIR;

    @files = grep (/[0-9]+\.html,urc/, @files);
    @files = sort {$a <=> $b} @files;
    &HNMsg ("The directory files are @files\n") if $move_debug;

    # Correct each file
    local($PreviousNum) = "";
    local($localcount) = 0;
    local(%oldin);
    local(%in);
    local($destinationURC);
    local($responsenum);
    my $totalNumMessages = 0;

    foreach $file (@files) {
	$responsenum = $file;
	$responsenum =~ s/\.html,urc$//;
	&HNMsg ("Parsing $responsesPath/$responsenum.html,urc \n")
            if $move_debug;

	%in = &parseURCFile("$responsesPath/$responsenum.html,urc");
	next if (!$in{'URL'});  # Is it a phantom?

	local($oldResponses) = $in{'Responses'};
	local($newResponses) = &untaintPath("$responsesDir/$responsenum");
	local($newURL) = "$getURL$newResponses.html";
	local($newBody) = $in{'Body'};

	if ($newBody =~ m,$oldResponses-body.html$,) {
	    $newBody = "$newResponses-body.html";
	}
	elsif ($newBody =~ m,$oldResponses-content.html$,) {
	    # We should really rename the file.
	    $newBody = "$newResponses-content.html";
	}

        $oldin{'NextNum'} = $responsenum;
        &HNMsg("count is $localcount<P>") if $move_debug;
        if ($localcount > 0) {
	   &HNMsg("writing URC($destinationURC) now<P>") if $move_debug;
           # We delayed writing previous urc until now to get the correct NextNum;
	   &writeURC($destinationURC, %oldin);
        }
        $localcount++;
	$totolNumMessages += 1;

	$in{'Body'} = $newBody;
	$in{'Responses'} = $newResponses;
	$in{'URL'} = $newURL;
	$in{'BaseURL'} = $base;  
	$in{'UpURL'} = $url;
        $in{'PreviousNum'} = $PreviousNum;        
        $in{'NextNum'} = "";
	$destinationURC = &untaintPath("$responsesPath/$responsenum.html,urc");
        $PreviousNum = $responsenum;

        %oldin = %in;
	&HNMsg ("
URL: $newURL
Body: $newBody
UpURL: $url
BaseURL: $base
Responses: $newResponses
") if $move_debug;

	# Do this recursively for the subdirectory, if it exists.
        &HNMsg ("Checking if $hndocs$responsesDir/$responsenum exists\n") 
	    if $move_debug;
        if (-d "$hndocs$responsesDir/$responsenum") {
	  my $numMessages = &fixURLs("$responsesDir/$responsenum.html",
				  $newResponses, $base, $newURL);
	  $oldin{'NumMessages'} = $numMessages;
	  $totolNumMessages += $numMessages;
        }
    }

    # Write the last one.
    $filecount = @files;
    &HNMsg("Writing URC ($destinationURC) directly 
$localcount = $filecount<P>") if $move_debug;
    my $urc_file_name = &untaintPath("$responsesPath/$responsenum.html,urc");
    &writeURC($urc_file_name, %oldin);

    # Now update the responses list for this directory.
    &HNMsg ("Calling updateResponseList for $responsesDir\n") if $move_debug;
    # &updateResponseList($responsesDir);
    &clearResponseList($responsesDir);

    return $totalNumMessages;
}


1;
