#!/usr/bin/perl
# setup-form.pl - generates a form for configuring the system.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

# Run this via your web browser with a URL like:
# http://your.server.com/cgi-bin/HyperNews/.scripts/setup-form.pl/

# If you are having trouble seeing any output, try uncommenting this:
# print "Content-type: text/plain\n\n";

#####################
# Work around error: "Can't locate /dev/.scripts/*"
# Only works if current dir is HyperNews or lower.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};

$0 =~ m{^(.*)/[^/]+$ }x;  # Get last component.
$hnbin = $1 || '.';  # If $0 is not a path, use current directory.

$scriptsdir = ".scripts"; # The directory of HyperNews scripts and libraries.
$securedir = "SECURED";	# Members only directory within $hnbin
$admindir = "Admin"; # Admin only directory.

$hnbin =~ s{/?$scriptsdir$ }{}x;  # Do this first.
$hnbin =~ s{/?$admindir$ }{}x;

if (!$hnbin || $hnbin eq '.') {
    # If the above fails, try this:
    $hnbin = `pwd`;
    chop($hnbin);

    # Then try again to remove components.
    $hnbin =~ s{/?$scriptsdir$ }{}x;  # Do this first.
    $hnbin =~ s{/?$admindir$ }{}x;
}

# If all else fails, just try current directory.
$hnbin = '.' if !$hnbin;

# Make hnlib be $scriptsdir, inside $hnbin.
$hnlib = "$hnbin/$scriptsdir";

unshift(@INC, $hnbin, $hnlib); 
####################


require "hn-lib.pl" ||
    die ("Installation Error!: Can't load $hnlib/hn-lib.pl");

local $debug = 0;

&feval('&setupForm()');

sub setupForm {

    # 5.001m is OK in most cases, but not always, and not 5.001.
    require 5.002;

    require "setup-lib.pl";
    &setup_form_context();

    ################################
    require "member-lib.pl";

#==================================
    &HNMsg("Check authentication and access control here.") if $debug;

    require "formlib.pl";
    my %form = &GetFormArgs();
    
    my $userid = $ENV{REMOTE_USER} || $form{UserID};
    my $password = $form{Password};

    &checkAuthenticated('', '', $userid, $password);

    # We always require admin if .htaccess exists or $authenticated
    # or manual or external security and @hnAdmin. 
    if ($authenticated ||
	-e "$hnlib/.htaccess" || 
	-e "$hnlib/.www_acl" || -e "$hnlib/.nsconfig"
	|| (($externalSecurity || $manualSecurity) && @hnAdmin)) {

	if (!$authenticated) {
	    my $thisURL = &URLencode($ENV{REQUEST_URI} || 
				     "$setup_long_form_URL");
	
	    my $redirectURL = "$loginURL?url=$thisURL";
	    &HNMsg("Try logging in first.") if $debug;
	    &printRedirect($redirectURL);
	}

	&access1(3, 2, $remoteUser, $remoteUser,
		 "Setup must be executed by an administrator.
<strong> $remoteUser </strong> is not an administrator!",
		 $authenticated);
    }

    ################################

    &print_HTML_HEAD ("HyperNews Setup Form");

    my $whoami = &sustring($remoteUser);
    if ($whoami) { $whoami = "You are: $whoami <BR>"; }


    print qq{
<H2>HyperNews Setup</H2>

$whoami

<FORM ACTION="$setupURL" METHOD="POST"> 

Before filling out this form, skim through it, and then read the
instructions on submitting at the end.  Also, you should carefully
read the installation instructions starting on the <A
HREF="http://www.hypernews.org/HyperNews/get/hypernews/source.html">
HyperNews source page</A>.  You may use the source.html page that
comes with the distribution you have but the latest version will
generally be more up to date.  Also see the <A 
HREF="http://www.hypernews.org/HyperNews/get/hypernews/bugs.html">
HyperNews Bugs</A> page for any changes
you may have to make.
<p>

This form is for setting up or changing the configuration options that
control how HyperNews version $hnversion operates at this site:
($ENV{SERVER_NAME}).  All the configuration options are stored in a
file called <tt>hnrc</tt> that is used by all the other HyperNews
scripts.  The first time you visit this setup form (or use another
setup method, described next), the <tt>hnrc</tt> file will not exist,
and default values will be given for all options.  After the file is
created, the next time you visit this form (unless you use your
browser Back button to revisit it), the values that are found in 
the <tt>hnrc</tt> file are used. <P>

Instead of using this form, you can run <tt>setup.pl</tt> in the
.scripts directory.  Run it once to create the <tt>hnrc</tt> file with
default values, edit that file, and then run <tt>setup.pl</tt>
again.  A couple more options are available if you manually edit the
<tt>hnrc</tt> file.  After editing the file, you can either use
this form or the command line.  One other option is that you can use
the <A HREF="$setup_short_form_URL">short form</A>, which is a simpler
version of this form.  But read the documentation in this form in any
case.  <P>

Both setup-form.pl and setup.pl look for <tt>hnrc</tt> in the
HyperNews directory called \$hnbin/Admin, currently
"<tt>$hnbin/Admin</tt>".  For compatibility with old versions of
HyperNews, we also check \$hnbin and \$hnbin/.scripts.  (These
variables that reference directories are always computed from the URL
used to access each script so you do not need to set them.)  <P>

<strong>$hnrcString</strong><P>

Each time you submit this setup form, the <tt>hnrc</tt> file will be
changed in "<tt>$hnbin/Admin</tt>".  The old version of this file will be
moved to <tt>hnrc-old</tt>.  If an error occurs, you can back up to
this form (which will still have the values you used) to correct the
problem and resubmit. <P>

It is possible to install HyperNews multiple times at the same site,
each time with different options.  In fact, the $scriptsdir
directories of each may be linked to a single copy, but you must use
the appropriate URL to access this form, and be sure to fill in the
\$hnGroup field below.  You may also want to define separate document
directories for each installation and protect them against direct http
access except by administrators.  All installations may share the same
set of users and passwords, and a different group of users may be
given access to each.<P>

When filling in values for the following options, as a convenience,
you can use the values of previously defined options (in the order
presented in this list) just by using the variable name for the
option.  For example, \$http is used several times below.  Another
variable that is available is \$hndocs, which is the directory where
the HyperNews forums and messages are stored, currently
"<tt>$hndocs</tt>". <P>

};

# Break this up into smaller chunks - some Perls have problems otherwise.

    print qq{
<HR>

<H2>HyperNews URLs</H2>

\$http is the URL prefix for your server; it is used in several other
places below to generate URLs for documents and scripts.  It can be a
fixed value such as <tt>http://www.hypernews.org</tt> or a computed value
using "\$ENV{SERVER_NAME}" (which is "$ENV{SERVER_NAME}" for you) and
\$ENV{SERVER_PORT} (which is $ENV{SERVER_PORT}).  
Do not include a trailing "/".  <P>

<INPUT NAME="http" SIZE=60 MAXLENGTH=120 VALUE="$http"> <p>


<HR>


<H2>HyperNews Documents Directory</H2>

\$hndocs is the full path to the directory where your HyperNews
forums and messages are stored.  For each forum, a
subdirectory will be created in \$hndocs to store message headers.
Message bodies will be stored there too (except when a URL is
provided as the message body). <P>

\$hndocs may be the same as the httpd root document directory, or some
subdirectory.  If you use a subdirectory of the document root such as
"hn", which is recommended, be sure the name is not the same as the
name of the directory where the HyperNews scripts are stored or you
and your server will be confused.  Actually, \$hndocs doesn\'t need to
be accessible to the http server, just to the HyperNews scripts.  But
if you make it inaccessible to the server, make sure the icons are
still accessible (see \$hnicons below). <P>

The user running your HyperNews programs is "<B>$user_name</B>".  
This user must be able
to write in the \$hndocs directory.  (Alternatively, the directory
could be group writable by the group the user is in, "<B>$group_name</B>").
You should create this directory yourself being sure to set the
correct access mode.  HyperNews will try to do it when needed but it
will only succeed if the parent directory is also writable. <P>

HyperNews documents directory full path (\$hndocs):
<INPUT NAME ="hndocs" SIZE=60 MAXLENGTH=120 VALUE="$hndocs"> <P>

When a user looks up a HyperNews URL that no longer exists, or perhaps
they typed it in wrong, they will be referred to another document.
\$hnRootDoc is the document that is referenced if nothing else can be
found.  This could be your top level list of HyperNews pages, for
example.  You may specify this URL as either a HyperNews path relative
to \$hndocs (starting with a '/'), or as any URL. <P>

Root Document (\$hnRootDoc):
<INPUT NAME="hnRootDoc" VALUE="$hnRootDoc" SIZE=60 MAXLENGTH=120>
<p>

<HR>

};

    print qq{


<H2>Formatting Options</H2>

\$global_body_tag is used to define the body tag at the beginning of
the HTML body when displaying forums and messages. You could use this
to give every page a background color or image.  It defaults to a
normal &lt;BODY&gt;.  <P>

Global Body Tag (\$global_body_tag):<BR>
<TEXTAREA NAME ="global_body_tag" ROWS=3 COLS=60>$global_body_tag_value_encoded</TEXTAREA><P>

Header and footer text are included at the top and bottom of each forum
and message.  These may be overridden for each forum.
This could be used to include a standard message or image map, for
example.  You could wrap everything in a table, but you can\'t use this
option to wrap with frames.  <P>

Alternatively, the header or footer text can be a simple http URL, or
a file name, in which case the document it refers to is fetched, and
any relative URLs in the document are fixed.  A file name may start
with '/', in which case it will be used as if relative to the \$hndocs
directory, not your file system root.  A relative path starting with a
non-/ character is also relative to the top level \$hndocs directory
because this is the global context.  <P>

Header Text (\$headerText):<BR>
<TEXTAREA NAME ="headerText" ROWS=3 COLS=60>$headerTextEncoded</TEXTAREA><P>

Footer Text (\$footerText):<BR>
<TEXTAREA NAME ="footerText" ROWS=3 COLS=60>$footerTextEncoded</TEXTAREA><P>


(Also see \$startMsgBodyTag and \$endMsgBodyTag in the hnrc file.)
<P>

\$globalDefaultOutlineDepth is the maximum number of levels of messages to
display when viewing the message outline on any page.  It may be
overridden on each forum.  Use "-1" for All. <p>

Default depth of the outline display (\$globalDefaultOutlineDepth): 
<INPUT NAME="globalDefaultOutlineDepth" 
       VALUE="$globalDefaultOutlineDepth" SIZE=5 MAXLENGTH =20>
<p> 

<HR>

<H2>Icons</H2>

The icons directory (\$hnicons) contains the icons used to show
the kind of message.  The
mapping between kinds of messages and icons is in \$hnicons/icons-table,
the default value of which is copied from $hnlib/icons-table-dist -
you should edit that to your liking before submitting or edit the actual
table after that.<P>

Icons directory full path (\$hnicons):
<INPUT NAME ="hnicons" SIZE=60 MAXLENGTH=120 VALUE="$hnicons"> <P>

Full URL for the same directory as \$hnicons (\$hniconsURL):
<INPUT NAME ="hniconsURL" SIZE=60 MAXLENGTH=120 VALUE="$hniconsURL"> <P>

You only need to copy icons from the distribution once each time you
install a new version of HyperNews, or not at all if you don\'t plan to
use the distributed icons. <P>

<INPUT TYPE="radio" NAME="iconsyn" VALUE="yes" $iconsyn{'yes'}> Copy Icons
to \$hnicons. <BR>
<INPUT TYPE="radio" NAME="iconsyn" VALUE="no" $iconsyn{'no'}> NO

<HR>


<H2>Directory and File Modes</H2>

These variables specify the access mode bits for creating directories
and files. Be sure to include a leading '0'.
To have directory user (or group) ownership transferred to files, 
use e.g. \$hndirmode = 04775;  (or 02775 for group).  <P>

Directory Creation Mode (\$hndirmode):
<INPUT NAME ="hndirmode"  SIZE=10 VALUE="$hndirmodeText"> <P>

File Creation Mode (\$hnfilemode):
<INPUT NAME ="hnfilemode" SIZE=10 VALUE="$hnfilemodeText"> <p>

<HR>

};

    print qq{

<H2>People and Passwords </H2>

Administrators for HyperNews are given special powers beyond what
normal members are permitted. Administrotors are defined only during
the setup process, partly for added security.  \@hnAdmin is the list
of administrators for this HyperNews installation.  If you manually
edit \@hnAdmin in <tt>hnrc</tt>, be sure to run setup again. <P>

In addition to this list of administrators for all HyperNews data,
each forum has an administrator (i.e. the owner) whose scope is
limited to that forum and all messages under it.  Only the
Group administrators have administrative powers for the
membership and reset password functions. <P>

Each administrator in the list (comma or whitespace separated) should
be a $UserIDorHandle (which may be an email address, if you allow it).
Parens and quotes should not be used.
E.g. <tt>liberte\@hypernews.org, rmelo\@hypernews.org</tt>.  The first
entry in the list will be given in error messages for users who need
help. <P>

HyperNews Administrators (\@hnAdmin):
<TEXTAREA NAME ="hnAdmin" ROWS=5 COLS=60>$administrators</TEXTAREA><P>

<HR>

\$hnPeople is the directory to store membership information files for each
member (including administrators), one file for each member.  The
directory will also have one subdirectory per group.<P>

People Directory (\$hnPeople):
<INPUT NAME ="hnPeople" SIZE=60 MAXLENGTH=120 VALUE="$hnPeopleU"> <p>

\$hnPassword is the full path to the password file. 
This is usually <tt>\$hnPeople/.htpasswd</tt>, but this can be shared
with another installation of HyperNews.  In cannot be a password file
managed by some other process outside of HyperNews, however.<P>

Password File Full Path (\$hnPassword):
<INPUT NAME ="hnPassword" SIZE=60 MAXLENGTH=120 VALUE="$hnPassword"> <p>

\$hnGroup is the name of the group of members who have access to
documents via this particular installation of HyperNews.  Each
installation of HyperNews may now have a different group of users but
all can share the same password file and personal info for each user.
Group specific files are in the directory \$hnPeople\$hnGroup. <P>

You can leave \$hnGroup empty and then HyperNews will behave
identically with earlier versions.  You must do this the first time to
set up the "root" installation, but you can later define subgroups as
well.  If you want to specify a group name, the format is a path
relative to \$hnPeople.  E.g. <tt>NCSA/SDG</tt>. Note that this is not
a file system path; it is a path relative to the \$hnPeople directory.
The members of this group will be stored in the directory named by
\$hnPeople\$hnGroup.  There should be no spaces in the group name.<P>

<B>Warning: </B> You must install the "root" installation with no
group (i.e. leave it blank) the first time!  But for multiple
installations, if you forget to include the group name when you had
intended to set up a group, you will very likely mess up your root
installation.  <P>

Group Name (\$hnGroup):
<INPUT NAME ="hnGroup" SIZE=60 MAXLENGTH=120 VALUE="$hnGroup"> <p>

There are several fairly complex reasons to rebuild your \$hnPassword
file and \$hnGroup files.  You may not need to do it at all, and you
should only need to do this once per installation when you finally
install the new version of HyperNews.  Passwords and group info are
maintained redundantly thereafter in the membership information, the
\$hnPassword file, and the group specific files.  If you specify an
\$hnGroup above, then only the group file will be rebuilt.  If you
select the \$externalSecurity option in the hnrc file, the password
file is never changed or used.<P>

<INPUT TYPE="radio" NAME="buildPasswordyn" VALUE="yes"> Rebuild
\$hnPassword and \$hnGroup files. <BR>
<INPUT TYPE="radio" NAME="buildPasswordyn" VALUE="no" CHECKED> NO
<p>

<hr> 

};

#Another reason to rebuild your \$hnPassword file is to work around a
#problem that the server has when there are a large number of users in
#a single group.  The single long line of users in one group will be
#broken into shorter lines of users in several groups all logically in
#the same group. <P>

    print qq{

<H2>Security Options</H2>

HyperNews makes use of the authentication mechanism of NCSA\'s
httpd, Apache, and other servers.  Making use of this server
authentication mechanism permits the server to request authentication of
users automatically, and browsers pop up a dialog asking the user for
their user ID and password the first time.  All this is accomplished
by putting the restricted scripts in a 
directory that has an access restriction file.<P>

If the server authentication mechanism is different for your server,
you could either turn off all security (which is probably unwise), or
attempt to use the "manual", form-based authentication mechanism by
setting the following option.  This is not guaranteed to work in all
cases, however, so check it out if you need it.  In any event, manual
security is <B>not</B> used to restrict reading of messages - only
commands (such as changing subscription or deletion) that have side
effects.  <P>

<INPUT TYPE="radio" NAME="manualSecurity" VALUE="yes" $manualSecurity{'yes'}> 
Use form-based authentication (\$manualSecurity). <BR>
<INPUT TYPE="radio" NAME="manualSecurity" VALUE="no" $manualSecurity{'no'}> 
NO - use server authentication.
<P>

<HR>

Each of the commands or operations described in the following sections
are controlled by a configuration variable.  There are four possible
values for each variable: <P>

<dl>
  <DT> <B>0. No One</B>

  <DD> The command is disabled.  Actually, administrators will
       still be able to do it, but no URL will be advertised. <P>

  <DT> <B>1. Anyone</B>

  <DD> Anyone can do it without authentication.
       But in most cases with this setting, if the user affected by
       an operation is a member, only that member may do it. <P>

  <DT> <B>2. Members Only </B>

  <DD> Authentication is required.  Administrators
       are members too.  In other words, non-members cannot do it. <P>

  <DT>  <B>3. Administrators Only </B>
 
  <DD> A special check is done after
       authentication to see if the member is an administrator.<P>

</dl>

Users can become members via the edit-member.pl script, but even that
script may be restricted.  Group administrators are only created by
adding to the \@hnAdmin list above.  Group administrators may always
perform any function.  In addition, the owner of a forum is an
administrator for the forum and all messages (this does not apply
to the membership and reset password functions). <P>

<HR>


<h3>Registering New Users</h3>

Who can register someone as a new member of this HyperNews group?
For all settings of this variable, once registered, only authenticated
members may change membership information about themselves, except that
Group administrators may always change anyone\'s information.

<DL>
 <DT> <INPUT NAME = "joinSF" TYPE="radio" VALUE="0" $joinSF[0]>
Disable Registering.

 <DT> <INPUT NAME = "joinSF" TYPE="radio" VALUE="1" $joinSF[1]>
 Anyone may register themselves.

 <DT> <INPUT NAME = "joinSF" TYPE="radio" VALUE="2" $joinSF[2]>
 Only members may register new members. 

 <DT> <INPUT NAME = "joinSF" TYPE="radio" VALUE="3" $joinSF[3]>
 Only administrators may register new members.  Once registered,
 members may edit their own membership info.

</DL>

<HR>

<h3>Resetting Passwords</h3>

Who may reset the password of a member?  <P>

The main purpose of allowing passwords to be reset is to deal with the
situation where a user has forgotten their password.  To avoid
bothering administrators to reset passwords, the option is available
here for anyone, or members only, to reset the password of any member.
The new password is not displayed - it is only emailed, in the clear,
to the person whose password has been reset.  Email is not
particularly secure, so you may wish to allow only administrators to
reset passwords.  Of course, email must be enabled for this to work -
see Notification below.<P>

As a precaution, no one, not even another authenticated administrator,
may reset an administrator password, no matter what the reset option
is.  Instead, you have to change the password explicitly on the
membership form. <P>

Note that if you allow anyone to register themselves, then you should
probably not allow members to reset passwords unless you are very
trusting. <P>

<DL>
 <DT> <INPUT NAME = "resetPasswordSF" TYPE="radio" VALUE="0" 
    $resetPasswordSF[0]> Disable Resetting.

 <DT> <INPUT NAME = "resetPasswordSF" TYPE="radio" VALUE="1" 
$resetPasswordSF[1]>
 No authentication required to reset member passwords.

 <DT> <INPUT NAME = "resetPasswordSF" TYPE="radio" VALUE="2" 
$resetPasswordSF[2]>
 Only members may reset member passwords.

 <DT> <INPUT NAME = "resetPasswordSF" TYPE="radio" VALUE="3" 
$resetPasswordSF[3]>
 Only administrators may reset member passwords.

</DL>


<HR>


<h3>Forum Creation</h3>

Who may create new forums or overwrite existing ones?
The owner may be specified as different from the creator.
The owner of a forum has administrative powers over just
that forum and all its messages. <P>

If a forum is owned by a member, only that member, or an
administrator may update the forum.

<DL>
 <DT> <INPUT NAME = "addArticleSF" TYPE="radio" VALUE="0" 
    $addArticleSF[0]>Disable Forum Creation.

 <DT> <INPUT NAME = "addArticleSF" TYPE="radio" VALUE="1" $addArticleSF[1]>
 Anyone may create new forums without authentication.

 <DT> <INPUT NAME = "addArticleSF" TYPE="radio" VALUE="2" $addArticleSF[2]>
 Only members may create new forums.

 <DT> <INPUT NAME = "addArticleSF" TYPE="radio" VALUE="3" $addArticleSF[3]>
 Only administrators may create new forums.
</DL>


<HR>


<h3>Reading Forums and Messages</H3>

Who may read forums and messages?  <P> 

If set to 2, all the other security variables are
effectively set to level 2 or greater. (This may change in the future.)<P>

<DL>
 <DT> <INPUT NAME = "hnReadSF" TYPE="radio" VALUE="0" $hnReadSF[0]>
Disable Reading.

 <DT> <INPUT NAME = "hnReadSF" TYPE="radio" VALUE="1" $hnReadSF[1]>
 Anyone may read without authentication.

 <DT> <INPUT NAME = "hnReadSF" TYPE="radio" VALUE="2" $hnReadSF[2]>
 Only members may read.

 <DT> <INPUT NAME = "hnReadSF" TYPE="radio" VALUE="3" $hnReadSF[3]>
 Only administrators may read - how boring - this is disabled for now.
</DL>

<HR>



<h3>Adding Messages</h3>

Who may add a message?

<DL>
 <DT> <INPUT NAME = "addResponseSF" TYPE="radio" VALUE="0" $addResponseSF[0]>
Disable Adding Messages.

 <DT> <INPUT NAME = "addResponseSF" TYPE="radio" VALUE="1" $addResponseSF[1]>
 Anyone may add new messages without authentication.

 <DT> <INPUT NAME = "addResponseSF" TYPE="radio" VALUE="2" $addResponseSF[2]>
 Only members may add messages.

 <DT> <INPUT NAME = "addResponseSF" TYPE="radio" VALUE="3" $addResponseSF[3]>
 Only administrators may add new messages.
</DL>


<HR>

<h3> Notification </h3>

Who may subscribe themselves to be notified by email of new messages? 
Once subscribed, only the user may unsubscribe if they are a member.<P>

<DL>
 <DT> <INPUT NAME = "notifySF" TYPE="radio" VALUE="0" 
    $notifySF[0]> Disable Email Notification.

 <DT> <INPUT NAME = "notifySF" TYPE="radio" VALUE="1" $notifySF[1]>
 Anyone can change his/her own notification information. Anyone can change non-members notification information.  

 <DT> <INPUT NAME = "notifySF" TYPE="radio" VALUE="2" $notifySF[2]>
 Only members can change their own notification information.
 
 <DT> <INPUT NAME = "notifySF" TYPE="radio" VALUE="3" $notifySF[3]>
 Only administrators may change notification information.
</DL>

<P>

<HR>


<h3>Deleting Messages</h3>

Who may delete a message?  Forums may not be deleted (except via command line).<P>

<DL>
 <DT> <INPUT NAME = "deleteSF" TYPE="radio" VALUE="0" $deleteSF[0]>
Disable Deleting Messages

 <DT> <INPUT NAME = "deleteSF" TYPE="radio" VALUE="1" $deleteSF[1]>
 Anyone may delete non-member or their own messages.

 <DT> <INPUT NAME = "deleteSF" TYPE="radio" VALUE="2" $deleteSF[2]>
 Only members may delete their own messages.

 <DT> <INPUT NAME = "deleteSF" TYPE="radio" VALUE="3" $deleteSF[3]>
 Only administrators may delete any messages.

</DL>


<HR>


<h3>Moving Messages</h3>

Who may move or copy messages?  Moving also deletes the source
messages, but leaves a placeholder.  The ownership of replies to a
message are not checked, but are left intact.  Forums may not
be moved (yet).  <P>

<DL>
 <DT> <INPUT NAME = "moveSF" TYPE="radio" VALUE="0" $moveSF[0]>
 Disable Moving Messages.

 <DT> <INPUT NAME = "moveSF" TYPE="radio" VALUE="1" $moveSF[1]>
 Anyone may move non-member messagess.  Members may move their own.

 <DT> <INPUT NAME = "moveSF" TYPE="radio" VALUE="2" $moveSF[2]>
 Only members may move their own messages.

 <DT> <INPUT NAME = "moveSF" TYPE="radio" VALUE="3" $moveSF[3]>
 Only administrators may move messages.

</DL>


<HR>

<h2> Submitting </h2>

When you submit this form, the values you entered will be checked for
several constraints, and then several things will be updated.  \$hnlib
("<tt>$hnlib</tt>") <STRONG>must be writable by the server</STRONG>
process when you submit in order for some of these things to be
updated.  Also \$hnbin ("<tt>$hnbin</tt>"), \$hnbin/securedir, and
\$hnbin/Admin (if they already exist) must be writable.  As a
precaution, <STRONG>Be sure to make them unwritable after you
submit.</STRONG> If you want to avoid making these directories
writable by the server process, instead run setup.pl directly from a
command line.<P>
};


    print qq{
<UL>

  <LI> The \$hndocs directory must already be created by you, writable
  by the server process. <P>

  <LI> The \$hnPeople directory is created. <P>

  <LI> The \$hnPassword and \$hnGroupMembers files are built, if you
  selected that option.
      <P>

  <li> The \$hnicons directory is created and icons installed, if you
  selected that option.
  <P>

  <LI> The <tt>hnrc</tt> file that holds the values of the
  configuration options is written in \$hnbin/Admin. <P>

  <LI> The scripts directory, "<tt>$hnlib</tt>", is protected with an
  access restiction file so it is accessible only by administrators
  named in \@hnAdmin.   <P>

  <LI> "<tt>\$hnbin/$securedir</tt>" is created,
  and links to the restricted scripts are created. 
  Links to unrestricted scripts are created in "<tt>\$hnbin</tt>" <P>


</UL>
<P>
};

    if (!$authenticated) {
	print qq{
If you have set up HyperNews previously, you need to enter your
Admin $UserIDorHandle and Password here. <P>

<STRONG>Admin $UserIDorHandle</STRONG>: <BR>
<INPUT NAME="UserID" SIZE=35 MAXLENGTH=65> <P>

<STRONG>Password</STRONG>: <BR>
<INPUT NAME="Password" TYPE="password" SIZE=8 MAXLENGTH=8> <P>
};
    }
    else {
      print qq{
<INPUT NAME="UserID" TYPE="hidden" VALUE="$remoteUser"> <P>
<INPUT NAME="Password" TYPE="hidden" VALUE="$remotePassword"> <P>
};

}

	print qq{
<INPUT TYPE="submit" VALUE="Proceed to Setup HyperNews"> 

};

&printEnd();

}
