#!/usr/local/bin/perl
# setup-lib.pl - HyperNews Installation library.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

sub setDefaultValues {
##############################################################
#
# Some default values are assigned here.
# These values will NOT be used, except for the version number, if
# the corresponding variables have values assigned in the hnrc file.
# And new versions of HyperNews will overwrite this file.  So it
# is not recommended that you make any changes here unless necessary.

    # Change this version number if you change the source significantly.
    # Also change the version in Makefile
    $hnversion = '1.10';

    # The type of server.
    $serverType = $ENV{SERVER_SOFTWARE};

    # We need $http so we can display full URLs.
    if (!$http) {
	$http = "http://\$ENV{SERVER_NAME}";
	if ($ENV{SERVER_PORT} && ($ENV{SERVER_PORT} ne "80")) {
	    # This doesnt work with command line setup.pl.
	    $http .= ":\$ENV{SERVER_PORT}";
	}
    }

    $httpEquiv ||= "$http";

    # Most uses of SERVER_NAME and SERVER_PORT should be avoided.
    $realhost = $ENV{SERVER_NAME};  # Save for later.
    # $realhost ||= `hostname`;  # May not be fully qualified.

    if (!$ENV{SERVER_NAME}) {
	$ENV{SERVER_NAME} = "\$ENV{SERVER_NAME}";
	$ENV{SERVER_PORT} = "\$ENV{SERVER_PORT}";
    }

    $hnprefix ||= '/HyperNews';  # Hard-coded URL prefix of HyperNews dir
    $hnprefixEquiv ||= "$hnprefix";
    $hnrootprefix ||= $hnprefix;

    # If you set $httpProxyHost, you must also set $httpProxyPort.
    $httpProxyHost = '';
    $httpProxyPort = '3128';  # Default for Squid

    # Whether HyperNews should use full URLs when making http requests.
    $useFullHTTPURL = 0 if !defined ($useFullHTTPURL);

    # Whether HN should support cachability of HN pages.
    # Should be off by default - those who need it can change it.
    $supportCaching = 0 if !defined $supportCaching;
    $disableCaching = 0 if !defined $disableCaching;

    $hndocs ||= "/usr/local/etc/httpd/htdocs/hn";
    # $hndocsURL ||= '$http/hn';  not used.

    # $hnRootDoc is a path relative to $hndocs.
    $hnRootDoc ||= '';  # e.g. /hnroot.html

    # HyperNews instructions.   It need not be a HyperNews page.
    $instructionsURL = 'http://www.hypernews.org/HyperNews/get/hypernews/instructions.html' if !defined $instructionsURL;

    # HyperNews test page.  e.g. "$getURL/test.html"
    $hntestURL = 'http://www.hypernews.org/HyperNews/get/hypernews/test.html'
	if !defined $hntestURL;

    $useHNIcons = 1 if !defined $useHNIcons;

    $hnicons ||= "\$hndocs/Icons";
    $hniconsURL ||= "\$http/hn/Icons";

    if (!defined($newIconXsize)) {
      # Assume none are defined.
      $newIconXsize = 31;
      $newIconYsize = 12;
      $relationIconXsize = 15;
      $relationIconYsize = 15;
      $depthIconXsize = 40;
      $depthIconYsize = 17;
      $commandIconXsize = 60;
      $commandIconYsize = 17;
      $threadIconXsize = 60;
      $threadIconYsize = 17;
    }


    $globalDefaultOutlineDepth ||=
	# For compatibility with older versions.  Can't be 0.
	$defaultOutlineDepth || $maxDisplaylevel || 3;

    $globalDefaultInlineDepth = 0 if !defined $globalDefaultInlineDepth;

    if (!defined @messageFormats) {
	@messageFormats = ('Smart Text', 'Word Processor', 'Plain Text', 'HTML');
    }

    # Whether email addresses are allowed for user IDs.
    # The CERN server requires that user IDs are not long email
    # addresses.  Some servers may get confused by having IDs that are
    # prefixes of email addresses.
    # It should be allowed if you are upgrading, for backward compatibility.
    if (!defined ($allowEmailUserID)) {
      if ($serverType =~ m,CERN,) { $allowEmailUserID =  0; }
      else { $allowEmailUserID =  1; }
    }
    $lower_case_IDs = 0 if !defined $lower_case_IDs;
    $allow_spaces_in_userids = 0 if !defined $allow_spaces_in_userids;

    $authenticateInquiries = 0 if !defined $authenticateInquiries;
    # Whether to request personal data on edit-member.pl
    $getPersonalData = 1 if !defined $getPersonalData;

    $showPriv = 1 if !defined $showPriv;

    # Need to change these to 'sometimes', 'never', 'always' options!!
    $hideUserIDs = 0  if !defined $hideUserIDs;
    $hideEmailAddresses = 0  if !defined $hideEmailAddresses;


    # Globals related to sendmail 
    $sendmail ||= '/usr/lib/sendmail -oi -oem';

    my $athost;
    if ($realhost && ($realhost !~ m,^\$ENV,)) { $athost = '@' . $realhost; }
    # $HyperNews Must be different from $HyperNewsOwner.
    $HyperNews = "HyperNews$athost";
    $HyperNewsOwner = "Owner-HyperNews$athost";
    # The following should be symbolic values instead of literals.
    $mail_to ||= $HyperNews;
    $mail_from ||= $HyperNewsOwner;  # Used only if author is unknown.
    $mail_sender ||= $HyperNewsOwner;
    $mail_errors_to ||= $HyperNewsOwner;
    # $mail_reply_to ||= $HyperNews;  # Not recommended.

    # We found out that using the "Errors-to:" line in messages causes
    # mail to fail sometimes.  Maybe it is only when the outgoing email
    # address doesnt exist, in which case it needs some place to send those
    # errors.  Still, it should mail to those it can, I would think.

    # Admin that posts HN replies from email on behalf of users.
    $mail_admin ||= 'HNMail';  
    # $mail_admin_password; no default for this.

    # The user name and password for the administrative user
    # that does the posting to the server on behalf of the email user.

#==========================

    # Whether email replies can be sent to HyperNews.
    # Also see $mail_admin
    $allowEmailIn = 0 if (!defined ($allowEmailIn));

    # The directory for message IDs.
    $hnmidDB ||= "\$hndocs/.midDB";

    # Whether the default for notification of responses is on or off.
    $defaultNotification = 1 if (!defined($defaultNotification));

    # Whether each member is subscribed to all-member forums by default.
    $defaultSubscribeAll = 1 if !defined($defaultSubscribeAll);

    # Which forums are all-member forums?  'none', 'some', or 'all'
    $allMemberForums = 'none' if !defined($allMemberForums);

    $hnEmailNotify = 1 if (!defined($hnEmailNotify));
    $hnEmail = 1 if (!defined($hnEmail));

    $printResponseNum = 1 if (!defined($printResponseNum));
    $printResponseDate = 1 if (!defined($printResponseDate));
    $reverseMessages = 0 if (!defined($reverseMessages));
    $buttonsBeforeMessages = 0 if (!defined($buttonsBeforeMessages));

    $simpleButtons = 0 if !defined $simpleButtons;
    $textButtons = 0 if !defined $textButtons;
    $centerButtons = 0 if !defined $centerButtons;

    # Whether the Members button goes to view-members or edit-member.
    $viewMembers = 1 if !defined($viewMembers);

    $allowThreads   = 1 if (!defined($allowThreads));
    $allowDirectPost = 0 if (!defined($allowDirectPost));
    $allowReferences = 0 if (!defined($allowReferences));
    $allowModeration = 0 if (!defined($allowModeration));
    $allowFrames = 1 if (!defined($allowFrames));
    $gotoForumAfterPost = 0 if !defined $gotoForumAfterPost;

    # Some people like 'Handle' better than 'UserID'
    $UserIDorHandle = 'User ID' if !defined $UserIDorHandle;

    $displayParents = 0 if !defined $displayParents;
    $displaySiblings = 0 if !defined $displaySiblings;
    $displayChildren = 0 if !defined $displayChildren;

    $autoGenerateSiblingForumList = 1 
	if !defined $autoGenerateSiblingForumList;
    $autoGenerateChildForumList = 1 
	if !defined $autoGenerateChildForumList;

    if (!defined($startButtons)) {
	$startButtons = qq{\n<P>}; # e.g. <CENTER>
	$endButtons = qq{\n}; # e.g </CENTER>
    }


    if (!defined($startInline)) {
	$startInline = qq{<P>

<TABLE WIDTH=100% BGCOLOR=lightyellow cellspacing=0 border=1> 
 <TR> <TD> <!-- start header -->
};
	$startInlineBody = qq{
 </TD></TR> <!-- end header -->
 <TR> <TD WIDTH=100%> <!-- start body -->
};
	$endInlineBody = qq{
 </TD> </TR> <!-- end body -->
};

	$startInlineResponses = qq{
 <TR> <TD WIDTH=100%> <!-- start response tree -->
};
	$endInlineResponses = qq{
 </TD> </TR> <!-- end response tree -->
};
	$endInline = qq{
</TABLE>
};
    }

# What to wrap around the message body.
    if (!defined($startMsgBodyTag)) {
# The BGCOLOR may conflict with the page color.
	$startMsgBodyTag = qq{
<table border cellpadding=5 cellspacing=0 width="100%" BGCOLOR="#FFFFFF">
 <TR><TD>
};
      $endMsgBodyTag = qq{
 </TD></TR>
</table>
};
    }

    # Sorting messages by dates.
    $sortByDate = 0 if !defined $sortByDate;

    # Date boundary specs.  Format is used by Date::Manip::ParseRecur.
    # e.g. '0:0:0:1*9,12,14,18,24:0:0' is every day at 9, 12, 2pm, etc.
    # e.g. '0:0:2*0:0:0:0' is every two weeks.
    # e.g. '0:1*0:1:0:0:0' is first day of every month.
    # Should be humanly readable.
    $dateBoundaries = '' if !defined $dateBoundaries;
    
    # Date format spec, used by Date::Manip::UnixDate.
    $dateFormat = '%Y %b %d, %H:%M' if !defined $dateFormat;

    # The name of the Access file
    if (!defined($htaccess)) {
	if ($serverType =~ /CERN/) {
	    # Should include some other servers.
	    $htaccess = '.www_acl';
	}
        elsif ($serverType =~ /Netscape/) {
            $htaccess = '.nsconfig';
        }
	# elsif ($serverType =~ /NCSA/)
	else {
	    # Should also include Apache and (most?) others.
	    $htaccess = '.htaccess';
	}
    } 

    # Name of HyperNews access file.  The $htaccess files will link to this.
    $hnaccess = '.hnaccess';

    # Name of file that lists group members needs to be the same 
    # for all groups that share the same hnpeople.
    # So the names are hard coded here.  But this is not used anymore.
    $hnGroupMembers = '.htgroupmembers';

    # Name of log files - leave null if no logging shoud be done.
    $hnLog ||= '.log';

    # Name of mailing list files (not used yet)
    $hnMailList ||= '.list';

    $hndirmode = $hndirmode || 0775;
    $hnfilemode = $hnfilemode || 0664;

    $manualSecurity ||= 0;
    $externalSecurity ||= 0;

    # Require passwords - no empty passwords allowed.
    $requirePasswords = 1 if (!defined($requirePasswords));
	
    # Generate passwords for new members.
    $generatePasswords = 0 if (!defined($generatePasswords));

# Whether HyperNews should generate and use cookies for authentication.
# 'always', 'never', or 'sometimes'. If 'sometimes', check per-member option.
    $UseCookies = 'sometimes' if !defined($UseCookies);
    $CookieLifespan = 7 if !defined $CookieLifespan;

# If non-null, this is the HN key used when making the cookie password.
# If you do this, make SURE only those you trust can see this hnrc file.
    $HNCookiePassword = '' if !defined($HNCookiePassword);


    $hnPeople ||= "\$hndocs/hnpeople";
    $hnPassword ||=  "\$hnPeople/.htpasswd";
    $hnGroup ||= "";  # null string is for "root" level.
    $hnRealm ||= "HyperNews$hnGroup";

    if (!defined ($hnSystemName)) {
	$hnSystemName = "HyperNews";
	$hnSystemName .= " at $ENV{SERVER_NAME}" if $ENV{SERVER_NAME};
	$hnSystemName .= " (group: $hnGroup)" if $hnGroup;
    }

    if (!defined($PWFileDBM)) {
      $PWFileDBM = 0;
    }

    if (!$hnReadSF) {
      # Assign default values for the settings.
      $joinSF = 1;
      $resetPasswordSF = 3;
      $hnReadSF = 1;
      $notifySF = 1;
      $addArticleSF = 3;
      $addResponseSF = 1;
      $deleteSF = 2;
      $moveSF = 2;
    }


# For http requests, how many seconds should we wait?
    $timeout = $timeout || 60;

}

# End of defaults
#
########################################################################


sub setup_form_context {
    # Use this when getting hnrc data ready for a form.

    # Where is hnrc?
    if (-e "$hnbin/Admin/hnrc") {
      require "$hnbin/Admin/hnrc";
      $hnrcString = "An <tt>hnrc</tt> file has been found in $hnbin/Admin.";
    } elsif (-e "$hnbin/hnrc") {
      # Check the old places too.
      require "$hnbin/hnrc";
      $hnrcString = "An <tt>hnrc</tt> file has been found in $hnbin.";
    } elsif (-e "$hnlib/hnrc") {
	# $hnlib is the $scriptsdir directory while doing the setup.
      require "$hnlib/hnrc";
      $hnrcString = "An <tt>hnrc</tt> file has been found in $hnlib.";
    } else {
      $hnrcString = "No <tt>hnrc</tt> file has been found in $hnbin/Admin
nor old places.  One will be created for you when you submit this form.";
    }


    if (!$hndocs) {
	# Compute default $hndocs from PATH_TRANSLATED,
	# assuming URL is .../setup-form.pl/
	if ($ENV{'PATH_TRANSLATED'}) {
	    $hndocs = $ENV{'PATH_TRANSLATED'};
	    $hndocs =~ s,/$,/hn,;
	}
    }

    &setDefaultValues();

#============================
# Prepare values for form.

    # Keep variables relative to $hndocs and $http, if they are already.

    # Expand variables in $http.
    #eval ("\$httpExpanded = \"$http\""); causes failure
    $httpExpanded = $http;
    $http =~ s,$ENV{SERVER_NAME},\$ENV{SERVER_NAME},;
    $http =~ s,:$ENV{SERVER_PORT},:\$ENV{SERVER_PORT},;

# If no previous $hnicons, then install icons.
    %iconsyn = ();
    if (!$hnicons || !-e "$hnicons/HyperNews.gif") {
	$iconsyn{'yes'} = "CHECKED"; 
    } else {
	$iconsyn{'no'} = "CHECKED"; 
    }

    $hnicons =~ s,$hndocs,\$hndocs,;
    $hniconsURL =~ s,$httpExpanded,\$http,;

    if ($headerText) {
	$headerTextEncoded = &escapeHTML2($headerText);
    }

    if ($footerText) {
	$footerTextEncoded = &escapeHTML2($footerText);
    }

    $global_body_tag ||= $bodytag;  # New name.
    if ($global_body_tag) {
      $global_body_tag_value_encoded = &escapeHTML2($global_body_tag);
    }

    #&HNMsg("hnPeople: $hnPeople");
    #&HNMsg("hnPassword: $hnPassword");

    $hnPeopleU = $hnPeople;  # The expanded form.
    $hnPeopleU =~ s,$hndocs,\$hndocs,;
    $hnPassword =~ s,$hnPeople,\$hnPeople,;

    $hnmidDB =~ s,$hndocs,\$hndocs,;

    #&HNMsg("hnPeople: $hnPeople");
    #&HNMsg("hnPassword: $hnPassword");

    @hnAdmin = ($mail_admin) if !defined @hnAdmin;

    $administrators = join(", ",@hnAdmin);

    if ($manualSecurity == 1) {
	$manualSecurity{'yes'} = "CHECKED";
    } else {
	$manualSecurity{'no'} = "CHECKED";
    }

    @joinSF[$joinSF] = 'CHECKED';
    @resetPasswordSF[$resetPasswordSF] = 'CHECKED';
    @hnReadSF[$hnReadSF] = 'CHECKED';
    @addArticleSF[$addArticleSF] = 'CHECKED';
    @addResponseSF[$addResponseSF] = 'CHECKED';
    @deleteSF[$deleteSF] = 'CHECKED';
    @moveSF[$moveSF] = 'CHECKED';
    @notifySF[$notifySF] = 'CHECKED';


    $hndirmodeText = sprintf("%0.4o", $hndirmode);
    $hnfilemodeText = sprintf("%0.4o", $hnfilemode);

    ($user_name, $group_name) = &user_group_names();

#====================================
# Check SCRIPT_NAME

    $script_name = $ENV{'SCRIPT_NAME'};
    if ($script_name =~ m{/$ }x) { # Is there a trailing / ?
      # Should have been stripped by cgiwrap already.
      $script_name = s{/$ }{}x;
      $ENV{PATH_INFO} = '/';
    }

    if ($script_name !~ m,/[^/]+$,) {
	&HNError("This will never work.  Your server does not provide
a SCRIPT_NAME environment variable that is a rooted path so HyperNews can tell
what directory we are running from.  All I see is \"$script_name\".

Try using setup.pl from the command line.");
    }

    $scripts_path = $script_name;
    $scripts_path =~ s,/[^/]+$,,;

#====================================
# Compute whether to use .pl or .cgi suffix

    $plorcgi = '';
    if ($script_name =~ m,\.cgi$,) {
	$plorcgi = "CHECKED";
    }

    if ($plorcgi) {
	$setupURL = "$scripts_path/setup.cgi";
	$setup_long_form_URL = "$scripts_path/setup-form.cgi";
	$setup_short_form_URL = "$scripts_path/setup-short-form.cgi";
    } else {
	$setupURL = "$scripts_path/setup.pl";
	$setup_long_form_URL = "$scripts_path/setup-form.pl";
	$setup_short_form_URL = "$scripts_path/setup-short-form.pl";
    }


#====================================
# Guess what prefix to remove.
# This is not really sufficient.  Any prefix might be removed, not
# just cgi-bin.  Installer needs to specify the prefix to remove.

    $scriptPrefix = "/cgi-bin";
    if ($script_name !~ m,$scriptPrefix, || ($remove_cgi_bin eq 'yes')) {
	$remove_cgi_bin{'yes'} = 'CHECKED';
    } else {
	$remove_cgi_bin{'no'} = 'CHECKED';
    }

}



sub user_group_names {
   local($id,$group_name,$user_name);
   local(@groupids);
   $id = $(;
   @groupids = split(/ /,$id);
   ($group_name) = getgrgid ("$groupids[0]");  # what about multiple groups
   ($user_name) = getpwuid ($<);
   return($user_name, $group_name);
}


# This may not be needed any more.  Only used for strings in textarea
# for the header, footer, and body tags.  Some browsers might need this.
sub escapeHTML2 {
    local($string) = @_;

    # Remove extra newlines at beginning and end of string.
    $string =~ s/^\s*//g;
    $string =~ s/\s*$//g;

    $string =~ s/\&/\&amp;/g ;
    $string =~ s/</\&lt;/g ;
    $string =~ s/>/\&gt;/g ;

    $string =~ s/\\\"/\&quot;/g ;  # what's this for?
    $string =~ s/\"/\&quot;/g ;  # HTMLize quote.
    $string;
}

1;
