#!/usr/bin/perl
# setup-short-form.pl - generates a simiple form for configuring the system.
# Copyright Board of Trustees of the University of Illinois at UC, Jan 1996

# Run this via your web browser with a URL like:
# http://your.server.com/cgi-bin/HyperNews/.scripts/setup-form.pl/

# If you are having trouble seeing any output, try uncommenting this:
# print "Content-type: text/plain\n\n";

#####################
# Work around error: "Can't locate /dev/.scripts/*"
# Only works if current dir is HyperNews or lower.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};

$0 =~ m{^(.*)/[^/]+$ }x;  # Get last component.
$hnbin = $1 || '.';  # If $0 is not a path, use current directory.

$scriptsdir = ".scripts"; # The directory of HyperNews scripts and libraries.
$securedir = "SECURED";	# Members only directory within $hnbin
$admindir = "Admin"; # Admin only directory.

$hnbin =~ s{/?$scriptsdir$ }{}x;  # Do this first.
$hnbin =~ s{/?$admindir$ }{}x;

if (!$hnbin || $hnbin eq '.') {
    # If the above fails, try this:
    $hnbin = `pwd`;
    chop($hnbin);

    # Then try again to remove components.
    $hnbin =~ s{/?$scriptsdir$ }{}x;  # Do this first.
    $hnbin =~ s{/?$admindir$ }{}x;
}

# If all else fails, just try current directory.
$hnbin = '.' if !$hnbin;

# Make hnlib be $scriptsdir, inside $hnbin.
$hnlib = "$hnbin/$scriptsdir";

unshift(@INC, $hnbin, $hnlib); 
####################


require "hn-lib.pl" ||
    die ("Installation Error!: Can't load $hnlib/hn-lib.pl");

local $debug = 0;

&feval('&setupForm()');

sub setupForm {

    # 5.001m is OK in most cases, but not always, and not 5.001.
    require 5.002;

    require "setup-lib.pl";


    &setup_form_context();

    ################################
    require "member-lib.pl";

#==================================
# Check authentication and access control here.


    require "formlib.pl";
    my %form = &GetFormArgs();
    
    my $userid = $ENV{REMOTE_USER} || $form{UserID};
    my $password = $form{Password};

    &checkAuthenticated('', '', $userid, $password);

    # We always require admin if .htaccess exists or $authenticated
    # or manual or external security and @hnAdmin. 
    if ($authenticated ||
	-e "$hnlib/.htaccess" || 
	-e "$hnlib/.www_acl" || -e "$hnlib/.nsconfig"
	|| (($externalSecurity || $manualSecurity) && @hnAdmin)) {

	if (!$authenticated) {
	    my $thisURL = &URLencode($ENV{REQUEST_URI} || 
				     "$setup_short_form_URL");
	
	    my $redirectURL = "$loginURL?url=$thisURL";
	    # Try logging in first.
	    &printRedirect($redirectURL);
	}

	&access1(3, 2, $remoteUser, $remoteUser,
		 "Setup must be executed by an administrator.
<strong> $remoteUser </strong> is not an administrator!",
		 $authenticated);
    }


    ################################


    my $whoami = &sustring($remoteUser);
    if ($whoami) { $whoami = "You are: $whoami <BR>"; }

    # Preparation specific to short-form.
    $securityLevel{'freeforall'} = 
	($hnReadSF == 1 &&
	 $notifySF == 1 &&
	 $addArticleSF == 3 &&
	 $addResponseSF == 1 &&
	 $joinSF == 1 &&
	 $resetPasswordSF == 3 &&
	 $deleteSF == 2 &&
	 $moveSF == 2 &&
	 'CHECKED');

    $securityLevel{'open'} = 
	($hnReadSF == 1 &&
	 $notifySF == 2 &&
	 $addArticleSF == 3 &&
	 $addResponseSF == 2 &&
	 $joinSF == 1 &&
	 $resetPasswordSF == 3 &&
	 $deleteSF == 2 &&
	 $moveSF == 2 &&
	 'CHECKED');
	
    $securityLevel{'private'} = 
	($hnReadSF == 2 &&
	 $notifySF == 2 &&
	 $addArticleSF == 2 &&
	 $addResponseSF == 2 &&
	 $joinSF == 2 &&
	 $resetPasswordSF == 2 &&
	 $deleteSF == 2 &&
	 $moveSF == 2 &&
	 'CHECKED');

    $securityLevel{'customized'} = 
	!($securityLevel{'freeforall'} ||
	  $securityLevel{'open'} ||
	  $securityLevel{'private'}) &&
	 'CHECKED';

    # Copied from edit-member.pl
    @who = ('No One', 'Anyone', 'Only Members', 'Only Admin');

    require "edit-lib.pl";

    $priv .= "<SELECT NAME=\"joinSF\">" . 
 &buildOptions($who[$joinSF], @who) . 
 " </SELECT> may register new members.  Members may change their own info.<BR>\n";

    $priv .= "<SELECT NAME=\"resetPasswordSF\">" .
 &buildOptions($who[$resetPasswordSF], @who) . 
 "</SELECT> may reset passwords of members.<BR>\n";

    $priv .= "<SELECT NAME=\"addArticleSF\">" .
 &buildOptions($who[$addArticleSF], @who) . 
 "</SELECT> may create new forums.
 The owner of a forum is the administrator for it.<BR>\n";

    $priv .= "<SELECT NAME=\"hnReadSF\">" .
 &buildOptions($who[$hnReadSF], @who) . 
 "</SELECT> may read forums and messages.<BR>\n";

    $priv .= "<SELECT NAME=\"notifySF\">" .
 &buildOptions($who[$notifySF], @who) . 
 "</SELECT> may add subscriptions for notification.
 Once a member is subscribed, only that member may unsubscribe.<BR>\n";

    $priv .= "<SELECT NAME=\"addResponseSF\">" .
 &buildOptions($who[$addResponseSF], @who) . 
 "</SELECT> may add messages.<BR>\n";

    $priv .= "<SELECT NAME=\"deleteSF\">" .
 &buildOptions($who[$deleteSF], @who) . 
 "</SELECT> may delete their own messages.<BR>\n";

    $priv .= "<SELECT NAME=\"moveSF\">" .
 &buildOptions($who[$moveSF], @who) . 
 "</SELECT> may move their own messages.<BR>\n";



    &print_HTML_HEAD ("HyperNews Setup Short Form");

    print qq{
<H2>HyperNews Short Form Setup</H2>

$whoami

<FORM ACTION="$setupURL" METHOD="POST"> 

This is the short form for configuring HyperNews.  Anything you don\'t
change from this form should stay the same.  If the options you need
to change are not available here, try the <A
HREF="$setup_long_form_URL"> long form</A>.  <P>

HyperNews is configured by modifying the "hnrc" file when you submit
this form. <P>

<strong>$hnrcString</strong><P>

<HR>

<H2>HyperNews Documents Directory</H2>

\$hndocs ($hndocs) is the full path to the directory where your HyperNews
forums and messages are stored.  This directory must be
created by you before submitting this form, and it must be writable by
the user running the server process, which is "$user_name". <P>

HyperNews documents directory full path (\$hndocs): <BR>
<INPUT NAME ="hndocs" SIZE=60 MAXLENGTH=120 VALUE="$hndocs"> <P>

<HR>

};

    print qq{

<H2>People and Passwords </H2>

Membership information is stored in \$hnPeople ($hnPeople) (change it
with the long form). <P>

Administrators for HyperNews are defined only during the setup
process.  The first entry in the list will be given in error messages
for users who need help.  E.g. <tt>liberte\@hypernews.org,
 rmelo\@hypernews.org</tt>.  <P>

HyperNews Administrators (\@hnAdmin):<BR>
<TEXTAREA NAME ="hnAdmin" ROWS=5 COLS=60>$administrators</TEXTAREA><P>

Group specific members are in the directory \$hnPeople\$hnGroup.  The
format of \$hnGroup is a path relative to \$hnPeople.
E.g. <tt>NCSA/SDG</tt>. <P>

Group Name (\$hnGroup):<BR>
<INPUT NAME ="hnGroup" SIZE=60 MAXLENGTH=120 VALUE="$hnGroup"> <p>

You should rebuild your password and group files once per
installation.  If you specify a group above, only the group file will
be rebuilt.<P>

<INPUT TYPE="radio" NAME="buildPasswordyn" VALUE="yes"> Rebuild
\$hnPassword and \$hnGroup files. <BR>
<INPUT TYPE="radio" NAME="buildPasswordyn" VALUE="no" CHECKED> NO
<p>

<hr> 
};

    print qq{

<H2>Security Options</H2>

The security options control who has access to what commands in HyperNews.
See the long form for detailed descriptions of each option.  Here you
can select one of the common configurations, or customize the settings. <P>

<DL>

  <DT> <INPUT NAME="securityLevel" TYPE="radio" VALUE="freeforall" 
        $securityLevel{'freeforall'}>

Free-for-All.  

  <DD> (The default - same as at the 
<A HREF="http://www.hypernews.org/HyperNews/get/hypernews.html">
HyperNews.org home site</A>.)
Non-members can read, respond, register, and subscribe.  Members can delete or
move their own messages. Only Admin can create forums and reset
passwords. <P>

  <DT> <INPUT NAME="securityLevel" TYPE="radio" VALUE="open" 
        $securityLevel{'open'}>

Open Forum. 

  <DD> Like Free-for-All, but only members can respond and subscribe.<P>

  <DT> <INPUT NAME="securityLevel" TYPE="radio" VALUE="private" 
        $securityLevel{'private'}>

Private Meeting.  

  <DD> Only members can do anything,
but they can also do everything. <P>


  <DT> <INPUT NAME="securityLevel" TYPE="radio" VALUE="customized" 
        $securityLevel{'customized'}>
       Customized to the following settings: <P>

  <DD> 
	   $priv

  <P>


</DL>
};

    print qq{

<HR>

<INPUT TYPE="hidden" NAME ="headerText" 
 VALUE="$headerTextEncoded">
<INPUT TYPE="hidden" NAME ="footerText" 
 VALUE="$footerTextEncoded">
<INPUT TYPE="hidden" NAME ="global_body_tag" 
 VALUE="$global_body_tag_value_encoded">
};

    print qq{

<h2> Submitting </h2>

When you submit this form, the following directories  <STRONG>must be writable by the server</STRONG>:

<UL>
  <LI> <tt>$hnlib</tt>
  <LI> <tt>$hnbin</tt>
  <LI> <tt>$hnbin/$securedir</tt>
  <LI> <tt>$hnbin/$admindir</tt> 
</UL>

As a precaution, <STRONG>be sure to make them unwritable when you are
done setting up.</STRONG>   The \$hndocs directory must also be writable
and must remain writable.<P>
};

     if (!$authenticated) {
	print qq{
If you have set up HyperNews previously, you need to enter your
Admin $UserIDorHandle and Password here. <P>

<STRONG>Admin $UserIDorHandle</STRONG>: <BR>
<INPUT NAME="UserID" SIZE=35 MAXLENGTH=65> <P>

<STRONG>Password</STRONG>: <BR>
<INPUT NAME="Password" TYPE="password" SIZE=8 MAXLENGTH=8> <P>
};
    }
    else {
      print qq{
<INPUT NAME="UserID" TYPE="hidden" VALUE="$remoteUser"> <P>
<INPUT NAME="Password" TYPE="hidden" VALUE="$remotePassword"> <P>
};
    }

    print qq{

<INPUT TYPE="submit" VALUE="Proceed to Setup HyperNews"> 

</FORM>
};

  &printEnd();

}


