#!/usr/bin/perl
# setup.pl - Configure HyperNews.  Create or replace hnrc file.
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

#################################
# NOTE:
# Before you run this program,
# you must first su to the same user who runs HyperNews
# (or arrange for common group writable permission).
#
# You can run setup.p starting from .scripts, Admin, or top-level
# HyperNews directory.  But if you link to .scripts from some other
# installation, you must call setup via the directory of that
# installation.
#################################

# This script may be called directly from a Unix shell command line or via
# the web as the action for setup-form.pl or setup-short-form.pl.

# Running setup.pl from command line the first time will create an
# hnrc file with default values.  Modify hnrc and then run setup.pl
# again.    Try setup.pl --help for options.


$ENV{PATH} = '/bin:/usr/bin';    

# These dirs are not really configurable.
$scriptsdir = ".scripts"; # The directory of HyperNews scripts and libraries.
$securedir = "SECURED";	# Members-only directory within $hnbin
$admindir = "Admin"; # Admin-only directory.

#####################
# Add current directory to load path.
# We do the following in each script.

# Work around error: "Can't locate /dev/.scripts/*"

# If running on OSes that replace $0 with something like /dev/fd/3, 
# maybe avoid that by making sure scripts are not writable.
# Or use the following line, with your own path to the HyperNews dir.
# $0 = '/usr/local/etc/apache/shared/cgi-bin/HyperNews/SCRIPT_FILENAME';

# But try this - it works if current dir is HyperNews or lower.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};

$0 =~ m{^(.*)/[^/]+$ }x;  # Get prefix
$hnbin = $1 || '.';  # If $0 is not a path, use current directory.

#####################
# But in addition to the above, 
# we do more work here to discover the HyperNews directory.

if ($0 =~ m{setup\.cgi}) {  $plorcgi = 'cgi'; }

# Figure out where we are and where the HyperNews .scripts libraries are.
# Assign $hnbin to the 'HyperNews' directory.

# At this point, $hnbin might be the 'HyperNews' directory,
# $scriptsdir, or $admindir, or somewhere else.   We need to
# normalize it to be just the HyperNews directory, but that might
# be named differently.  

# $hnbin =~ s{/$ }{}x;  # Remove trailing /

$hnbin =~ s{/?$scriptsdir$ }{}x;  # Do this first.
$hnbin =~ s{/?$admindir$ }{}x;

if (!$hnbin || $hnbin eq '.') {
    # If the above fails, try this:
    $hnbin = `pwd`;
    chop($hnbin);

    # Then try again to remove components.
    $hnbin =~ s{/?$scriptsdir$ }{}x;  # Do this first.
    $hnbin =~ s{/?$admindir$ }{}x;
}

# If all else fails, just try current directory.
$hnbin = '.' if !$hnbin;

# Make hnlib be $scriptsdir, inside $hnbin.
$hnlib = "$hnbin/$scriptsdir";
unshift(@INC, $hnbin, $hnlib); 

# Where the hnrc file should be, relative to $hnbin.
$hnrc = "$admindir/hnrc";  

local $debug = 0;  # Don't debug here if used via web.
print qq{
hnbin = $hnbin
hnlib = $hnlib
} if $debug;

# If we can find hn-lib.pl, we can find the rest.
# We need hn-lib.pl to be able to report errors.
require "hn-lib.pl" ||
    die ("Can't load hn-lib.pl");


local $plaintext = 0;  # Must be local out here.

# Default global setup options.  Need to be package vars.
$::plorcgi = 'no';  # now this is always computed.
$::iconsyn = 'no';
$::buildPasswordyn = 'no';


# Map from internal names to externally visible names.
# See scriptsplorcgi below.
%script_names = ();
%other_script_names = ();


&feval('&setup()');

sub setup {
    local $debug = 0;

    # 5.001m may be OK, but not 5.001.
    require 5.002;

    require "setup-lib.pl";

    local ($user_name, $group_name) = &user_group_names();  # Must be local.


    # Decide whether this is a command line or web installation
    if ($ENV{'REQUEST_METHOD'}) {
	$setupMode = 'web';
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
	    &HNUserError(qq{Use <A HREF="setup-form.pl">setup-form.pl</A> or 
<A HREF="setup-short-form.pl">setup-short-form.pl</A>});
	}
    }
    else {
	$setupMode = 'commandline';
	$plaintext = 1;
    }

    
    # Assign default to  global vars, if needed.
    $hndirmode ||= 0775;
    $hnfilemode ||= 0664;

    # Can't print yet, so this better not fail!!
    &create_dir($hnbin);
    &create_dir("$hnbin/$admindir");
    &create_dir("$hnbin/$securedir");


    require "change-member-lib.pl";

    if ($setupMode eq 'web') {
	&setupFromForm();
    } else {
	&setupFromCommandLine();
    }

    &doSetup();
}



sub setupFromForm {

    &print_HTML_HEAD("HyperNews Setup Status");

    print "
<H2>HyperNews Setup Status</H2>

<STRONG>User running HyperNews setup process</STRONG>: $user_name<br>
<STRONG>Group running HyperNews setup process</STRONG>: $group_name<br>";

    &requireHnrc();
    &setDefaultValues();

    require "member-lib.pl";

#==================================
# Check authentication and access control here.

    require "formlib.pl";
    my %form = &GetFormArgs();
    
    my $userid = $ENV{REMOTE_USER} || $form{UserID};
    my $password = $form{Password};

    &checkAuthenticated('', '', $userid, $password);

    # We always require admin if .htaccess exists or $authenticated
    # or external security or manual security and @hnAdmin.
    if ($authenticated ||
	-e "$hnlib/.htaccess" || 
	-e "$hnlib/.www_acl" || -e "$hnlib/.nsconfig"
	|| (($externalSecurity || $manualSecurity) && @hnAdmin)) {
	&access1(3, 2, $remoteUser, $remoteUser,
		 "Setup must be executed by an administrator.
<strong> $remoteUser </strong> is not an administrator!",
		 $authenticated);
    }

#==================================
# First get the parameters from the setup-form or setup-short-form
# so we can save them to the hnrc file.


    # Transfer to variables.
    $httpW = $form {'http'} || $http;

    $remove_cgi_bin = $form {'remove_cgi_bin'} || $remove_cgi_bin;

    $hndocsW = $form {'hndocs'} || $hndocs;
    $hnRootDoc = $form {'hnRootDoc'} || $hnRootDoc;

    $hndirmodeW = $form {'hndirmode'} || $hndirmode;
    $hnfilemodeW = $form {'hnfilemode'} || $hnfilemode;;

    $hniconsW = $form {'hnicons'} || $hnicons;
    $hniconsURLW = $form {'hniconsURL'} || $hniconsURL;
    $iconsyn = $form{'iconsyn'} || $iconsyn;

    $headerText = &unescapeHTML($form {'headerText'});
    $footerText = &unescapeHTML($form {'footerText'});
    $global_body_tag = &unescapeHTML($form {'global_body_tag'});

    $globalDefaultOutlineDepth = $form {'globalDefaultOutlineDepth'} ||
	$globalDefaultOutlineDepth;

    $hnAdminString = $form {'hnAdmin'};
    $hnAdminString =~ s/[ \t\n\r,]+/ /g;  # Reduce whitespace and ','.
    $hnAdminString =~ s/^ *//;  # Delete leading
    $hnAdminString =~ s/ *$//;  # and trailing spaces
    # Somehow, empty hnAdminString still results in @hnAdmin = ('')
    # &HNMsg("admin: '$hnAdminString'");
    @hnAdmin = ();
    @hnAdmin = split (/ /, $hnAdminString) if $hnAdminString;
    
    &HNUserError("At least one administrator must be specified") 
	if !@hnAdmin;

    $manualSecurity = $form{'manualSecurity'} || $manualSecurity;
    # $manualSecurity = $manualSecurity eq 'yes';  -- doesnt work!
    if (($manualSecurity == 1) || ($manualSecurity eq 'yes')) {
	$manualSecurity = 1;
    } else {
	$manualSecurity = 0;
    }

    $hnPeopleW = $form {'hnPeople'} || $hnPeople;
    $hnPasswordW = $form {'hnPassword'} || $hnPassword;

    $buildPasswordyn = $form{'buildPasswordyn'};
    $hnGroupW = $form {'hnGroup'} || $hnGroup; # This might be a problem

    $securityLevel = $form {'securityLevel'};

    if (!$securityLevel) {
	$hnReadSF = $form {'hnReadSF'};
	$notifySF = $form {'notifySF'};
	$addArticleSF = $form {'addArticleSF'};
	$addResponseSF = $form {'addResponseSF'};
	$joinSF = $form {'joinSF'};
	$resetPasswordSF = $form {'resetPasswordSF'};
	$deleteSF = $form {'deleteSF'}; 
	$moveSF = $form {'moveSF'};

    } elsif ($securityLevel eq 'customized') {
	@who = ('No One', 'Anyone', 'Only Members', 'Only Admin');

	$hnReadSF = &lindex($form {'hnReadSF'}, @who);
	$notifySF = &lindex($form {'notifySF'}, @who);
	$addArticleSF = &lindex($form {'addArticleSF'}, @who);
	$addResponseSF = &lindex($form {'addResponseSF'}, @who);
	$joinSF = &lindex($form {'joinSF'}, @who);
	$resetPasswordSF = &lindex($form {'resetPasswordSF'}, @who);
	$deleteSF = &lindex($form {'deleteSF'}, @who);
	$moveSF = &lindex($form {'moveSF'}, @who);

    } elsif ($securityLevel eq 'freeforall') {
	$hnReadSF = 1;
	$notifySF = 1;
	$addArticleSF = 3;
	$addResponseSF = 1;
	$joinSF = 1;
	$resetPasswordSF = 3;
	$deleteSF = 2;
	$moveSF = 2;
    } elsif ($securityLevel eq 'open') {
	$hnReadSF = 1;
	$notifySF = 2;
	$addArticleSF = 3;
	$addResponseSF = 2;
	$joinSF = 1;
	$resetPasswordSF = 3;
	$deleteSF = 2;
	$moveSF = 2;
    } elsif ($securityLevel eq 'private') {
	$hnReadSF = 2;
	$notifySF = 2;
	$addArticleSF = 2;
	$addResponseSF = 2;
	$joinSF = 2;
	$resetPasswordSF = 2;
	$deleteSF = 2;
	$moveSF = 2;
    }

    $hnEmailNotify = $notifySF > 0;

    #============================================

    # This is not really sufficient.  Any suffix might be used, not
    # just .cgi.  Installer needs to specify the suffix to use.
    # But more changes are required below.
    $scriptSuffix = ".cgi";
    if ($ENV{SCRIPT_NAME} =~ m|$scriptSuffix$| ) {
	$plorcgi = 'cgi';
    }

    &createHnrc();

    # Reload hnrc file.
    do "$hnbin/$hnrc" ||
	&HNError ("Loading $hnbin/$hnrc\ncaused error: $@");
}


sub lindex {
    my ($string, @list) = @_;
    # Return the index of the $string in the @list, -1 if not there.
    my $i = 0;
    foreach $e (@list) {
	return $i if $string eq $e;
	$i++;
    }
    return -1;
}



sub setupFromCommandLine {

  $plaintext = 1;

    print qq{
HyperNews Setup
};

  # Running from command line.
  my $hnrcIsNew = !&requireHnrc();

  &setDefaultValues();

  # Default command line values.  Need to be package vars.
  $::cgiwrap = 'no';  # use `make cgiwrap`

  $::copy_or_link ||= 'symlink';  # stored in hnrc

  print "User running HyperNews setup process: $user_name\n";
  print "Group running HyperNews setup process: $group_name\n";
    
  #============================
  # Prepare for createHnrc.

  $httpW = $http;

  $hndocsW = $hndocs;
#    $hndocsURLW = $hndocsURL;

  $hniconsW = $hnicons;	
  $hniconsURLW = $hniconsURL;

  $hnPeopleW = $hnPeople;
  $hnPasswordW = $hnPassword;
  $hnGroupW = $hnGroup;

  $hndirmodeW = sprintf("%0.4o", $hndirmode);
  $hnfilemodeW = sprintf("%0.4o", $hnfilemode);


# Options for command line execution:
#  --cgi - add '.cgi' to scripts.  - default is 'yes' if you run setup.cgi.
#  --cgiwrap - enable use of cgiwrap - default is 'no'
#  --copy scripts rather than link to them. - default is symlink
#  --hardlink scripts rather than copy them. - default is symlink
#  --symlink scripts rather than copy or hardlink them. - default is symlink
#  --icons - copy icons to icon dir. - default is no except first time.
#  --rebuild - rebuild password (and group) file. - default is 'no'.

  # Assign parameters from the command line to override hnrc.
  print qq{
Command line options:
};


  push (@INC, "$hnbin/.scripts/lib");
  require Getopt::Declare; import Getopt::Declare;
  # like: use Getopt::Declare;


  # Command line args:
  #  [required] doesnt work with [ditto]
  # --verbose		Verbose mode - doesnt work with --version
  # { $::debug = 1; }

  ##################################################################
  my $optctl = q{
	--cgi			Add .cgi to scripts.
	{ $::plorcgi = 'cgi' }

	--copy			Copy scripts rather than link.
	{ $::copy_or_link = 'copy' }
	-N			[ditto]

	--symlink		Symlink scripts, not hard link.
	{ $::copy_or_link = 'symlink' }
	--link			[ditto]

	--hardlink		Hard link scripts, not symlink.
	{ $::copy_or_link = 'hardlink'}

	--cgiwrap		Add links to use cgiwrap.
	{ $::cgiwrap = 'yes' }

	--icons			Install Icons directory.
	{ $::iconsyn = 'yes' }
        -I			[ditto]

	--rebuild		Rebuild the password file.
	{ $::buildPasswordyn = 'yes' }
	-P			[ditto]

	--debug			Include debug output.
	{ $::debug = 1 }

        --check			Check only.
	{ $::check_only = 1 }

        --help			This display.
};

  ##################################################################

    $args = new Getopt::Declare($optctl);

  print qq{
  Copy or Link Scripts: $::copy_or_link
  Add links for cgiwrap: $::cgiwrap
  Install Icons Directory: $::iconsyn
  Build Password Info:  $::buildPasswordyn
};   #  if $debug


  exit 0 if ($check_only);

  # Now create it.
  &createHnrc();

  if ($hnrcIsNew) {
    # Give user a chance to fix it up.
    print "\nNow edit hnrc and rerun setup.pl.\n";
    exit;
  }

  do "$hnbin/$hnrc" ||
    &HNError ("Syntax error or cant locate hnrc in $hnbin/$hnrc.
  Error: $@\n");

  &HNUserError("At least one administrator must be specified") 
    if !@hnAdmin;
}


######################################################################
######################################################################
######################################################################
######################################################################
######################################################################

sub doSetup {
# Do the rest of the setup from the hnrc configuration and options.

#======================================
# Create/check directories

# Need to create required directories not already there and test them
    &setupScriptDirs();

    print &TextorHTML("<p>Creating/Checking \$hndocs directory...");
    &create_dir($hndocs); 
    &testReadWrite($hndocs); 
    print &TextorHTML("<STRONG>OK</STRONG>");

    print &TextorHTML("<p>Creating/Checking \$hnicons directory...");
    &create_dir($hnicons);
    &testReadWrite($hnicons);
    print &TextorHTML("<STRONG>OK</STRONG>");

    print &TextorHTML("<p>Creating/Checking \$hnPeople directory...");
    &create_dir($hnPeople);
    &testReadWrite($hnPeople);
    print &TextorHTML("<STRONG>OK</STRONG>");

    print &TextorHTML("<p>Creating/Checking \$hnGroup directory...");
    &create_dir("$hnPeople$hnGroup");
    &testReadWrite("$hnPeople$hnGroup");
    print &TextorHTML("<STRONG>OK</STRONG>");


#======================================

    if ($buildPasswordyn eq "yes") {
	&rebuildPasswordGroup ();
    }

    &checkAdmin();

    #=================================================
    # Check to see if group and password files were created properly,
    # after having added Admin.

#    if (!-e "$hnPeople$hnGroup/$hnGroupMembers") {
#	&HNError ("HyperNews did not create $hnPeople$hnGroup/$hnGroupMembers.");
#    }

    if (!-e $hnPassword) {
	# Just touch it.
	open (HN, ">$hnPassword") ||
	    &HNError ("HyperNews could not create $hnPassword");
	close HN;
    }

    &setupScripts();

    &setupIcons();

#####################################################################
# Finished Successfully.

    # Need to fix up URLs for edit-article.pl and edit-member.pl
    # when setup.pl is run from command line.

    print &TextorHTML (qq{

<HR>
<h3>HyperNews setup was successful.</h3>

You may want to change the permissions on <P>
<PRE>
  $hnbin
</PRE>

and its subdirectories to be non-writable, but you\'ll have to change
them back before running setup again. <P>

Now you may add new members at:
<PRE>
  <A HREF="$editMemberURL">$editMemberURL</A>
</PRE>

Or you can create a new base article at:
<PRE>
  <A HREF="$editArticleURL">$editArticleURL</A> 
</PRE>

<P>
For a top level page with brief instructions for administrators, see:

<PRE>
  <A HREF="$AdmintopURL">$AdmintopURL</A>
</PRE>

See the full instructions for creating forums at: 

<PRE>
  <A HREF="http://www.hypernews.org/HyperNews/get/hypernews/base-articles.html">http://www.hypernews.org/HyperNews/get/hypernews/base-articles.html</A>
</PRE>
});

    exit(0);	# this exit is used to make it work with PERL 4
    # exit 1 looks like an error.

} # End of doSetup()



sub checkAdmin {


    print &TextorHTML("<P>Checking Administrators...");
    print STDERR "$hnAdminString\n" if $debug;

# Check that there are non-zero admin.
    local($Admincount) = @hnAdmin + 0;
    if (!$Admincount) {
	&HNUserError ("No Administrators listed");
    }

    print &TextorHTML("<STRONG>OK</STRONG> ($Admincount entered)");

#=====================================
# Create new Admin group file.
# We do this so any deletions from @hnAdmin take effect.
# Problem: if error occurs while adding admin, all later admin will be dropped.

# First save the current version in -old file.
    &create_dir("$hnPeople$hnGroup/Admin");

# No longer needed.
#    local($adminMemberFile) = "$hnPeople$hnGroup/Admin/$hnGroupMembers";
#    if (-e $adminMemberFile) {
#	unlink ("$adminMemberFile-older");
#	rename ($adminMemberFile, "$adminMemberFile-older");
#    }

    
#=====================================
# Add any new admin, and check for passwords.
    foreach $userid (@hnAdmin) {
	local (%member) = &getMember($userid);
	local ($olduserid) = $member{'UserID'};
	local ($password) = $member{'Password'};
	local($debug) = 0;


	&HNMsg("Checking $userid") if $debug;

	if (!&useridOK($userid)) {
	    print &TextorHTML("
<P> <STRONG> Warning: Admin, \"$userid\", 
 has a bad userid or email address.</STRONG>\n");
	    next;
	}
	if (!$password  && !$externalSecurity) {
	    if (!$olduserid) {
		# Create a random password for a new admin.
		# my $password = &rand_passwd();
		my $password = '';  # Leave it blank.
		my $email;
		if (&addressOK($userid)) {
		    $email = $userid;
		}
		&changeMember($remoteUser, $userid, 'authenticated',
			      ('Name' => 'New Admin',
			       'Password1' => $password, 
			       'Password2' => $password,
			       'Email' => $email));

		print &TextorHTML("
<P> <STRONG> New administrator, $userid, has password: \"$password\". </STRONG>\n");
	    }
	    elsif ($requirePasswords) {
		# Reset password of existing admin.
		# $password = &resetPassword($userid);  No! causes confusion.
		# print &TextorHTML("
#<P> <STRONG> Administrator, $userid, has a new password
#      \"$password\". </STRONG>\n");
	    }
	    else {
		print &TextorHTML("
<P> <STRONG>Warning:</STRONG> $userid has no password.\n");
	    }
	}

	&HNMsg("adding $userid to $hnGroup/Admin") if $debug;
	# Really should do this with addtogroupBatch.
	&addtogroup("$hnGroup/Admin", $userid);
    }
}


sub setupScriptDirs {
# Set up each of the scripts directories, 

#======================================
# Set up the $securedir

    local $debug = 0;

    print &TextorHTML("<p>Setting up $hnbin directories ...");
    &create_dir("$hnbin/$securedir");
    &symlinkIt ("../$scriptsdir", "$hnbin/$securedir/$scriptsdir");
#    &symlinkIt("../$admindir/$hnrc", "$hnbin/$securedir/hnrc");
    &symlinkIt ("../$securedir", "$hnbin/$scriptsdir/$securedir");
    print &TextorHTML("<STRONG>OK</STRONG>"); 

#======================================
# Set up the $admindir
# This provides Admin only access
# to all scripts, the same as you get in the .scripts directory.
# But .scripts might be shared by other installations.
# Even so, access to SECURED scripts checks whether admin is required.
# The only difference is when using Cookies - see member-lib.pl.

    print &TextorHTML("<p>Setting up $hnbin/$admindir ...");
    &create_dir("$hnbin/$admindir");
    &symlinkIt ("../$scriptsdir", "$hnbin/$admindir/$scriptsdir");

    # Link Admin to itself, for transparency.
    &symlinkIt (".", "$hnbin/$admindir/$admindir");

    # Link from .scripts and SECURED to Admin.
    &symlinkIt ("../$admindir", "$hnbin/$scriptsdir/$admindir");
    &symlinkIt ("../$admindir", "$hnbin/$securedir/$admindir");

    print &TextorHTML("<STRONG>OK</STRONG>"); 
}


sub setupScripts {
# setup access files, and links to scripts.

    #======================================
    # Creating Access Files

    &unlinkIt("$hnbin/$htaccess");
    &unlinkIt("$hnbin/$securedir/$htaccess");
    &unlinkIt("$hnbin/$admindir/$htaccess");
    # other dirs? other files?

    if ($manualSecurity == 0 && $externalSecurity == 0) {
	print &TextorHTML("<p>Writing $htaccess files ...");

	if ($cgiwrap eq 'yes') {
	  # Add links to cgiwrap.
	  # Also need to link to the real cgiwrap, wherever it is.
	  if (!-e "$hnbin/cgiwrap") {
	    # Only link this one if not already linked.
	    &symlinkIt("../cgiwrap", "$hnbin/cgiwrap");
	  }
	  &symlinkIt("cgiwrap", "$hnbin/cgiwrap.cgi");
	  &symlinkIt("cgiwrap", "$hnbin/cgiwrapd");
	}

	# Generate the correct paths for the CERN server
	my($hnRelativeDir) = $ENV{SCRIPT_NAME};
	$hnRelativeDir =~ s|/$scriptsdir/[^/]*$||;

	# Create the access file.
	# We need to write this new access file, but not while
	# still using the old version that relies on server checking
	# of group membership.  So the $hnaccess file is used instead..
	&createAccessFile($hnGroup, $groupString);

	if (!$hnGroup) { # Only do this for root level.
	    # Access to $scriptsdir should be Admin only.
	    # Access control is now done by checkAuthenticated.
	    &createAccessFileLink("$hnbin/$scriptsdir/$htaccess",
				  "$hnPeople/$hnaccess",
				  "$hnPeople/$hnGroupMembers",
				  "$hnRelativeDir/$scriptsdir");
	    # Access to the $hnPeople dir should be Admin only.
	    &createAccessFileLink("$hnPeople/$htaccess",
				  "$hnPeople/$hnaccess",
				  "$hnPeople/$hnGroupMembers",
				  # This might be wrong
				  "$hnPeople");

	    if ($cgiwrap eq 'yes') {
	      &symlinkIt("../cgiwrap", "$hnbin/$scriptsdir/cgiwrap");
	      &symlinkIt("cgiwrap", "$hnbin/$scriptsdir/cgiwrap.cgi");
	      &symlinkIt("cgiwrap", "$hnbin/$scriptsdir/cgiwrapd");
	    }
	}

	# Access to Admin directory should be Admin only.
	&createAccessFileLink("$hnbin/$admindir/$htaccess",
			      "$hnPeople$hnGroup/$hnaccess",
			      "$hnPeople$hnGroup/$hnGroupMembers",
			      "$hnRelativeDir/$admindir");

	&createAccessFileLink("$hnbin/$securedir/$htaccess",
			      "$hnPeople$hnGroup/$hnaccess",
			      "$hnPeople$hnGroup/$hnGroupMembers",
			      "$hnRelativeDir/$securedir");

	if ($cgiwrap eq 'yes') {
	  &symlinkIt("../cgiwrap", "$hnbin/$admindir/cgiwrap");
	  &symlinkIt("cgiwrap", "$hnbin/$admindir/cgiwrap.cgi");
	  &symlinkIt("cgiwrap", "$hnbin/$admindir/cgiwrapd");

	  &symlinkIt("../cgiwrap", "$hnbin/$securedir/cgiwrap");
	  &symlinkIt("cgiwrap", "$hnbin/$securedir/cgiwrap.cgi");
	  &symlinkIt("cgiwrap", "$hnbin/$securedir/cgiwrapd");
	}

	if ($hnReadSF == 1) {
	    &unlinkIt("$hnbin/$htaccess");   # We need to do this.
	    &unlinkIt("$hndocs/$htaccess");  # Should we do this?
                          # The Icons dir needs to be readable
	    if ($serverType =~ 'CERN') {
		# Make the dir accessible.
		open (ACL, ">$hnbin/$htaccess");
		print ACL '* : GET,POST : @*.*.*.*' . "\n";
		close ACL;
		print &TextorHTML("
<H3>Add this to your server configuration - fix as needed</H3>

<pre>
Protect $hnRelativeDir/* {
        UserID  &lt;uid&gt;  # user id for server process to run as
        GroupId &lt;gid&gt;  # group id for server process to run as
}
</PRE>
");
	    }
	} else {
	    &createAccessFileLink("$hnbin/$htaccess",
				  "$hnPeople$hnGroup/$hnaccess",
				  "$hnPeople$hnGroup/$hnGroupMembers",
				  $hnRelativeDir);
	    # Protect $hndocs the same as $hnbin.
	    &createAccessFileLink("$hndocs/$htaccess",
				  "$hnPeople$hnGroup/$hnaccess",
				  "$hnPeople$hnGroup/$hnGroupMembers",
				  # This might be wrong
				  $hndocs);
	}
	print &TextorHTML("<STRONG>OK</STRONG>"); 
    }

#======================================
# Link all the scripts into appropriate directories.

    print &TextorHTML ("<P>Linking to scripts...");
    &scriptsplorcgi();

    &linkScript(1, 'top.pl', 'login.pl');
    # Should link (or rename) top.pl to index.cgi, for default Apache config!!

    &linkScript(1, 'get', 'urc', 'thread.pl');  
    #		'an_get', 'get_ann', 'ann_cgi',
    # Could be $hnReadSF instead of 1, but that is broken for now.
    # If $hnReadSF > 1, then the HyperNews dir will be protected anyway.

    # Hardwire the view-member commands to be either members only or public.
    # Should be a new viewMemberSF.
    &linkScript($authenticateInquiries ? 2 : 1, 'view-member.pl');
    &linkScript(1, # was: $authenticateInquiries ? 2 : 1
		# Maybe needs to be 1 so users can get to the "Register" link.
		'view-members.pl');

    &linkScript($joinSF, 'edit-member.pl', 'change-member.pl');
    &linkScript(3, 'delete-members.pl', 'delete-members.pl');
    &linkScript($resetPasswordSF, 'reset-password.pl');

    &linkScript($addArticleSF, 'edit-article.pl', 'add-article.pl',);
    &linkScript($addResponseSF, 
		'edit-response.pl', 'preview-response.pl', 'add-response.pl');

    # What about $moveSF?
    &linkScript($deleteSF, 'admin-responses.pl');
    &linkScript($notifySF, 'edit-subscribe.pl');

    &linkScript(3, 'setup.pl', 'setup-form.pl', 'setup-short-form.pl');

    print &TextorHTML("<STRONG>OK</STRONG>"); 
}


sub setupIcons {

    print STDERR "hnfilemode = $hnfilemode\n" if $debug;
    if ($iconsyn eq "yes" || !-e "$hnicons/HyperNews.gif") {
	print STDERR "Checking Installatiyon of Icons ...\n" if $debug;
	print &TextorHTML("<P>Installating Icons.");

	&create_dir($hnicons);

	opendir(DIR, "$hnlib/Icons");
	@IconsinDir = readdir(DIR);
	closedir(DIR);

	foreach $file (@IconsinDir) {
	    if (($file ne ".") && ($file ne "..")) {
	      print STDERR "Changing mode for $file\n" if $debug;
	      &doexec("/bin/cp", "$hnlib/Icons/$file", "$hnicons/$file");
	      chmod ("$hnfilemode", "$hnicons/$file");
	    }
	}
    }

    if ($setupMode eq "web") {
	print ("<P>If installation of icons has been done, 
you should see the HyperNews icon here: 
<IMG SRC=\"$hniconsURL/HyperNews.gif\">\n");
    }
    else {
	print ("OK"); 
    }

    if (!-e "$hniconsTable") {
	print &TextorHTML("Writing Icons Table<p>");
	&doexec("cp", "$hnlib/icons-table-dist", $hniconsTable);
	chmod ("$hnfilemode", $hniconsTable);
	if (!-e $hniconsTable) {
	    &HNError("Could not copy '$hnlib/icons-table-dist' to 
'$hniconsTable'<P>");      
	}
    }
}


sub requireHnrc {
  # Return 1 if hnrc exists already, otherwise return 0.
    if (-e "$hnbin/$hnrc") {
	require "$hnbin/$hnrc";
	print &TextorHTML ("<P>Found hnrc in $hnbin/$admindir");
	return 1;
    } 
    # Check the old places too.
    elsif (-e "$hnbin/$hnrc") {
	require "$hnbin/$hnrc";
	print &TextorHTML ("<P>Found hnrc in $hnbin");
	return 1;
    } 
    elsif (-e "$hnlib/hnrc") {
	require "$hnlib/hnrc";
	print &TextorHTML ("<P>Found hnrc in $hnlib");
	return 1;
    }
    return 0;
}


sub createHnrc {
    # Create hnrc file.

    # Computing Script names
    &scriptsplorcgi();
    print &TextorHTML ("<p>Writing $hnbin/$hnrc ... ");

    &HNMsg("hnlib = $hnlib \nhnbin = $hnbin") if $debug;

    #=========================
    # General preparation.

    # The part of the SCRIPT_NAME that might be removed.
    $scriptPrefix = "/cgi-bin";

    $hnbinURL = "\$hnprefix";  # Maybe this is sufficient. - not -

    # Some servers seem to not like relative redirects, so make it absolute.
    if ($hnprefix !~ m,^http,) {
      $hnbinURL = "\$http\$hnprefix";
    }

    # Add $mail_admin to @hnAdmin if it's not already there.
#    if ($mail_admin) {
#	if (&lindex($mail_admin, @hnAdmin) < 0) 
#	{ push @hnAdmin, $mail_admin; }
#    }

    # Put the adminstrators in a list of quoted strings.
    $hnAdminString = "('" . join("', '", @hnAdmin) . "')";

    $hnPeopleW =~ s,$hndocs/,\$hndocs/,;
    $hnPeopleW =~ s,/$,,;
    # The following line might be causing problems, if $hnPasswordW is null.
    $hnPasswordW =~ s{$hnPeople/}{\$hnPeople/} if $hnPasswordW;

    $hnGroupW =~ s,/$,,;  # Remove trailing /, if it is there.
    $hnGroupW =~ s,^[^/],/$&,;	# Prepend /, if non-/ is there.

    $headerText = &quotify($headerText);
    $footerText = &quotify($footerText);
    $global_body_tag = &quotify($global_body_tag);

    $hndocsW =~ s,/$,,;
    $hnRootDoc =~ s,$hndocs\B,\$hndocs\B,;

    $hniconsW =~ s,$hndocs/,\$hndocs/,;
    $hniconsW =~ s,/$,,;

    $hniconsURLW =~ s,$http,\$http,;
    $hniconsURLW =~ s,/$,,;

    $messageFormatsString = "('" . join("', '", @messageFormats) . "')";

    $hnmidDB =~ s,$hndocs,\$hndocs,;

    $hnReadSF ||= 0;
    $notifySF ||= 0;
    $addArticleSF ||= 0;
    $addResponseSF ||= 0;
    $joinSF ||= 0;
    $resetPasswordSF ||= 0;
    $deleteSF ||= 0;
    $moveSF ||= 0;

    #=========================================
    # Write the file.
    &create_dir($hnbin);
    open(FILE, ">$hnbin/${hnrc}_tmp") 
	|| &HNError("Cannot open for writing $hnbin/${hnrc}_tmp");

    print FILE qq{#!/powered/by/perl

# This is a configuration file for HyperNews.

# Note: you must run setup.pl to have some changes take effect.
# If you change any known variables here, the new values should 
# continue to be used each time you run setup.pl.
# Some variables get initial values in setup-lib.pl.
# See instructions in source.html and setup-form.pl to do it right.

# If you change the HyperNews source significantly, change \$hnversion in
# setup-lib.pl Also change VERSION in Makefile.
\$hnversion = "$hnversion";

require 5.002;      # 5.001m may be OK, but not 5.001.
require 'hn-lib.pl';

# Avoid tainting by limiting execution path.
\$ENV{'PATH'} = '/bin:/usr/bin';
 
# If you setup from command line (not via http), assign the right values.
# These are used by the mail-incoming gateway.  E.g.
# \$ENV{SERVER_NAME} ||= 'www.hypernews.org';
# \$ENV{SERVER_PORT} ||= '80';
\$ENV{SERVER_NAME} ||= "$ENV{SERVER_NAME}";
\$ENV{SERVER_PORT} ||= "$ENV{SERVER_PORT}";

# HTTP URL prefix for docs on your server.  E.g. "http://\$ENV{SERVER_NAME}";
\$http = "$httpW";  # No trailing "/".

# Path prefix of HyperNews cgi-bin directory, relative to server root.
# E.g. \$hnprefix ||= "/HyperNews";
\$hnprefix ||= "$hnprefix";
# \$hnrootprefix ||= "$hnrootprefix";  # corresponds to root hnpeople.

    &repair_SCRIPT_NAME();
    &compute_hnprefix();

# Regexp for matching equivalent prefixes - all have the same \$hndocs.  
# E.g. '/HyperNews|/HyperNewsTest'  - not used yet.
\$hnprefixEquiv = '$hnprefixEquiv';

# Compute full URL prefix for HyperNews scripts.
\$hnbinURL = "$hnbinURL";

# Maybe remove '$scriptPrefix' from \$hnbinURL
\$remove_cgi_bin = "$remove_cgi_bin";
if (!\$ENV{REQUEST_URI} && \$remove_cgi_bin eq 'yes') {
  \$hnbinURL =~ s,$scriptPrefix,,;
} 

# Whether scripts are copied or linked: 'copy', 'link', or 'hardlink'.
\$copy_or_link = '$copy_or_link';

    &compute_hnSECUREDURL();

# For http.pl
# If you set \$httpProxyHost, you must also set \$httpProxyPort, e.g. to 3180
\$httpProxyHost = '$httpProxyHost';
\$httpProxyPort = '$httpProxyPort';

# Whether HyperNews should use full URLs when making http requests.
# Must use full URL if using a proxy.
\$useFullHTTPURL = '$useFullHTTPURL' || '$httpProxyHost';

# Whether HN should support cachability of HN pages.
\$supportCaching = $supportCaching;
# Whether to actively defeat caching, even within browser history.
\$disableCaching = $disableCaching;

# How long to wait for http result.
\$timeout = $timeout;

} || &HNError("Cannot write to $hnbin/${hnrc}_tmp");

# Break this up into smaller chunks - some Perls have problems otherwise.

print FILE qq{

# Where HN documents (not scripts) are stored.  
# Don\'t put them in your cgi-bin.
# E.g. $hndocs = "/usr/local/etc/httpd/htdocs/hn"
\$hndocs = "$hndocsW";
# \$hndocsURL = "$hndocsURLW"; # same directory as \$hndocs - not used anymore.

# Last resort when getting fails.  Relative to \$hndocs, or full URL.
# E.g. "/hnroot.html"
\$hnRootDoc = "$hnRootDoc";

# Whether to use the HN icons for kinds of messages.
\$useHNIcons = $useHNIcons;

# Where HN icons are stored.
# E.g. "\$hndocs/Icons"
\$hnicons = "$hniconsW";
\$hniconsURL = "$hniconsURLW";  # URL for same directory as \$hnicons
\$hniconsTable = "\$hnicons/icons-table";

\$newest=1;
\$newestIcon="\$hniconsURL/newest.gif";
\$newer=7;
\$newerIcon="\$hniconsURL/newer.gif";
\$new=14;
\$newIcon="\$hniconsURL/new1month.gif";

# Icon sizes
\$newIconXsize = $newIconXsize;
\$newIconYsize = $newIconYsize;
\$relationIconXsize = $relationIconXsize;
\$relationIconYsize = $relationIconYsize;
\$depthIconXsize = $depthIconXsize;
\$depthIconYsize = $depthIconYsize;
\$commandIconXsize = $commandIconXsize;
\$commandIconYsize = $commandIconYsize;
\$threadIconXsize = $threadIconXsize;
\$threadIconYsize = $threadIconYsize;

# Display customizations

\$UserIDorHandle = '$UserIDorHandle';

\$headerText = '$headerText';
\$footerText = '$footerText';
# BODY tag.
\$global_body_tag = '$global_body_tag';

\$startButtons = '$startButtons';
\$endButtons = '$endButtons';

# What to wrap around inlined messages and outline response tree.
\$startInline = '$startInline';
\$startInlineBody = '$startInlineBody';
\$endInlineBody = '$endInlineBody';
\$startInlineResponses = '$startInlineResponses';
\$endInlineResponses = '$endInlineResponses';
\$endInline = '$endInline';

# What to wrap around the message body, both when inlined and not.
\$startMsgBodyTag = '$startMsgBodyTag';
\$endMsgBodyTag = '$endMsgBodyTag';

# How deep the outline display is by default.  Overridden by forums.
\$globalDefaultOutlineDepth = '$globalDefaultOutlineDepth';
\$globalDefaultInlineDepth = '$globalDefaultInlineDepth';

# Whether message numbers should be displayed in the outline (0 or 1).
\$printResponseNum = $printResponseNum;
# Whether the date should be displayed in the outline (0 or 1).
\$printResponseDate = $printResponseDate;
# Whether the normal order of messages should be reversed, when written. 
# (0 = no reverse; -1 = reverse all;  1 = reverse top-level only).
# (Takes effect after new messages are written.)
\$reverseMessages = $reverseMessages;
# Whether the command buttons, and Add button, should also be on top.
# It is always on top if \$reverseMessages == 1.
\$buttonsBeforeMessages = $buttonsBeforeMessages;

\$simpleButtons = $simpleButtons;
\$textButtons = $textButtons;
\$centerButtons = $centerButtons;

# Whether the Members button goes to view-members or edit-member.
\$viewMembers = $viewMembers;

# Whether frames support should be provided.
\$allowFrames = $allowFrames;
# Whether messages can have replies (resulting in threads).  Not fully working.
\$allowThreads = $allowThreads;
# Whether messages may be posted without preview.
\$allowDirectPost = $allowDirectPost;
# Whether user is returned to the forum after a posting.
\$gotoForumAfterPost = $gotoForumAfterPost;

# Whether subforums, sibling forums, and parent forums should be listed.
\$displayParents = $displayParents;
\$displaySiblings = $displaySiblings;
\$displayChildren = $displayChildren;

# Whether sibling and child forum lists should be generated as needed.
\$autoGenerateSiblingForumList = $autoGenerateSiblingForumList;
\$autoGenerateChildForumList = $autoGenerateChildForumList;

# Whether to sort by dates and display date selectors.
\$sortByDate = $sortByDate;
# Date boundary specs, used by Date::Manip::ParseRecur; see setup-lib.pl
\$dateBoundaries = '$dateBoundaries';
# Format of date selectors; used by Date::Manip::UnixDate
\$dateFormat = '$dateFormat';

# Which formats to allow for message bodies.  Could be: 
# \@messageFormats = ('Smart Text', 'Word Processor', 'Plain Text',
#	              'HTML', 'URL');
\@messageFormats = $messageFormatsString;

# Default URL for Search Form, for all forums.
\$globalSearchURL = '$globalSearchURL';

# Directory and file modes.  Leading zero is required!
# To have directory user (or group) ownership transferred to files, 
# use e.g. \$hndirmode = 04775;  (or 02775 for group)
\$hndirmode = $hndirmodeW;
\$hnfilemode = $hnfilemodeW;

# Server Type -  Use CERN, or NCSA.  Use NCSA for Apache.
\$serverType = '$serverType' || 'NCSA';

# HyperNews Administrators: comma separated Email or UserID strings.
# First one is special; it is listed as the contact for problems.
# E.g. ('liberte\@hypernews.org', 'HNMail')
\@hnAdmin = $hnAdminString;

# Whether HyperNews should always authenticate "manually" via forms.
# If 1, no server authentication will be setup by HyperNews.
\$manualSecurity = $manualSecurity;

# Whether authentication is *managed* outside of HyperNews.
# May be server based or cookie based or something else.
# If 1, no server authentication will be setup and managed by HyperNews.
# Use per-user authentication if you want HyperNews to know who the user is.
\$externalSecurity = $externalSecurity;

# Require passwords - no empty passwords allowed - should be '1'.
\$requirePasswords = $requirePasswords;
# Automatically generate passwords for new members - emailed to them.
\$generatePasswords = $generatePasswords;

# Whether HyperNews should generate and use cookies for authentication.
# 'always', 'never', or 'sometimes'. If 'sometimes', check per-member option.
\$UseCookies = '$UseCookies';
# If cookie lifespan is 0, cookies die with the browser.
\$CookieLifespan = $CookieLifespan;  # in days.

# HNCookiePassword is the key used when making the cookie password.
# If you do this, make SURE only those you trust can see this hnrc file.
\$HNCookiePassword = '$HNCookiePassword';

# Where member info is stored.
\$hnPeople = "$hnPeopleW";

# Where the password file is.
\$hnPassword = "$hnPasswordW";

# The name of the subgroup for this installation.
# May be "" or "/groupname" or "/group/subgroup".
\$hnGroup = "$hnGroupW";

# The identifier for the realm - should be unique across all groups.
# This is shown to the user when authenticating.
\$hnRealm = "$hnRealm";

# The name of the installation.  Could be the same as the group, if any.
# This is displayed to the user to refer to the system. E.g. "Car Talk".
\$hnSystemName = "$hnSystemName";

# For Netscape DBM password files, set \$PWFileDBM to 1.  This may not
# work with groups.  Doesnt work with rebuild of password file.
\$PWFileDBM = $PWFileDBM;

# Special Admin that only posts HN replies from email on behalf of users.
# Don\'t use a normal Admin.  But do add it to \@hnAdmin, and run setup.
\$mail_admin = '$mail_admin';
# The password you have given \$mail_admin - not encrypted.  
# This password must be non-blank to be functional.
\$mail_admin_password = '$mail_admin_password';

# Whether email is used for other things besides notification.
\$hnEmail = $hnEmail;
# Whether email addresses are allowed for user IDs.
# If you change this, also rebuild the password file.
\$allowEmailUserID = $allowEmailUserID;

# Don\'t change these at all, unless you want to experiment.  Not done yet!
# Don\'t change these after adding members unless you convert.
# Whether User IDs and Email Addresses are made lower case for uniqueness.
\$lower_case_IDs = $lower_case_IDs;
# Whether User IDs may contain spaces (converted to '_') - may be dropped.
# Only works with form-based authentication anyway.
\$allow_spaces_in_userids = $allow_spaces_in_userids;

# Privacy settings.  The User Name is not hidden.
\$hideUserIDs = $hideUserIDs;
\$hideEmailAddresses = $hideEmailAddresses;
# Whether users must authenticate to inquire about their own info.
\$authenticateInquiries = $authenticateInquiries;
# Whether to request personal data on edit-member.pl
\$getPersonalData = $getPersonalData;

# Whether to show the priviledges in edit-member.pl
\$showPriv = $showPriv;

# Security options: 0 = No one; 1 = Any one; 2 = Members Only; 3 = Admin Only
# Note: you must rerun setup for changes to take effect.
\$joinSF = $joinSF;  # Who can join
\$resetPasswordSF = $resetPasswordSF;  # Who may reset passwords
\$addArticleSF = $addArticleSF;   # Who may create new base articles
\$hnReadSF = $hnReadSF;  # Who may read articles and responses
\$notifySF = $notifySF;  # Who may change their subscription info
\$addResponseSF= $addResponseSF;  # Who may add responses
\$deleteSF = $deleteSF;  # Who may delete responses
\$moveSF = $moveSF;  # Who may move responses

# Whether email is used for notification.  \$hnEmail must be 1 also.
\$hnEmailNotify = \$notifySF > 0;

# Whether the default for notification of responses is on (1) or off (0).
\$defaultNotification = $defaultNotification;

# Whether each member is subscribed to all-member forums by default.
\$defaultSubscribeAll = $defaultSubscribeAll;

# Which forums are all-member forums?  'none', 'some', or 'all'
# But all members are subscribed only if \$defaultSubscribeAll is non-zero.
\$allMemberForums = '$allMemberForums';

}; 


# Break this up into smaller chunks - some Perls have problems otherwise.

print FILE qq{

# The program to use for sendmail.  Many options to consider.
# -oem means mail error notification to sender.
# -oeq means be quiet about errors, but write to ~/dead.letter.
# -oi means ignore leading dots in message
# -odb means background (asynchronous) delivery.
# -odi means interactive (synchronous) delivery. might help
# -odq means queued delivery.  Need 'sendmail -q' later
\$sendmail = '$sendmail';

# Defaults for who email is sent to, from, who is the sender, etc.
# These may need to be existing mail aliases on your system.
# If Null strings, they are not used.
\$mail_to = '$mail_to';
\$mail_from = '$mail_from';  # Used only if author is unknown.
\$mail_sender = '$mail_sender';
\$mail_reply_to = '$mail_reply_to'; # Not recommended.
\$mail_errors_to = '$mail_errors_to';

# Defaults for allowing mail gateway, messageid-database.
\$allowEmailIn = $allowEmailIn;
\$hnmidDB = "$hnmidDB";

# The name of the access restriction file.
\$htaccess = '$htaccess';

# Name of HyperNews access file.  The $htaccess files will link to this.
\$hnaccess = '$hnaccess';

# These must be the same names for all groups that share the same \$hnpeople.
# So they are hardwired in to setup-lib.pl.
# The name of the list of group members.  Not used anymore.
# \$hnGroupMembers = '$hnGroupMembers';

# Name of log files - leave null if no logging should be done.
\$hnLog = '$hnLog';

#===============================
# URLs to scripts:
};



    &printNV(1, 'top', $script_names{'top.pl'});

    &printNV(1, 'login', $script_names{'login.pl'});

    print FILE "\n\$get=\"$script_names{'get'}\";\n";

    #print FILE "\n\$getURL=\"\$hnbinURL/\$get\";\n";
    # Almost same as:
    &printNV(1, 'get', $script_names{'get'});

    &printNV(1, 'thread', $script_names{'thread.pl'});
    &printNV(1, 'an_get', $script_names{'get_ann'}); 

    &printNV($authenticateInquiries ? 2 : 1, 
	     'viewMember', $script_names{'view-member.pl'});
    &printNV(1,   # was: $authenticateInquiries ? 2 : 1
	     'viewMembers', $script_names{'view-members.pl'});
    &printNV($joinSF, 'editMember', $script_names{'edit-member.pl'});
    &printNV($joinSF, 'changeMember', $script_names{'change-member.pl'});
#    &printNV(3, 'deleteMembers', $script_names{'delete-members.pl'});
    &printNV($resetPasswordSF, 'editReset', $script_names{'reset-password.pl'});
    &printNV($notifySF, 'editSubscribe', $script_names{'edit-subscribe.pl'});
    &printNV($addArticleSF, 'addArticle', $script_names{'add-article.pl'});
    &printNV($addArticleSF, 'editArticle', $script_names{'edit-article.pl'});

    &printNV($addResponseSF, 'editResponse', $script_names{'edit-response.pl'});
    &printNV($addResponseSF, 'previewResponse', $script_names{'preview-response.pl'});
    &printNV($addResponseSF, 'addResponse', $script_names{'add-response.pl'});
    &printNV($moveSF, 'adminResponses', $script_names{'admin-responses.pl'});


print FILE qq{

# HyperNews instructions.   It need not be a HyperNews page.
\$instructionsURL = "$instructionsURL";

# HyperNews test forum.  This must be a HyperNews forum to make sense.
# e.g. "$getURL/test.html"
\$hntestURL = "$hntestURL";
};

    print FILE "\n1;\n"; # So 'do' will succeed.
    close(FILE);

# Save STDERR, and then merge it with STDOUT.
#open(SAVEERR, ">&STDERR");
#open(STDERR, ">&STDOUT") || die "Cant dup stdout";

#select(STDERR); $| = 1;     # make unbuffered
#select(STDOUT);  $| = 1;     # make unbuffered

# Do we need a &flush(STDOUT) before we exit or die?
# Maybe do:
# <STRONG> Error number</STRONG>: $errno <BR>
# <STRONG> Eval Error msg</STRONG>: $@ <BR>


#===============================================
# Test the hnrc file.

    if (!-e "$hnbin/${hnrc}_tmp") {
	&HNError ("Failed to create $hnbin/${hnrc}_tmp");
    }

# Now make the tmp one real.

    if (-e "$hnbin/$hnrc") {
	rename ("$hnbin/$hnrc", "$hnbin/${hnrc}-old") ||
	    &HNError("Can't rename $hnbin/$hnrc to hnrc-old");
    }

    rename ("$hnbin/${hnrc}_tmp", "$hnbin/$hnrc") ||
	&HNError("Can't rename $hnbin/${hnrc}_tmp to hnrc");
    chmod ("$hnfilemode", "$hnbin/$hnrc");

    print &TextorHTML("<STRONG>OK</STRONG>"); 
}


sub printNV {
    local($flag, $scriptNameVar, $scriptName) = @_;
    my $path;
    my $line;

    if ($flag == 1) {
	$path = "\$hnbinURL/$scriptName";
    } elsif ($flag == 2) {  # if 3 is needed, extra check is done.
	$path = "\$hnSECUREDURL/$scriptName";
    } elsif ($flag == 3) {
	$path = "\$hnAdminURL/$scriptName";
    }
    $line = "\$${scriptNameVar}URL = \"$path\";\n";
    $line .= "\$$securedir${scriptNameVar}URL = \"\$hnSECUREDURL/$scriptName\";\n";
    $line .= "\$$admindir${scriptNameVar}URL = \"\$hnAdminURL/$scriptName\";\n";

    print FILE $line;
}

    

sub createAccessFileLink {
    local($accessfilename, $groupaccess, $groupmembers, $relativeDir) = @_;

    # $groupaccess is now just $hnaccess.

    &symlinkIt($groupaccess, $accessfilename);

    if ($serverType =~ 'CERN') {
	print &TextorHTML ("
<H3>Add this to your server configuration - fix as needed</H3>

<pre>
# Access file: \"$accessfilename\"
# Note: remove: 	GroupFile     $groupmembers
Protect $relativeDir/* {
        UserID <uid>
        GroupId <gid>
        AuthType      Basic
	ServerId      $hnPeople
        PasswordFile  $hnPassword
}
</pre>
");
    }

}






sub unescapeHTML {
   local($string) = @_;
   # This is used to undo the escaping of HTML content by escapeHTML2.
 
   $string =~ s/^\s*//;  # delete leading space
   $string =~ s/\&lt;/</g ;
   $string =~ s/\&gt;/>/g ;
   $string =~ s/\&quot;/\"/g ;
   $string =~ s/\&amp;/\&/g ;
   $string =~ s/\015\012/\012/g;	# Convert PC's CR/LF into just LF.
   $string =~ s/\015/\012/g;	# Convert Mac's CR into LF.
   return $string;
}


sub quotify {
  # Make string safe for single quotes
  my ($string) = @_;
  $string =~ s,',\\',g;
  return $string;
}


sub doexec {
    local(@exarray) = @_;
    &HNMsg ("In doexec @exarray\n") if $debug;
    local($procid) = fork();
    if ($procid == 0) {
      exec(@exarray);
    } else {
      waitpid ($procid,0);
    }
    return 1;
}


sub create_dir {
    local($dir) = @_;
    # Create $dir, relative or absolute path.
    # The directory may already exist, or any part of it may be missing.
    # All remaining parts are created, using $hndirmode.

    # Need to loop rather than use mkdir -p because the latter doesnt
    # work on all OSes.  Perl should have a similar option.

    # local($debug) = 1;

    &HNMsg ("create_dir: '$dir'") if $debug;

    my $directory;
    while ($dir =~ s,^(/?[^/]+),,) {
	if ($directory) {
	    $directory = "$directory$1";
	} else {
	    $directory = $1;
	}
        # &HNMsg ("directory is $directory") if $debug;
	if (! -d $directory) {
	    mkdir($directory, "$hndirmode") || 
		&HNError ("Can't create directory: $directory");
            chmod("$hndirmode", $directory);
            &TestPermissions($hndirmode,$directory);
	}
    }
    if (! -d $directory) {
	&HNError ("Unable to create directory: $dir");
    }
}


sub TestPermissions {
    local($permission, $file) = @_;
    # Test that the file has the correct permissions
    # The process expects permissios in hex form, the next line converts it
    # to text. Read the Permissions for the file
    local($dumb, $dumb, $PermissionResult) = stat($file);
    local($textResult) = sprintf("%0.4o", $PermissionResult);
    $textResult =~ s/\d\d\d$//;
    $textResult = "0$&";

    #Convert the Hex permissions to text
    local($permissionText) =  sprintf("%0.4o", $permission);

    #Error if the Permissions are incorrect
    if (!($permissionText =~ /$textResult/)) {
	&HNError ("The file $file has incorrect permissions<p>
        File permission is $textResult.<br>
        Expected permission is $permissionText.<br>");
    }
    return(1);
}

#sub Testownership {
#    local($user_name,$file)=@_;
#    local($fileuser_name);
#    ($dumb,$dumb,$dumb,$dumb,$fileuser_name) = stat($file);
#    $fileuser_name = getpwuid ($fileuser_name);
#    if ($fileuser_name ne $user_name) {
#	&HNError ("Incorrect owner ($fileuser_name) for $file.");
#    }
#    return("OK"); 
#}


sub scriptsplorcgi {
    my @scripts = 
	('get', 'urc', 'thread.pl', 'top.pl', 'login.pl',
	 # 'get_ann', 'ann_cgi',
         'edit-article.pl', 'add-article.pl', 
	 'edit-response.pl', 'preview-response.pl', 'add-response.pl',
	 'admin-responses.pl',
	 'edit-subscribe.pl', 
	 'view-member.pl', 'view-members.pl',
	 'edit-member.pl', 'change-member.pl', 'reset-password.pl',
	 'delete-members.pl',
	 'setup.pl', 'setup-form.pl', 'setup-short-form.pl'
	 );
    my $script;
    foreach $script (@scripts) {
      my $pl_file = $script;
      my $cgi_file = $pl_file;
      $cgi_file =~ s{\.pl$ }{}x;
      $cgi_file .= '.cgi';
      if ($plorcgi eq 'cgi') {
	$script_names{$script} = $cgi_file;
	$other_script_names{$script} = $pl_file;
      }
      else {
	$script_names{$script} = $pl_file;
	$other_script_names{$script} = $cgi_file;
      }
    }
}



sub unlinkIt {
    local($filename) = @_;
    # Unlink the $filename and return status.
    # The file linked to might not exist, and so -e would fail.
    # The file might not be a link, so -l would fail.
    # Raise error if the file should be unlinkable but isnt.
    # local($debug) = 1;
    if (-l $filename || -e $filename) {
	unlink ($filename) || &HNError("Unable to unlink $filename");
	# &HNMsg("unlinked $filename") if $debug;
	return 1;
    }
    return 0;
}

sub symlinkIt {
    my ($oldfile, $newfile) = @_;
    # Link the newfile to the oldfile, removing current newfile if it exists.

    &unlinkIt($newfile);
    #$pwd = `pwd`;
    #&HNMsg("cwd: $pwd") if $debug;
    my ($package, $filename, $linenumber) = caller;
    symlink ($oldfile, $newfile)
	|| &HNError("Unable to create symlink($oldfile, $newfile) <BR>
symlinkIt called from $filename, $linenumber");
    return 1;
}

sub hardlinkIt {
    my ($oldfile, $newfile) = @_;
    # Link the newfile to the oldfile, removing current newfile if it exists.

    &unlinkIt($newfile);
    #$pwd = `pwd`;
    #&HNMsg("cwd: $pwd") if $debug;
    my ($package, $filename, $linenumber) = caller;
    link ($oldfile, $newfile)
	|| &HNError("Unable to create hard link($oldfile, $newfile) <BR>
linkIt called from $filename, $linenumber");
    return 1;
}


sub linkOrCopyIt {
    local($oldfile, $newfile) = @_;
    # Either link or copy $oldfile to $newfile
    if ($copy_or_link eq 'copy') {
	&doexec ('cp', $oldfile, $newfile);
    }
    elsif ($copy_or_link eq 'hardlink') {
	&hardlinkIt($oldfile, $newfile);
     }
    else {
	&symlinkIt($oldfile, $newfile);
    }
}

sub linkScript {
    my ($flag, @scripts) = @_;
    # Link (or copy) the @scripts into the appropriate directories
    # The $flag indicates the access control level: 0=none, 1=non-members, ...
    my $script;

    foreach $script (@scripts) {
	my $filename = $script_names{$script};
	my $other_filename = $other_script_names{$script};
	next if (!$filename);

	&unlinkIt("$hnbin/$other_filename");
	&unlinkIt("$hnbin/$filename");

	&unlinkIt("$hnbin/$securedir/$other_filename");
	&unlinkIt("$hnbin/$securedir/$filename");

	&unlinkIt("$hnbin/$admindir/$other_filename");
	&unlinkIt("$hnbin/$admindir/$filename");

	&linkOrCopyIt("$scriptsdir/$script", "$hnbin/$filename")
	    if ($flag && $flag <= 1);

	# 3 is OK here because an extra check for admin is done when needed.
	&linkOrCopyIt("$scriptsdir/$script", "$hnbin/$securedir/$filename")
	    if ($flag && $flag <= 3);
	
	# Link from admindir will be used to provide admin functions.
	&linkOrCopyIt("$scriptsdir/$script", "$hnbin/$admindir/$filename")
	    if ($flag <= 3);  # even if $flag == 0
    }
}



sub testReadWrite {
    # Test whether we can read and write the file or directory.
    # The file may or may not exist, or it may be a link to a 
    # file that may or may not exist.
    local($file, $type) = @_;
    # Oops, forgot about checking $type.

    # Climb up until we find something that exists.
    while ($file && !(-e $file || -l $file)) {
	# Delete last component
	$file =~ s,/[^/]*,,;
    }
    if ($file && (-e $file || -l $file)) {
	-r $file || &HNError("Can't read from $file");
	-w $file || &HNError("Can't write to $file");
    }
    return $file;
}
