#!/usr/local/bin/perl
# urc.pl - URC parsing
# Copyright Board of Trustees of the University of Illinois at UC, April 1995

sub parseHyperNewsURC {
# DONT USE THIS HORRIBLE THING ANYMORE - dan.

  my (%in) = @_;

# Assign to the following global variables (and more):
# Title = any encoded text, but not too long
# articleURL = the URL for the article header file.
# Body = the URL of the body of the article, which may be anywhere.
# UpTitle = the encoded text of the parent article or response.
# UpURL = the URL of that article or response.
# Responses = the relative path to the responses without .html suffix
#    e.g. /hypernews/www  for www/responses.html
# address, userName, userURL of the owner of the article
# date = the date the article was created.

    $articleURL = $in{'URL'}; # wrong name.
    $annotationType = $in{'AnnotationType'};
    $contentType = $in{'ContentType'};
    $title = $in{'Title'};
    $keywords = $in{'Keywords'};
    $referenceURL = $in{'ReferenceURL'}; # only used if its a reference.
    $bodyURL = $in{'Body'};
    $upURL = $in{'UpURL'};
    $upRel = $in{'UpRel'};
    $base = $in{'BaseURL'};
    $responses = $in{'Responses'}; # wrong name, redundant.

    $userid = $in{'From'};
    $userName = $in{'Name'};
    $userURL = $in{'UserURL'};
    $date = $in{'Date'};
    $deleted = $in {'Deleted'};
    $moved = $in{'Moved'};
}

sub printHypernewsURC {
  # Don't use this.  Use &printAssoc.
}


sub printURCFile {
    local($urcFile) = @_;
    # Print $urcFile contents to standard output.
    local($urcline);

    open (URCH, "<$urcFile");
    while (<URCH>) {
	print $_ . "<BR>";
    }
    close URCH;
}

sub HNMsgURC {
    local(%urc) = @_;
    &printAssoc("URC", %urc);
}


# This assumes URCs are formatted similar to mail headers.
# This code could be changed to use real rfc822 parsing, but why bother?

# use IO::File;  # requires perl 5.004 or so.

sub parseURCFile {
    my ($urcFile) = @_;
    # Returns values in associative array %in.
    # e.g. %urc = &parseURCFile("filename");
    return () if !-e $urcFile;

    my %urc = ();
    local $debug = 0;

    &HNMsg("About to parse '$urcFile'") if $debug;
    # print ("About to parse '$urcFile'<BR>\n") if $debug;

#    my $urch = new IO::File;
    open (URCH, "<$urcFile") || # Do not die if this fails.
	return ();  #  Just return empty list.
      # &HNError("Couldnt open '$urcFile'");  # useful for debugging.
    my $line;
    while ($line = <URCH>) {
	if ($line =~ m{^([^:\s]+):?\s*(.*)$}) { 
	    my $name = $1;
	    my $string = $2 || '';
	    print "'$name': '$string'<BR>\n" if $debug;
	    # Unencode chars in string.
	    $string =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack('c',hex($1))/eg;
	    $urc{$name} = $string; 
	}
	else { 
	  # &HNError ("What's this: '$line'"); 
	  # Return null if not every line looks like name: value.
	  return (); }
    }
    close URCH;
    return %urc;
}

sub parseURC {
    my ($urcline) = @_;
    # Returns values in associative array %in.
    my @urc = split(/\n/, $urcline);
    my %in = ();
    my $line;
    foreach $line (@urc) {
	if ($line =~ m,^([^:]+): *(.*),) 
	{ 
	    my $name = $1;
	    my $string = $2;
	    $string =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack('c',hex($1))/eg;
	    $in{$name} = $string; 
	}
	# Return null if not every line looks like name: value.
	else { return (); }
    }
    return %in;
}

sub writeURC {
  my ($filename, %nvpairs) = @_;
  # Write the URC file to $filename using the name-value pairs.  e.g.
  # &writeURC("$urcFileName",
  #		'Title' => $title,
  #		'URL' => $URL)
  # &HNMsg("Opening $filename-new for writing URC file\n"); 

  # If changing existing data, callers should first lock before
  # reading, then change the data, write it out, and then unlock.

  require 'lock.pl';
  &lock($filename);
  &_writeURC($filename, %nvpairs);

  # This won't get called if the above fails.  Need unwind-protect
  &unlock($filename);
}

sub _writeURC {
  my ($filename, %nvpairs) = @_;
  # Non-locking version of writeURC.

  # tracedump() if $debug;
  &HNMsg("Filename: $filename") if $debug;

  open(URC, ">$filename-new");
  my($name);
  foreach $name (keys (%nvpairs)) {
      my $value = $nvpairs{$name};
      $value =~ s/%/%25/g;  # Encode % first.
      $value =~ s/\n/%0a/g; # Encode newlines.

      print URC "$name: $value\n" || 
	  &HNError("Problem writing to '$filename-new'");
  }
  close URC;
  chmod "$hnfilemode", "$filename-new";

  &moveItSafely($filename, "$filename-old", "$filename-new");
  unlink("$filename-old");
}


sub moveItSafely {
    local($filename, $oldname, $newname) = @_;
    # Move $filename safely.
    if (-e $oldname) {
      # First get rid of the old old version.  OK if it doesn't exist.
      unlink($oldname);
    }
    if (-e $filename) {
      # Move current to old.  Ok if it doesn't exist.
      rename($filename, $oldname); 
    }
    if (-e $newname) {
      # Move new to current.
      rename($newname, $filename) ||
	&HNError("Problem with rename('$newname', '$filename')");
    }
    chmod("$hnfilemode", $filename);
    # Remove the old file.
    unlink($oldname);
}


sub copy_file {
    my ($src, $target, $delete) = @_;
    # Copy the $src file to the $target file.
    # If $delete, then delete the source after copying.
    # Thanks to Wojciech Komornicki <sys@piper.hamline.edu>
	 
    if (open(SRC, $src)) {
	if (open(TARGET, "> $target")) {
	    while (<SRC>) {
		print TARGET;
	    }
	    close(TARGET);
	    close(SRC);
	    unlink($src) if ($delete);
	    1;
	} else {
	    close(SRC);
	    0;
	}
    } else {
	0;
    }
}
    

sub file_List {
    local($dir) = @_;
    # return a sorted array of responses in the directory named by $dir
    local(@files);
    local($tmp) = "$dir";

    &HNMsg("dir, $dir");
    opendir (DIR, "$tmp");
    @files = readdir DIR;
    closedir DIR;
    @files = grep (/\,urc$/, @files);
    @files = sort {$a <=> $b} @files;
    return @files;
}
1;
