#!/usr/local/bin/perl
#
# url.pl        --- recognize, parse and retrieve URLs
#
# This package contains:
#
# url'get:      parse an URL and perform an http get
# url'do_ftp:   perform an ftp request and return the result
#
# Oscar Nierstrasz 26/8/93 oscar@cui.unige.ch
#
# 25/3/94 -- moved subroutines into packages `http' and `html'
# 5/4/94 -- commented out ftp stuff (problems at various sites)
#
# For a more complete treatment of all kinds of URLs,
# see `Jack Lund's urlget in:
# http://cui_www.unige.ch/ftp/PUBLIC/oscar/scripts/README.html
# Or: ftp://cui.unige.ch:/PUBLIC/oscar/scripts/

package url;

# Add current directory to load path.
if ($0 =~ m,^(.*)/[^/]+$,) { unshift(@INC, $1, "$1/.scripts"); }

# Gene Spafford's ftp package (and using the chat package).
# Added ftp'grab -- a variant of ftp'get that returns its result
# rather than writing to a local file.
# Get this from a perl archive or from: cui.unige.ch:/PUBLIC/oscar/scripts/
# require "ftplib.pl";

require "http.pl";
require "html.pl";

# Should be replaced ...
# require "gopher.pl";

# parse an URL, issue the request and return the result
# Relative URLs are supported if defaults are provided for
# type, host, port, and path.
sub get {
        local($url, $type, $host, $port, $path) = @_;
        ($type,$host,$port,$path,$request) =
                &html'parse($type,$host,$port,$path,$url); #';
	# print STDERR "parsed URL: $type, $host, $path\n";
        if ($host) {
                if ($type eq "http") {
		    &http'get($host,$port,$request); #';
		}
                # elsif ($type eq "gopher") { 
		#   &gopher'get($host,$port,$request); }
                # elsif ($type eq "ftp") { 
		#   &do_ftp($host,$request); }
                else { 
		    print STDERR "url'get: $type requests unimplemented\n" 
			if $debug; 
		}
        }
        else {
                undef;
        }
}

# default ports
sub defport {
        local($type) = @_;
        if ($type eq "http") { 80; }
        elsif ($type eq "gopher") { 70; }
        else { undef; }
}

# ftp'grab is a version of ftp'get that returns the page
# retrieved rather than writing it to a local file.
# Perhaps not so nice for big files, but what the heck.
# sub do_ftp {
#       local($host,$file) = @_;
#       &ftp'open($host, "ftp", "$user@$thishost") || &fail;
#       &ftp'type("i") || &fail;
#       $page = &ftp'grab($file) || &fail;
#       &ftp'close;
#       $page;
# }
# 
# sub fail {
#       $save = &ftp'error;
#       &ftp'close;
#       die $save;
# }

1;

