#!/usr/bin/perl -T
# view-members.pl - view a list of all members in the current group.
# Copyright HyperNews.org, 1998

# This is used via a URL like:
#  http://www.hypernews.org/HyperNews/view-members.pl?<start>
# where <start> is the string to start with.

# Add current directory to load path.
$0 = $ENV{SCRIPT_FILENAME} if $0 =~ m{^/dev};
$0 =~ m{^(.*)/[^/]+$ }x;   $hnbin = $1 || '.';
unshift(@INC, $hnbin, "$hnbin/.scripts"); 

#print "INC: @INC\n";

require "Admin/hnrc";

###############################################

# Some options - should be in hnrc.
# No matter what the settings, admin will see everything, if authenticated.

# Maximum number of members to list in a single query.
$Maximum_Members = 100;

# AlwaysPageOutput == 1 if multiple pages of output should be allowed always.
$AlwaysPageOutput = 0;

# Number of members to list on each page.
$NumPerPage = 20;
# Number of pages to group together.  e.g. 1-10 for pages 1 to 10.
$PagesPerGroup = 10;

# AlwaysShowAdmin == 1 if admin should be shown to everyone.
$AlwaysShowAdmin = 1;

# Whether invitees are hidden, until they accept the invitation that is.
$HideInvited = 0;

# Set in hnrc.
# $hideEmailAddresses = 0;

# Whether deletion of members is allowed.  Not working yet...
$AllowMemberDelete = 0;

###############################################

&feval('&ViewMembers()');

sub ViewMembers {
 
#    &HNMsg("hndocs: $hndocs");
#    &HNMsg("SCRIPT_NAME: $ENV{SCRIPT_NAME}");
#    &HNMsg("hnprefix: $hnprefix");
#    &HNMsg("hnbinURL: $hnbinURL");

    local $debug = 0;

    require "urc.pl";
    require "member-lib.pl";
    require 'formlib.pl';

    &checkAuthenticated();
    my $isAdmin = $authenticated && &isAdmin($remoteUser);

    # Whether to hide the administrators from appearing in this list.
    $showAdmin = $AlwaysShowAdmin || $isAdmin;   # Only admin see admin here.

    $pageOutput = $AlwaysPageOutput || ($joinSF > 1) || $isAdmin; 

    my $whoami = &sustring($remoteUser);
    if ($whoami) { $whoami = "You are: $whoami <BR>"; }

    my $adminEmail = &addressOf(@hnAdmin[0]) || $hnAdmin[0];

    my %query = &parseQueryString();

    my $numPages;
    my $pageNumber = $query{page} || 1;
    local $pageQS = "?page=$pageNumber";  #  if $query{page} - always include.

    my $adminOption = $isAdmin && $query{admin};
    # &HNMsg("admin: $adminOption") if $debug;
    local $adminQS = '';
    $adminQS = "&admin" if $adminOption;
 
    my $userIDorEmail = $UserIDorHandle if !$hideUserIDs;
    if (!$hideEmailAddresses) {    #  || $isAdmin
	$userIDorEmail .= ' or ' if $userIDorEmail;
	$userIDorEmail .= 'Email Address';
    }

    $gifsOK = $ENV{'HTTP_ACCEPT'} =~ m,(image/gif)|(\*/\*),;
    $nogifsOption = $query{nogifs};
    $qsofnogifs = 'nogifs' if $nogifsOption;

    my $goal = uc($query{find});
    local $queryQS = "&find=$goal" if $goal;
    if ($goal !~ m{^\^}) {
	$goal = quotemeta($goal);
    }

    # Get the members list
    opendir (DIR, "$hnPeople$hnGroup");
    my @members = readdir DIR;
    closedir DIR;

    # Select the members that match the goal.
    if ($goal) {
	@members = eval { grep (/$goal/i, @members); };  # ignore errors
    }

    # Remove files not for members.
    my @newMembers = ();
    foreach $member (@members) {
	if (&useridOK($member) &&
	    $member !~ m,-old$, &&
	    $member !~ m,-new$, &&
	    $member !~ m,^\., &&
#	    $member !~ m{\@} &&  email is OK
	    $member !~ m,^urc$,) {
	    push @newMembers, $member;
	}
    }
    @members = @newMembers;

    my $numMembers = 0 + @members;

    # Sort the members by userid/email.  
    # Could sort by name or other, but that requires reading up member info.
    if ($hideEmailAddresses && !$isAdmin) {
	# Filter out the email addresses
	@members = grep (/^[^\@]+$/, @members);
    }
    if ($hideUserIDs && !$isAdmin) {
	# Filter out the userids
	@members = grep (/[\@]/, @members);
    }
    @members = sort {lc($a) cmp lc($b)} @members;
    
    &print_HTML_HEAD("$hnSystemName Members");

    # Build index for each letter.
    my $alphaIndex = 
	join ("\n",
	      map { qq{<A HREF="$viewMembersURL?find=^$_$adminQS">$_</A>} }
	      ('A'..'Z', '0'..'9'));

    my $displayGoal = uc($goal);
    $displayGoal =~ s{\\}{}; # Unquote things.
    if ($goal) {
	if ($displayGoal =~ m{^\^}) {
	    $displayGoal =~ s{^\^}{};
	    $displayGoal = "begins with `<font size+1>$displayGoal</font>'";
	}
	else {
	    $displayGoal = "contains `<font size+1>$displayGoal</font>'";
	}
	$displayGoal = qq{
<H3>Members whose $userIDorEmail <B>$displayGoal</B></H3>
};
	$thatMatch = ' that match your query';
    }
    else {
	$displayGoal = "<H3>All Members</H3>";
    }

    my $click_on_email = qq{
To send a person email, click on their email address 
 in the right column.
} if !$hideEmailAddresses;

    print qq{
<H2>
$hnSystemName <A HREF="$viewMembersURL">Members</A>
</H2>
    
 This page displays information about $hnSystemName members.
 To find out more about another member, click on a name below 
 (the left column).  $click_on_email  <P>
 };


    print qq{

 <FORM ACTION="$viewMembersURL">
 <STRONG>Find members whose $userIDorEmail contain:</STRONG>
 <INPUT TYPE=text NAME=find>
 <INPUT TYPE=submit VALUE="Search">
 <INPUT TYPE=hidden NAME=admin VALUE="$adminOption">
 </FORM>


$alphaIndex
<HR>

$displayGoal
};

    if ($AllowMemberDelete && $adminOption) {
	# Doesnt work yet.
	print qq{
<FORM ACTION="$AdmindeleteMembersURL" 
 METHOD="POST"> 
 };
    }

    ###############################################
    # Compute the number of pages.
    {
        use integer;
        # Compute number of pages, rounding up.
	$numPages = (@members - 1 + $NumPerPage) / $NumPerPage;
	# &HNMsg("numPages: " . $numPages);
	# &HNMsg("calc: " . (($pageNumber + $NumPerPage) % $numPages));
	# $pageNumber = 1 + (($pageNumber + $NumPerPage) % $numPages);
    }

    # Compute index into @members.
    my $startIndex = ($pageNumber - 1) * $NumPerPage;
    my $endIndex = &min($#members, $startIndex + $NumPerPage - 1);
    &HNMsg("startIndex: $startIndex  
 endIndex: $endIndex  
 #members: $#members") if $debug;

#    if ($numPages == 1) {
#        @membersOnPage = @members;
#    }
#    elsif ($numPages > 1) {
#        @membersOnPage = @members[$startIndex .. $endIndex];
#    } 

    # $max = min ((@members - $i - 1), $NumPerPage);
    my %membersSeen = ();

    &HNMsg("Members: @members") if $debug;

    my $i = $startIndex;
    my $done = 0;
    my @results = ();
    my $numDisplayed = 0;

    ###################################################
    # Loop to generate text for members.

    # if ($numPages == 1 || $pageOutput)  - heck, loop anyway.

    if ($numMembers > $Maximum_Members) {
	# We must have over filled the page.
	print qq{
<B>There are possibly $numMembers members$thatMatch, which
is too many to show.</B>  <BR>
};
	print ($queryQS 
	       ? qq{ Please change your search string } 
	       : qq{ Try clicking on a letter or enter a search string });
	print "to find fewer members.";
    }
    else {

    while (!$done) {
        my $id = $members[$i++];
	my %member = &getMember($id);
	$done = $i >= $numMembers;

	my $userid = $member{UserID};
	my $email = $member{Email};
	&HNMsg("ID: $id  userid: $userid") if $debug;

	next if !$userid || $membersSeen{$userid};
	next if !$showAdmin && &isAdmin($userid);

	# Don't show any merely 'invited' members.
	next if $HideInvited && $member{Status} eq 'invited'; 


        my $hideEmailAddress = ($member{Hide} eq 'Email');

	# If there is only an email, and it should be hidden,
	# we still want to show the name. And admin see everything.
	#   next if $hideEmailAddress && ($userid eq $email);

	# Same with userids.
	# next if $hideUserIDs && ($id ne $email);

	# Remember this userid so we dont show it again.
        $membersSeen{$userid} = 1;

	$numDisplayed++;
	if ($numDisplayed > $NumPerPage) {
	    $done = 1;
	    next;
	}
	&HNMsg("Prepare to show") if $debug;

        my $deleteOption = '';
        if ($AllowMemberDelete && $adminOption) {
            $deleteOption = qq{<INPUT TYPE=checkbox NAME="$userid">\n};
        }
 
	my $nameText;
	my $userURL = 
	    ((!$hideUserIDs || $isAdmin) ? "$viewMemberURL?$userid" : 
	     (($email && !$hideEmailAddress) ? "$viewMemberURL?$email" : ''));
	# was $member{'UserURL'} || $userURL ||
	$nameText = ($userURL 
		     ? qq{<A HREF="$userURL">$member{Name}</A>} 
		     : $member{Name});

	##############################
	# Start pushing results
	push @results, "<TR>\n";

	push @results, qq{
<TD valign=top>$deleteOption $nameText &nbsp;</TD>
};

	if (!$hideEmailAddresses || $isAdmin) {
	    my $emailCell = '';
	    if ($isAdmin || ($email && !$hideEmailAddress)) {
		$emailCell .= qq{<A HREF="mailto:$email">$email</A>};
	    }
	    if ($email && $hideEmailAddress) {
		$emailCell .= ' (hidden)';
	    }
	    push @results, qq{<TD valign=top>$emailCell &nbsp;</TD>\n};
	}

	if (!$hideUserIDs || $isAdmin) {
	    my $useridCell = '';
	    if ($isAdmin) {
		$useridCell .= qq{
 (<A HREF="$SECUREDeditMemberURL?userid=$userid">edit</A>)
};
	    }
	    $useridCell .= qq{
 <A HREF="$viewMemberURL?$userid"><B>$userid</B></A>
};
	    push @results, qq{<TD valign=top>$useridCell &nbsp;</TD>\n};
	}

	push @results, '</TR>';
    
    } # while

    # &HNMsg("numDisplayed: $numDisplayed");


    if ($numDisplayed == 0) {
	print qq{
<B>There are no members$thatMatch.</B> <BR>
};
	if ($queryQS) {
	    print qq{  Maybe your query is too specific.};
	}
    }

    #=========================================================
    # Print out the table of members.
    else {
#	print qq{
#<B>There are $numDisplayed members$thatMatch.</B> <P>
#};
	if ($startIndex > 0 || $i < $#members) {
	    &selectPages($pageNumber, $numPages); 
	}

	print qq{

<TABLE BORDER=1>
<TR>
<TD valign=top><B>Name</B></TD>
};
	print qq{
<TD valign=top><B>Email</B> <!--<br>(click to write email)--></TD>
} if !$hideEmailAddresses || $isAdmin;

	print qq{
<TD valign=top><B>$UserIDorHandle</B> <!--<br>(click for more info)--></TD>
} if !$hideUserIDs || $isAdmin;

	print qq{
</TR>
};
	# Now print all the entries.
	print join ("\n", @results);

	print qq{
</TABLE>
<P>
};
	if ($startIndex > 0 || $i < $#members) {
	    &selectPages($pageNumber, $numPages); 
	}
    }

    if ($AllowMemberDelete && $adminOption) {
    print qq{
<P> <INPUT TYPE="submit" VALUE="Delete Selected Members">
</FORM> 
};
}

  }

#    print qq{
#<HR>
#$alphaIndex
#
#};

    ####################################
    # Form for entering a string to find.  Same as at top.

#    print qq{
# <FORM ACTION="$viewMembersURL">
# <STRONG>Find members whose  or $userIDorEmail contain:</STRONG>
# <INPUT TYPE=text NAME=find>
# <INPUT TYPE=submit VALUE="Search">
# <INPUT TYPE=hidden NAME=admin VALUE="$adminOption">
# </FORM>
# };


    #########################################
    # Smallprint.

    my $member_confusion = qq{
Each HyperNews site has its own set of members,
 so being a member here does not mean you are a member at other sites.

This site is located at <A HREF="$topURL">$topURL</A>
} if 0;  # Should be displayed for hypernews.org

    print qq{
<P>
 Note that this site uses the <A
 HREF="http://www.hypernews.org/"><B>HyperNews</B></A> system (version
 $hnversion).  
 $member_confusion
 This site is called "<B>$hnSystemName</B>".  The administrator is <a
 HREF="mailto:$adminEmail"><B>$adminEmail</B></A>.
 <P>

$whoami

 };

    ########################################3
    # Command links
    
    {
	print "\n<P><TABLE><TR>\n";

	if ($joinSF == 1 && !$authenticated) {
	    print qq{
<TD>
<FONT SIZE=+1><B>
[<A HREF="$editMemberURL?new">Register (Become a Member)</A>]
</B></FONT>
</TD>
 };
    }

	print qq{
<TD>
<FONT SIZE=+1><B>
[<A HREF="$SECUREDeditMemberURL">Edit YOUR Member Info</A>]
</B></FONT>
</TD>
 };

	print "\n</TR></TABLE><P>\n";

	if (0) {
 	    # Problems: doesnt use current settings.
	    # Forgets admin mode.

	    if ($adminOption) {
		&commandLink("$SECUREDviewMembersURL", '',
			     "Turn off Admin mode",
			     "noadmin.gif",
			     $commandIconXsize, $commandIconYsize,
			     "No Admin Mode");
	    }
	    else {
		&commandLink("$SECUREDviewMembersURL?admin", '',
			     "Turn on Admin mode",
			     "admin.gif",
			     $commandIconXsize, $commandIconYsize,
			     "Admin Mode");
	    }
	    print "\n";
	}

	if ($instructionsURL) { #  && !$oFrame && !$rFrame
	    &commandLink("$instructionsURL", $targetBlank,
			 "Help for $hnSystemName $hnversion",
			 "help.gif", 
			 $commandIconXsize, $commandIconYsize,
			 "Help for $hnSystemName $hnversion");
	}
	
	print "\n<P>\n";
    }

    &printEnd();
}


sub min {
    my ($a, $b) = @_;
    $a < $b ? $a : $b;
}


sub commandLink {
  # Print a command link.
  my ($commandURL, $target, $msg, $gif, $width, $height, $alt, $align) = @_;
  my $alignment = ($align ?  'ALIGN="absmiddle"' : '');
  my $anchor = "<A HREF=\"$commandURL\" $target
 onMouseOver=\"return window.help('$msg');\"
 onMouseOut=\"return window.help('');\">";
  if ($gifsOK) {
    # Netscape 2.0 needs the width and height with the javascript.
    print "
$anchor
 <IMG SRC=\"$hniconsURL/$gif\" BORDER=0  
 WIDTH=$width HEIGHT=$height
 ALT=\"$alt\"
 $alignment></A>\n";
      }
  else {
    # The $TD$B is not always right.
    print "$TD$B
$anchor
 $alt</A>$endB$endTD \n";
  }
}


sub selectPages {
    my ($pageNumber, $numPages) = @_;
    &HNMsg("page number: $pageNumber  number of pages: $numPages") if $debug;
    return if ($numPages <= 1);

# <CENTER>
    print qq{
	<U>Page <B>$pageNumber</B> of $numPages</U>
	    &nbsp; &nbsp;
    };
    print qq{
	&nbsp; &nbsp; Select Page:
    };

    if ($pageNumber > 1) {
	my $previousNum = $pageNumber - 1;
	print qq{
	    &nbsp;  <A HREF="$viewMembersURL?page=$previousNum$queryQS$adminQS">Previous Page
		</A> \n};
    }

    my $pageNum = 1;
    while ($pageNum <= $numPages) {

	my $pageNumEnd = &min($numPages, $pageNum+$PagesPerGroup-1);
	&HNMsg("pageNum: $pageNum  end: $pageNumEnd") if $debug;

	if ($pageNum <= $pageNumber && $pageNumber <= $pageNumEnd) {
	    # Expand this group

	    while ($pageNum <= $pageNumEnd) {

		if ($pageNum == $pageNumber) {
		    # This is the current page.
		    print "
 &nbsp; <B>$pageNum</B>
";
		}
		else {
		    print qq{
			&nbsp; <A HREF="$viewMembersURL?page=$pageNum$queryQS$adminQS">$pageNum</A> \n};
		}
		$pageNum++;
	    }
	}

	else {
	    # Abbreviate group
	    if ($pageNum == $pageNumEnd) {
		print qq{
		    &nbsp; <A HREF="$viewMembersURL?page=$pageNum$queryQS$adminQS">$pageNum</A> \n};
	    }
	    else {
		print qq{
		    &nbsp; <A HREF="$viewMembersURL?page=$pageNum$queryQS$adminQS">$pageNum-$pageNumEnd</A> \n};
	    }
	    $pageNum += $PagesPerGroup;
	}
    }

    if ($pageNumber < $numPages) {
	my $nextNum = $pageNumber + 1;
	print qq{
	    &nbsp; <A HREF="$viewMembersURL?page=$nextNum$queryQS$adminQS">Next Page</A> \n};
    }

    print qq{
<P>
}
# </CENTER>

}
 
